# -*- coding: utf-8 -*-
#
# This file is part of Harvesting Kit.
# Copyright (C) 2015 CERN.
#
# Harvesting Kit is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Harvesting Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Basic mapping between CDS/INSPIRE."""

mappings = {
  "config": {
    "languages": [
      {
        "cds": "nno",
        "inspire": "Norwegian"
      },
      {
        "cds": "eng",
        "inspire": "English"
      },
      {
        "cds": "rom",
        "inspire": "Romany"
      },
      {
        "cds": "jpn",
        "inspire": "Japanese"
      },
      {
        "cds": "por",
        "inspire": "Portuguese"
      },
      {
        "cds": "ita",
        "inspire": "Italian"
      },
      {
        "cds": "ara",
        "inspire": "Arabic"
      },
      {
        "cds": "pol",
        "inspire": "Polish"
      },
      {
        "cds": "jap",
        "inspire": "Japanese"
      },
      {
        "cds": "tur",
        "inspire": "Turkish"
      },
      {
        "cds": "spa",
        "inspire": "Spanish"
      },
      {
        "cds": "glg",
        "inspire": "Galician"
      },
      {
        "cds": "arm",
        "inspire": "Armenian"
      },
      {
        "cds": "cze",
        "inspire": "Czech"
      },
      {
        "cds": "sue",
        "inspire": "Swedish"
      },
      {
        "cds": "est",
        "inspire": "Estonian"
      },
      {
        "cds": "che",
        "inspire": "Chechen"
      },
      {
        "cds": "fre",
        "inspire": "French"
      },
      {
        "cds": "scr",
        "inspire": "Serbo-Croatian"
      },
      {
        "cds": "chi",
        "inspire": "Chilean"
      },
      {
        "cds": "hrv",
        "inspire": "Croatian"
      },
      {
        "cds": "swe",
        "inspire": "Swedish"
      },
      {
        "cds": "ukr",
        "inspire": "Ukrainian"
      },
      {
        "cds": "ice",
        "inspire": "Icelandic"
      },
      {
        "cds": "lit",
        "inspire": "Lithuanian"
      },
      {
        "cds": "gre",
        "inspire": "Greek"
      },
      {
        "cds": "peo",
        "inspire": "Persian"
      },
      {
        "cds": "heb",
        "inspire": "Hebrew"
      },
      {
        "cds": "mul",
        "inspire": "Multiple"
      },
      {
        "cds": "kor",
        "inspire": "Korean"
      },
      {
        "cds": "fin",
        "inspire": "Finnish"
      },
      {
        "cds": "hun",
        "inspire": "Hungarian"
      },
      {
        "cds": "ser",
        "inspire": "Serbian"
      },
      {
        "cds": "ger",
        "inspire": "German"
      },
      {
        "cds": "rum",
        "inspire": "Romanian"
      },
      {
        "cds": "dan",
        "inspire": "Danish"
      },
      {
        "cds": "nob",
        "inspire": "Norwegian"
      },
      {
        "cds": "lat",
        "inspire": "Latin"
      },
      {
        "cds": "rus",
        "inspire": "Russian"
      },
      {
        "cds": "nor",
        "inspire": "Norwegian"
      },
      {
        "cds": "dut",
        "inspire": "Dutch"
      },
      {
        "cds": "slv",
        "inspire": "Slovenian"
      },
      {
        "cds": "esp",
        "inspire": "Spanish"
      },
      {
        "cds": "cat",
        "inspire": "Catalan"
      },
      {
        "cds": "srp",
        "inspire": "Serbian"
      },
      {
        "cds": "slo",
        "inspire": "Slovenian"
      },
      {
        "cds": "und",
        "inspire": "Undetermined"
      },
      {
        "cds": "tha",
        "inspire": "Thaiwanese"
      }
    ],
    "experiments": [
      {
        "cds": "CERN LHC---ATLAS",
        "inspire": "CERN-LHC-ATLAS"
      },
      {
        "cds": "CERN LHC---CMS",
        "inspire": "CERN-LHC-CMS"
      },
      {
        "cds": "CERN LHC---LHCb",
        "inspire": "CERN-LHC-LHCb"
      },
      {
        "cds": "CERN LHC---ALICE",
        "inspire": "CERN-LHC-ALICE"
      },
      {
        "cds": "CERN LHC---TOTEM",
        "inspire": "CERN-LHC-TOTEM"
      },
      {
        "cds": "CERN LHC---LHCf",
        "inspire": "CERN-LHC-LHCf"
      },
      {
        "cds": "CERN LHC---MoEDAL",
        "inspire": "CERN-LHC-MoEDAL"
      },
      {
        "cds": "CERN PS---nTOF",
        "inspire": "CERN-nTOF"
      },
      {
        "cds": "CAST",
        "inspire": "CERN-CAST"
      },
      {
        "cds": "CERN PS---nTOF",
        "inspire": "CERN-PS-nTOF"
      },
      {
        "cds": "CERN LEP---ALEPH",
        "inspire": "CERN-LEP-ALEPH"
      },
      {
        "cds": "CERN LEP---DELPHI",
        "inspire": "CERN-LEP-DELPHI"
      },
      {
        "cds": "CERN LEP---OPAL",
        "inspire": "CERN-LEP-OPAL"
      },
      {
        "cds": "CERN LEP---L3",
        "inspire": "CERN-LEP-L3"
      },
      {
        "cds": "CERN SPS---ICARUS CNGS2",
        "inspire": "ICARUS"
      },
      {
        "cds": "CERN SPS---OPERA CNGS1",
        "inspire": "OPERA"
      },
      {
        "cds": "Not applicable---RD",
        "inspire": "CERN-RD-0"
      },
      {
        "cds": "CERN LHC---LHCf",
        "inspire": "CERN-LHC-LHCf"
      },
      {
        "cds": "CERN SPS---NA",
        "inspire": "CERN-NA-0"
      },
      {
        "cds": "CERN SPS---WA",
        "inspire": "CERN-WA-0"
      },
      {
        "cds": "DESY HERA---ZEUS",
        "inspire": "DESY-HERA-ZEUS"
      }
    ],
    "categories_cds": [
      {
        "cds": "General Relativity and Cosmology",
        "inspire": "Gravitation and Cosmology"
      },
      {
        "cds": "Detectors and Experimental Techniques",
        "inspire": "Instrumentation"
      },
      {
        "cds": "Accelerators and Storage Rings",
        "inspire": "Accelerators"
      },
      {
        "cds": "Computing and Computers",
        "inspire": "Computing"
      },
      {
        "cds": "Mathematical Physics and Mathematics",
        "inspire": "Math and Math Physics"
      },
      {
        "cds": "Astrophysics and Astronomy",
        "inspire": "Astrophysics"
      },
      {
        "cds": "Physics in General",
        "inspire": "General Physics"
      },
      {
        "cds": "Other",
        "inspire": "Other"
      },
      {
        "cds": "Particle Physics - Experiment",
        "inspire": "Experiment-HEP"
      },
      {
        "cds": "Particle Physics - Phenomenology",
        "inspire": "Phenomenology-HEP"
      },
      {
        "cds": "Particle Physics - Theory",
        "inspire": "Theory-HEP"
      },
      {
        "cds": "Particle Physics - Lattice",
        "inspire": "Lattice"
      },
      {
        "cds": "Nuclear Physics - Experiment",
        "inspire": "Experiment-Nucl"
      },
      {
        "cds": "Nuclear Physics - Theory",
        "inspire": "Theory-Nucl"
      }
    ],
    "categories_inspire": [
      {
        "cds": "General Relativity and Cosmology",
        "inspire": "Gravitation and Cosmology"
      },
      {
        "cds": "General Theoretical Physics",
        "inspire": "General Physics"
      },
      {
        "cds": "Detectors and Experimental Techniques",
        "inspire": "Instrumentation"
      },
      {
        "cds": "Engineering",
        "inspire": "Instrumentation"
      },
      {
        "cds": "Accelerators and Storage Rings",
        "inspire": "Accelerators"
      },
      {
        "cds": "Computing and Computers",
        "inspire": "Computing"
      },
      {
        "cds": "Mathematical Physics and Mathematics",
        "inspire": "Math and Math Physics"
      },
      {
        "cds": "Astrophysics and Astronomy",
        "inspire": "Astrophysics"
      },
      {
        "cds": "Condensed Matter",
        "inspire": "General Physics"
      },
      {
        "cds": "General Theoretical Physics",
        "inspire": "General Physics"
      },
      {
        "cds": "Physics in General",
        "inspire": "General Physics"
      },
      {
        "cds": "Other",
        "inspire": "Other"
      },
      {
        "cds": "Chemical Physics and Chemistry",
        "inspire": "Other"
      },
      {
        "cds": "Information Transfer and Management",
        "inspire": "Other"
      },
      {
        "cds": "Commerce, Economics, Social Science",
        "inspire": "Other"
      },
      {
        "cds": "Biography, Geography, History",
        "inspire": "Other"
      },
      {
        "cds": "Science in General",
        "inspire": "Other"
      },
      {
        "cds": "Particle Physics - Experiment",
        "inspire": "Experiment-HEP"
      },
      {
        "cds": "Particle Physics - Phenomenology",
        "inspire": "Phenomenology-HEP"
      },
      {
        "cds": "Particle Physics - Theory",
        "inspire": "Theory-HEP"
      },
      {
        "cds": "Particle Physics - Lattice",
        "inspire": "Lattice"
      },
      {
        "cds": "Nuclear Physics - Experiment",
        "inspire": "Experiment-Nucl"
      },
      {
        "cds": "Nuclear Physics - Theory",
        "inspire": "Theory-Nucl"
      }
    ],
    "journals": [
      {
        "cds": "Ann. Math.",
        "inspire": "Annals Math."
      },
      {
        "cds": "Invent. Math.",
        "inspire": "Invent.Math."
      },
      {
        "cds": "Radiat. Prot. Dosim.",
        "inspire": "Radiat.Prot.Dosim."
      },
      {
        "cds": "Comput. Commun.",
        "inspire": "Comput.Commun."
      },
      {
        "cds": "Trans. Am. Math. Soc.",
        "inspire": "Trans.Am.Math.Soc."
      },
      {
        "cds": "Acta Phys. Pol. Suppl.",
        "inspire": "Acta Phys.Polon.Supp."
      },
      {
        "cds": "Acta Phys.Austriaca",
        "inspire": "Acta Phys.Austriaca"
      },
      {
        "cds": "Publ. Astron. Soc. Pac.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "Europhys. Lett.",
        "inspire": "Europhys.Lett."
      },
      {
        "cds": "Sov. Phys. - JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Proc. Camb. Philos. Soc.",
        "inspire": "Proc.Cambridge Phil.Soc."
      },
      {
        "cds": "New Astron. Rev.",
        "inspire": "New Astron.Rev."
      },
      {
        "cds": "Opt. Express",
        "inspire": "Opt.Express"
      },
      {
        "cds": "Fortschr. Phys.",
        "inspire": "Fortsch.Phys."
      },
      {
        "cds": "Nature",
        "inspire": "Nature"
      },
      {
        "cds": "IEEE Electron. Dev. Lett.",
        "inspire": "IEEE Electron. Dev. Lett."
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Comments Nucl. Part. Phys.",
        "inspire": "Comments Nucl.Part.Phys."
      },
      {
        "cds": "J. Quant. Spectrosc. Radiat. Transf",
        "inspire": "J.Quant.Spectrosc.Radiat.Trans."
      },
      {
        "cds": "Indiana Univ.Math.J.",
        "inspire": "Indiana Univ.Math.J."
      },
      {
        "cds": "Trans. Faraday Soc.",
        "inspire": "Trans.Faraday Soc."
      },
      {
        "cds": "Acta Physica Acad.Sci.Hung.",
        "inspire": "Heavy Ion Phys."
      },
      {
        "cds": "Netw. Comput. Neural Syst.",
        "inspire": "Netw.Comput.Neural Syst."
      },
      {
        "cds": "Bull. Soc. Math. Fr.",
        "inspire": "Bull.Soc.Math.Fr."
      },
      {
        "cds": "Johns Hopkins Univ.Circulars",
        "inspire": "Johns Hopkins Univ.Circulars"
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Prog. Part. Nucl. Phys.",
        "inspire": "Prog.Part.Nucl.Phys."
      },
      {
        "cds": "J. Chem. Educ.",
        "inspire": "J.Chem.Educ."
      },
      {
        "cds": "Bol. Soc. Mat. Mex.",
        "inspire": "Bol.Soc.Mat.Mexicana"
      },
      {
        "cds": "Z.Phys.",
        "inspire": "Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "C. R. Phys.",
        "inspire": "Comptes Rendus Physique"
      },
      {
        "cds": "Proc. R. Soc. Lond., A",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Comput. Math. Appl.",
        "inspire": "Comput.Math.Appl."
      },
      {
        "cds": "Acta Phys. Slovaca",
        "inspire": "Acta Phys.Slov."
      },
      {
        "cds": "Mathematika",
        "inspire": "Mathematika"
      },
      {
        "cds": "Comments Nucl.Part.Phys.",
        "inspire": "Comments Nucl.Part.Phys."
      },
      {
        "cds": "J. Geod. Soc. Jpn.",
        "inspire": "J.Geodet.Soc.Jap."
      },
      {
        "cds": "Statist.Papers",
        "inspire": "Statist.Papers"
      },
      {
        "cds": "Ann. Fis.",
        "inspire": "Anales Fis."
      },
      {
        "cds": "Acta Math. Univ. Comen.",
        "inspire": "Acta Math.Univ.Comenianae"
      },
      {
        "cds": "Aust. J. Phys.",
        "inspire": "Austral.J.Phys."
      },
      {
        "cds": "Curr. Sci.",
        "inspire": "Curr.Sci."
      },
      {
        "cds": "Lett. Math. Phys.",
        "inspire": "Lett.Math.Phys."
      },
      {
        "cds": "Ramanujan J.",
        "inspire": "Ramanujan J."
      },
      {
        "cds": "Rev.Mat.Iberoam.",
        "inspire": "Rev.Mat.Iberoam."
      },
      {
        "cds": "SLAC Beam Line",
        "inspire": "SLAC Beam Line"
      },
      {
        "cds": "Annu. Rev. Nucl. Part. Sci.",
        "inspire": "Complete"
      },
      {
        "cds": "Eos",
        "inspire": "Eos"
      },
      {
        "cds": "Biomaterials",
        "inspire": "Biomaterials"
      },
      {
        "cds": "Comput.Chem.",
        "inspire": "Comput.Chem."
      },
      {
        "cds": "Chem.Brit.",
        "inspire": "Chem.Brit."
      },
      {
        "cds": "Funkt.Anal.Pril.",
        "inspire": "Funkt.Anal.Pril."
      },
      {
        "cds": "J. Dyn. Differ. Equ.",
        "inspire": "J.Dyn.Differ.Equ."
      },
      {
        "cds": "Eur. Biophys. J.",
        "inspire": "Eur.Biophys.J."
      },
      {
        "cds": "Opt. Acta",
        "inspire": "Opt.Acta"
      },
      {
        "cds": "Collect. Math.",
        "inspire": "Collect.Math."
      },
      {
        "cds": "Camb. Monogr. Part. Phys. Nucl. Phys. Cosmon.",
        "inspire": "Camb.Monogr.Part.Phys.Nucl.Phys.Cosmol."
      },
      {
        "cds": "Sov. Phys. - JETP Lett.",
        "inspire": "JETP Lett."
      },
      {
        "cds": "Cryst.Lattice Defects",
        "inspire": "Cryst.Lattice Defects"
      },
      {
        "cds": "Proteins",
        "inspire": "Proteins Struct.Funct.Genet."
      },
      {
        "cds": "J. Phys. (Paris)",
        "inspire": "J.Phys.(France)"
      },
      {
        "cds": "Teor. Mat. Fiz.",
        "inspire": "Teor.Mat.Fiz."
      },
      {
        "cds": "Geophys. J. R. Astron. Soc.",
        "inspire": "Geophys.J.R.Astron.Soc."
      },
      {
        "cds": "Part. World",
        "inspire": "Part.World"
      },
      {
        "cds": "Astron. J.",
        "inspire": "Astron.J."
      },
      {
        "cds": "Geophys.",
        "inspire": "Geophysics"
      },
      {
        "cds": "Physik. Z.",
        "inspire": "Phys.Z."
      },
      {
        "cds": "Acta Phys. Pol.",
        "inspire": "Acta Phys.Polon."
      },
      {
        "cds": "J.Austral.Math.Soc.",
        "inspire": "J.Austral.Math.Soc."
      },
      {
        "cds": "Acta Electron.",
        "inspire": "Acta Electron."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Acta Phys. Hung.",
        "inspire": "Astropart.Phys."
      },
      {
        "cds": "Indian J. Pure Appl. Phys.",
        "inspire": "Indian J.Pure Appl.Phys."
      },
      {
        "cds": "Ergodic Theory Dyn. Syst.",
        "inspire": "Ergod.Theor.Dynam.Syst."
      },
      {
        "cds": "Discover",
        "inspire": "Discover"
      },
      {
        "cds": "Contemp. Phys.",
        "inspire": "Contemp.Phys."
      },
      {
        "cds": "Izv. Akad. Nauk Fiz.",
        "inspire": "Izv.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Funkt.Anal.Pril.",
        "inspire": "Funkt.Anal.Pril."
      },
      {
        "cds": "J. Symb. Comput.",
        "inspire": "J.Symb.Comput."
      },
      {
        "cds": "Phys. Fluids",
        "inspire": "Phys.Fluids"
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Phys. Rep.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "J. Electron Spectrosc. Relat. Phenom.",
        "inspire": "J.Electron Spectrosc.Rel.Phenom."
      },
      {
        "cds": "Phys. Part. Nucl.",
        "inspire": "Part.Nucl."
      },
      {
        "cds": "Fluid Dynamics Res.",
        "inspire": "Fluid Dynamics Res."
      },
      {
        "cds": "Bull.Israel Phys.Soc.",
        "inspire": "Bull.Israel Phys.Soc."
      },
      {
        "cds": "Springer Tracts Mod. Phys.",
        "inspire": "Springer Tracts Mod.Phys."
      },
      {
        "cds": "Yad. Fiz.",
        "inspire": "Yad.Fiz."
      },
      {
        "cds": "Funct. Anal. Appl.",
        "inspire": "Funkt.Anal.Pril."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "J. Phys. IV",
        "inspire": "J.Phys.IV"
      },
      {
        "cds": "J. Phys. II",
        "inspire": "J.Phys.II"
      },
      {
        "cds": "J. Phys.: Conf. Ser.",
        "inspire": "J.Phys.Conf.Ser."
      },
      {
        "cds": "Phys. Rep.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Prog. Opt.",
        "inspire": "Prog.Opt"
      },
      {
        "cds": "Phys. Lett. A",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Am. Math. Soc. Transl.",
        "inspire": "Am.Math.Soc.Transl."
      },
      {
        "cds": "Part. Accel.",
        "inspire": "Part.Accel."
      },
      {
        "cds": "IEEE Trans. Ultrason., Ferroelectr. Freq. Control",
        "inspire": "IEEE Trans.Ultrason.Ferroelectr.Freq.Control"
      },
      {
        "cds": "Plasma Phys.",
        "inspire": "Plasma Phys."
      },
      {
        "cds": "Ann.Soc.R.Sci.Med.Nat.Bruxelles",
        "inspire": "Ann.Soc.R.Sci.Med.Nat.Bruxelles"
      },
      {
        "cds": "Phys. Bl.",
        "inspire": "Phys.Bl."
      },
      {
        "cds": "Soryushiron-Kenkyu",
        "inspire": "Soryushiron Kenkyu"
      },
      {
        "cds": "Ukr.Phys.J.",
        "inspire": "Ukr.Phys.J."
      },
      {
        "cds": "J.Petrol.",
        "inspire": "J.Petrol."
      },
      {
        "cds": "Few Body Syst.",
        "inspire": "Few Body Syst."
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Bull. Am. Math. Soc.",
        "inspire": "Bull.Am.Math.Soc."
      },
      {
        "cds": "Bell Syst. Tech. J.",
        "inspire": "Bell Syst.Tech.J."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "HEP-LAT",
        "inspire": "HEP-LAT"
      },
      {
        "cds": "Adv. Stud. Pure Math.",
        "inspire": "Adv.Stud.Pure Math."
      },
      {
        "cds": "Astrophys. Nor.",
        "inspire": "Astrophys.Nor."
      },
      {
        "cds": "Ann. Phys. (San Diego)",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Acta Univ.Carolinae.Math.Phys.",
        "inspire": "Acta Univ.Carolinae.Math.Phys."
      },
      {
        "cds": "Physiol.Behavior",
        "inspire": "Physiol.Behavior"
      },
      {
        "cds": "Space Sci. Rev.",
        "inspire": "Space Sci.Rev."
      },
      {
        "cds": "Tectonophys.",
        "inspire": "Tectonophys."
      },
      {
        "cds": "SIAM J. Optim.",
        "inspire": "SIAM J.Optim."
      },
      {
        "cds": "Laser Part. Beams",
        "inspire": "Laser Part.Beams"
      },
      {
        "cds": "Space Sci. Rev.",
        "inspire": "Space Sci.Rev."
      },
      {
        "cds": "Ann. Probab.",
        "inspire": "Annals Probab."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Found. Phys.",
        "inspire": "Found.Phys."
      },
      {
        "cds": "Camb. Monogr. Part. Phys. Nucl. Phys. Cosmon.",
        "inspire": "Camb.Monogr.Part.Phys.Nucl.Phys.Cosmol."
      },
      {
        "cds": "Dokl. Akad. Nauk SSSR",
        "inspire": "Dokl.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Adv. Cryog. Eng.",
        "inspire": "Adv.Cryog.Eng."
      },
      {
        "cds": "Math. Z.",
        "inspire": "Math.Z."
      },
      {
        "cds": "Microelectron. Eng.",
        "inspire": "Microelectron.Eng."
      },
      {
        "cds": "Ann. Isr. Phys. Soc.",
        "inspire": "Annals Israel Phys.Soc."
      },
      {
        "cds": "Europhys. Lett.",
        "inspire": "Europhys.Lett."
      },
      {
        "cds": "Exp. Astron.",
        "inspire": "Exper.Astron."
      },
      {
        "cds": "Annals Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Mater. Sci. Forum",
        "inspire": "Mater.Sci.Forum"
      },
      {
        "cds": "Annu. Rev. Astron. Astrophys.",
        "inspire": "Ann.Rev.Astron.Astrophys."
      },
      {
        "cds": "Earth Planet. Sci. Lett.",
        "inspire": "Earth Planet.Sci.Lett."
      },
      {
        "cds": "Eur.Phys.J.ST",
        "inspire": "Eur.Phys.J.ST"
      },
      {
        "cds": "Int. J. Math.",
        "inspire": "Int.J.Math."
      },
      {
        "cds": "Celest. Mech.",
        "inspire": "Celestial Mech."
      },
      {
        "cds": "Ann.Rev.Mater.Sci.",
        "inspire": "Ann.Rev.Mater.Sci."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Zh. Eksp. Teor. Fiz. Pis'ma v Red.",
        "inspire": "Pisma Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Quantum Inf. Comput.",
        "inspire": "Quant.Inf.Comput."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Int.Rev.Cytology",
        "inspire": "Int.Rev.Cytology"
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Z. Phys. Chem.",
        "inspire": "Z.Phys.Chem."
      },
      {
        "cds": "J. Phys. G",
        "inspire": "J.Phys.G"
      },
      {
        "cds": "Electron. Lett.",
        "inspire": "Electron.Lett."
      },
      {
        "cds": "Radiochem.Radioanal.Lett.",
        "inspire": "Radiochem.Radioanal.Lett."
      },
      {
        "cds": "Chem. Rev.",
        "inspire": "Chem.Rev."
      },
      {
        "cds": "IEEE Trans. Microw. Theory Tech.",
        "inspire": "IEEE Trans.Microwave Theor.Tech."
      },
      {
        "cds": "Nucl. Data Sheets",
        "inspire": "Nucl.Data Sheets"
      },
      {
        "cds": "Mod. Phys. Lett.",
        "inspire": "Mod.Phys.Lett."
      },
      {
        "cds": "J.Nonlin.Opt.Phys.Mater.",
        "inspire": "J.Nonlin.Opt.Phys.Mater."
      },
      {
        "cds": "Mem. Am. Math. Soc.",
        "inspire": "Mem.Am.Math.Soc."
      },
      {
        "cds": "Chirality",
        "inspire": "Chirality"
      },
      {
        "cds": "Publ. Inst. Math.",
        "inspire": "Publ.Inst.Math."
      },
      {
        "cds": "Tellus",
        "inspire": "Tellus"
      },
      {
        "cds": "Prog. Low Temp. Phys.",
        "inspire": "Prog.Low Temp.Phys."
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Bulg. J. Phys.",
        "inspire": "Bulg.J.Phys."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "J. Complexity",
        "inspire": "J.Complexity"
      },
      {
        "cds": "Proc. Camb. Philos. Soc.",
        "inspire": "Proc.Cambridge Phil.Soc."
      },
      {
        "cds": "Radiography",
        "inspire": "Radiography"
      },
      {
        "cds": "Laser Phys.",
        "inspire": "Laser Phys."
      },
      {
        "cds": "Am. J. Phys.",
        "inspire": "Am.J.Phys."
      },
      {
        "cds": "Eur.J.Appl.Math.",
        "inspire": "Eur.J.Appl.Math."
      },
      {
        "cds": "Stat. Comput.",
        "inspire": "Stat.Comput."
      },
      {
        "cds": "Astrophys.J.Suppl.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "IEEE Photonics Technol. Lett.",
        "inspire": "IEEE Photonics Tech.Lett."
      },
      {
        "cds": "Proc.Imp.Acad.Tokyo",
        "inspire": "Proc.Imp.Acad.Tokyo"
      },
      {
        "cds": "Am. J. Phys.",
        "inspire": "Am.J.Phys."
      },
      {
        "cds": "Adv. Space Res.",
        "inspire": "Adv.Space Res."
      },
      {
        "cds": "J. Am. Assoc. Var. Star Obs.",
        "inspire": "J.Am.Assoc.Var.Star Obs."
      },
      {
        "cds": "Comments Astrophys. Space Phys.",
        "inspire": "Comments Astrophys. Space Phys."
      },
      {
        "cds": "Math. Comput.",
        "inspire": "Math.Comput."
      },
      {
        "cds": "Ark.Mat.Astron.Fys.",
        "inspire": "Ark.Mat.Astron.Fys."
      },
      {
        "cds": "J. Chromatogr.",
        "inspire": "J.Chromatogr."
      },
      {
        "cds": "Phys. Lett.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Chem. Phys. Lett.",
        "inspire": "Chem.Phys.Lett."
      },
      {
        "cds": "Geophys. Res. Lett.",
        "inspire": "Geophys.Res.Lett."
      },
      {
        "cds": "Phys. Rev. D",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "Pis'ma Zh. Eksp. Teor. Fiz.",
        "inspire": "Pisma Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Trans. Am. Nucl. Soc.",
        "inspire": "Trans.Amer.Nucl.Soc."
      },
      {
        "cds": "J. Geom.",
        "inspire": "J.Geom."
      },
      {
        "cds": "Rep. Prog. Phys.",
        "inspire": "Rept.Prog.Phys."
      },
      {
        "cds": "J.Nonequil.Thermo.",
        "inspire": "J.Nonequil.Thermo."
      },
      {
        "cds": "Curr. Sci.",
        "inspire": "Curr.Sci."
      },
      {
        "cds": "Phys. Rev. Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "J.Univ.Bombay",
        "inspire": "J.Univ.Bombay"
      },
      {
        "cds": "Adv. Phys. Sci.",
        "inspire": "Adv.Phys.Sci."
      },
      {
        "cds": "Comments Nucl.Part.Phys.",
        "inspire": "Comments Nucl.Part.Phys."
      },
      {
        "cds": "J. Phys. D",
        "inspire": "J.Phys.D"
      },
      {
        "cds": "Commun. Algebra",
        "inspire": "Commun.Algebra"
      },
      {
        "cds": "Manuscr.Geod.",
        "inspire": "Manuscr.Geod."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annals Phys."
      },
      {
        "cds": "J. Phys. Soc. Jpn.",
        "inspire": "J.Phys.Soc.Jap."
      },
      {
        "cds": "Bull. Acad. Polon.",
        "inspire": "Bull.Acad.Polon.Sci.(Phys.Astron.)"
      },
      {
        "cds": "Balt. Astron.",
        "inspire": "Baltic Astron."
      },
      {
        "cds": "Adv. Chem. Phys.",
        "inspire": "Adv.Chem.Phys."
      },
      {
        "cds": "Annu. Rev. Nucl. Part. Sci.",
        "inspire": "Complete"
      },
      {
        "cds": "Nuovo Saggiatore",
        "inspire": "Nuovo Saggiatore"
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "J.Molec.Spectrosc.",
        "inspire": "J.Molec.Spectrosc."
      },
      {
        "cds": "Proc. Symp. Pure Math.",
        "inspire": "Proc.Symp.Pure Math."
      },
      {
        "cds": "J. Phys., Colloq.",
        "inspire": "J.Phys.Colloq."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "J.Am.Statist.Assoc.",
        "inspire": "J.Am.Statist.Assoc."
      },
      {
        "cds": "Adv. Appl. Math.",
        "inspire": "Adv.Appl.Math."
      },
      {
        "cds": "Commun.Math.Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Ergeb.Exakt.Naturwiss.",
        "inspire": "Ergeb.Exakt.Naturwiss."
      },
      {
        "cds": "At. Data Nucl. Data Tables",
        "inspire": "Atom.Data Nucl.Data Tabl."
      },
      {
        "cds": "Transp. Theory Stat. Phys.",
        "inspire": "Transp.Theory Statist.Phys."
      },
      {
        "cds": "Electrochim. Acta",
        "inspire": "Electrochim.Acta"
      },
      {
        "cds": "Am. Inst. Aero. Astron.",
        "inspire": "AIAA J."
      },
      {
        "cds": "Osaka J. Math.",
        "inspire": "Osaka J.Math."
      },
      {
        "cds": "Astron.J.",
        "inspire": "Astron.J."
      },
      {
        "cds": "RIKEN Accel.Prog.Rep.",
        "inspire": "RIKEN Accel.Prog.Rep."
      },
      {
        "cds": "Part.Nucl.",
        "inspire": "Part.Nucl."
      },
      {
        "cds": "Math.Mon.",
        "inspire": "Math.Mon."
      },
      {
        "cds": "Behav.Sci.",
        "inspire": "Behav.Sci."
      },
      {
        "cds": "J. Appl. Phys.",
        "inspire": "J.Geophys.Res."
      },
      {
        "cds": "Zygon",
        "inspire": "Zygon"
      },
      {
        "cds": "Adv. Stud. Pure Math.",
        "inspire": "Adv.Stud.Pure Math."
      },
      {
        "cds": "New Astron.",
        "inspire": "New Astron."
      },
      {
        "cds": "Notas Fis.",
        "inspire": "Notas Fis."
      },
      {
        "cds": "Pisma Zh.Eksp.Teor.Fiz.",
        "inspire": "Pisma Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Mat. Vesn.",
        "inspire": "Mat.Vesn."
      },
      {
        "cds": "Exp. Math.",
        "inspire": "Exper.Math."
      },
      {
        "cds": "J. Nondestruct. Eval.",
        "inspire": "J.Nondestruct.Eval."
      },
      {
        "cds": "Int.J.Comput.Vision",
        "inspire": "Int.J.Comput.Vision"
      },
      {
        "cds": "Prog. Theor. Phys., Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "IEEE Trans. Biomed. Eng.",
        "inspire": "IEEE Trans.Biomed.Eng."
      },
      {
        "cds": "Adv.Astron.Astrophys.",
        "inspire": "Adv.Astron.Astrophys."
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Sov.Atom.Energ.",
        "inspire": "Sov.Atom.Energ."
      },
      {
        "cds": "Contemp. Phys.",
        "inspire": "Contemp.Phys."
      },
      {
        "cds": "Nuovo Cim.",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "Sov. J. Nucl. Phys.",
        "inspire": "Sov.J.Nucl.Phys."
      },
      {
        "cds": "Hadron. J., Suppl.",
        "inspire": "Hadronic J.Suppl."
      },
      {
        "cds": "Astron. Lett.",
        "inspire": "Astron.Lett."
      },
      {
        "cds": "Phys.Fenn.",
        "inspire": "Phys.Fenn."
      },
      {
        "cds": "Life Sci.Space Res.",
        "inspire": "Life Sci.Space Res."
      },
      {
        "cds": "Phys. Scr.",
        "inspire": "Phys.Scripta"
      },
      {
        "cds": "Publ. Astron. Soc. Jpn.",
        "inspire": "Publ.Astron.Soc.Jap."
      },
      {
        "cds": "Eur.Phys.J.direct",
        "inspire": "Eur.Phys.J.direct"
      },
      {
        "cds": "Zh. Eksp. Teor. Fiz.",
        "inspire": "Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Appl. Surf. Sci.",
        "inspire": "Appl.Surf.Sci."
      },
      {
        "cds": "Macromolecules",
        "inspire": "Macromolec."
      },
      {
        "cds": "J. Phys. Stud.",
        "inspire": "J.Phys.Stud."
      },
      {
        "cds": "Nuovo Cimento, Lett.",
        "inspire": "Lett.Nuovo Cim."
      },
      {
        "cds": "Funct. Anal. Appl.",
        "inspire": "Funct.Anal.Appl."
      },
      {
        "cds": "Nuovo Cimento A",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "Nuovo Cimento B",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "Sov. J. Part. Nucl.",
        "inspire": "Sov.J.Part.Nucl."
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Hyperfine Interact.",
        "inspire": "Hyperfine Interact."
      },
      {
        "cds": "Info.Sci.",
        "inspire": "Info.Sci."
      },
      {
        "cds": "Phys. Usp.",
        "inspire": "Phys.Usp."
      },
      {
        "cds": "Impact Comput.Sci.Eng.",
        "inspire": "Impact Comput.Sci.Eng."
      },
      {
        "cds": "Comment. Math. Helv.",
        "inspire": "Comment.Math.Helv."
      },
      {
        "cds": "Ark. Fys.",
        "inspire": "Ark.Fys."
      },
      {
        "cds": "Adv. Imaging Electron Phys.",
        "inspire": "Adv.Imaging Electron Phys."
      },
      {
        "cds": "New J. Phys.",
        "inspire": "New J.Phys."
      },
      {
        "cds": "Funk. Anal. Pril.",
        "inspire": "Funkt.Anal.Pril."
      },
      {
        "cds": "Stud.Cerc.Mat.",
        "inspire": "Stud.Cerc.Mat."
      },
      {
        "cds": "Manuscr. Math.",
        "inspire": "Manuscr.Math."
      },
      {
        "cds": "Adv. Comput.",
        "inspire": "Adv.Comput."
      },
      {
        "cds": "Eur. J. Inorg. Chem.",
        "inspire": "Eur.J.Inorg.Chem."
      },
      {
        "cds": "Int.Statist.Rev.",
        "inspire": "Int.Statist.Rev."
      },
      {
        "cds": "Funct. Anal. Appl.",
        "inspire": "Funct.Anal.Appl."
      },
      {
        "cds": "Mol.Cryst.Liq.Cryst.",
        "inspire": "Mol.Cryst.Liq.Cryst."
      },
      {
        "cds": "Commun. Anal. Geom.",
        "inspire": "Commun.Anal.Geom."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Can. J. Phys.",
        "inspire": "Phys.Canada"
      },
      {
        "cds": "Rep. Prog. Phys.",
        "inspire": "Rept.Prog.Phys."
      },
      {
        "cds": "Rend. Fis. Accad. Lincei",
        "inspire": "Rend. Acc. Naz. Lincei"
      },
      {
        "cds": "Gravit. Cosmol.",
        "inspire": "Grav.Cosmol."
      },
      {
        "cds": "J.Nucl.Sci.Tech.",
        "inspire": "J.Nucl.Sci.Tech."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "IEEE Trans. Instrum. Meas.",
        "inspire": "IEEE Trans.Instrum.Measur."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Nucl.Sci.Technol.",
        "inspire": "Nucl.Sci.Technol."
      },
      {
        "cds": "J. Exp. Theor. Phys.",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Nukleonika",
        "inspire": "Nukleonika"
      },
      {
        "cds": "J.Photogr.Sci.",
        "inspire": "J.Photogr.Sci."
      },
      {
        "cds": "At. Energy",
        "inspire": "At.Energ."
      },
      {
        "cds": "Annu. Rev. Astron. Astrophys.",
        "inspire": "Ann.Rev.Astron.Astrophys."
      },
      {
        "cds": "J.Combin.Theor.",
        "inspire": "J.Combin.Theor."
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Prog. Theor. Phys., Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "Can.J.Res.",
        "inspire": "Can.J.Res."
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Proc. Am. Math. Soc.",
        "inspire": "Proc.Am.Math.Soc."
      },
      {
        "cds": "Nucl. Instrum. Methods Phys. Res., ",
        "inspire": "Nucl.Instrum.Meth."
      },
      {
        "cds": "IEEE Trans. Power Deliv.",
        "inspire": "IEEE Trans.Power Deliv."
      },
      {
        "cds": "Comput. Mater. Sci.",
        "inspire": "Comput.Mater.Sci."
      },
      {
        "cds": "Earth Planet. Sci. Lett.",
        "inspire": "Earth Planet.Sci.Lett."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "J. Exp. Theor. Phys.",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Phil.Trans.Roy.Soc.Lond.",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "Rev. Sci. Instrum.",
        "inspire": "Rev.Sci.Instrum."
      },
      {
        "cds": "Rev. Accel. Sci. Technol.",
        "inspire": "Rev.Accel.Sci.Tech."
      },
      {
        "cds": "Acta Sci.Math.(Szeged)",
        "inspire": "Acta Sci.Math.(Szeged)"
      },
      {
        "cds": "Acta.Soc.Sci.Fenn.",
        "inspire": "Acta.Soc.Sci.Fenn."
      },
      {
        "cds": "New Astron. Rev.",
        "inspire": "New Astron.Rev."
      },
      {
        "cds": "Erkenntnis",
        "inspire": "Erkenntnis"
      },
      {
        "cds": "Annali Scuola Sup.Norm.Pisa Sci.Fis.Mat.",
        "inspire": "Annali Scuola Sup.Norm.Pisa Sci.Fis.Mat."
      },
      {
        "cds": "Zh.Eksp.Teor.Fiz.",
        "inspire": "Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Sov.Phys.Collect.",
        "inspire": "Sov.Phys.Collect."
      },
      {
        "cds": "Ber.Verh.Sachs.Akad.Wiss.Leipzig",
        "inspire": "Ber.Verh.Sachs.Akad.Wiss.Leipzig"
      },
      {
        "cds": "C.R.Acad.Sci.I,Math.",
        "inspire": "Compt.Rend.Math."
      },
      {
        "cds": "J. Lie Theory",
        "inspire": "J. Lie Theory"
      },
      {
        "cds": "J. Nonlinear Sci.",
        "inspire": "J.Nonlin.Sci."
      },
      {
        "cds": "Living Rev. Relativ.",
        "inspire": "Living Rev.Rel."
      },
      {
        "cds": "Living Rev. Relativ.",
        "inspire": "Living Rev.Rel."
      },
      {
        "cds": "Neural Comput.",
        "inspire": "Neural Comput."
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "AIP Conf. Proc.",
        "inspire": "AIP Conf.Proc."
      },
      {
        "cds": "Theor. Math. Phys.",
        "inspire": "Theor.Math.Phys."
      },
      {
        "cds": "J. Differ. Geom.",
        "inspire": "J.Diff.Geom."
      },
      {
        "cds": "Acta Astron.",
        "inspire": "Acta Astron."
      },
      {
        "cds": "Contemp. Math.",
        "inspire": "Contemp.Math."
      },
      {
        "cds": "Fiz.Metal.Metal",
        "inspire": "Fiz.Metal.Metal"
      },
      {
        "cds": "J. Math. Chem.",
        "inspire": "J.Math.Chem."
      },
      {
        "cds": "Z. Naturforsch.",
        "inspire": "Z.Naturforsch."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "J. Differ. Geom.",
        "inspire": "J.Diff.Geom."
      },
      {
        "cds": "Prog.Cosmic Ray Phys.",
        "inspire": "Prog.Cosmic Ray Phys."
      },
      {
        "cds": "Sov. Phys. - JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "J. Mech. Phys. Solids",
        "inspire": "J.Mech.Phys.Solids"
      },
      {
        "cds": "Prib.Tekh.Eksp.",
        "inspire": "Prib.Tekh.Eksp."
      },
      {
        "cds": "J. Res. Natl. Inst. Stand. Technol.",
        "inspire": "J.Res.Natl.Inst.Stand.Tech."
      },
      {
        "cds": "Am. J. Phys.",
        "inspire": "Am.J.Phys."
      },
      {
        "cds": "Prog. Part. Nucl. Phys.",
        "inspire": "Prog.Part.Nucl.Phys."
      },
      {
        "cds": "Ann. Phys. (Les Ulis)",
        "inspire": "Annales Phys.(France)"
      },
      {
        "cds": "Semicond.Sci.Tech.",
        "inspire": "Semicond.Sci.Tech."
      },
      {
        "cds": "Physica",
        "inspire": "Physica"
      },
      {
        "cds": "Physics",
        "inspire": "Physics"
      },
      {
        "cds": "Linear Algebra Appl.",
        "inspire": "Linear Algebra Appl."
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Pis'ma Astron. Zh.",
        "inspire": "Pisma Astron.Zh."
      },
      {
        "cds": "Sol. Phys.",
        "inspire": "Solar Phys."
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "New Astron.",
        "inspire": "New Astron."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Phys. Rep.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Int. J. Quantum Chem. Symp.",
        "inspire": "Int.J.Quant.Chem."
      },
      {
        "cds": "J.Atmos.Chem.",
        "inspire": "J.Atmos.Chem."
      },
      {
        "cds": "J.Phys.A",
        "inspire": "J.Phys.A"
      },
      {
        "cds": "Nucl.Sci.Res.Conf.Ser.",
        "inspire": "Nucl.Sci.Res.Conf.Ser."
      },
      {
        "cds": "J. Phys.: Condens. Matter",
        "inspire": "J.Phys.Condens.Matter"
      },
      {
        "cds": "Statist.Sci.",
        "inspire": "Statist.Sci."
      },
      {
        "cds": "Appl. Radiat. Isot.",
        "inspire": "Int.J.Appl.Radiat.Isotopes"
      },
      {
        "cds": "Math. Intellig.",
        "inspire": "Math.Intelligencer"
      },
      {
        "cds": "Analele Univ.Timisoara (Stiinte Fiz.)",
        "inspire": "Analele Univ.Timisoara (Stiinte Fiz.)"
      },
      {
        "cds": "Publ. Astron. Soc. Pac.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "AAPPS Bull.",
        "inspire": "AAPPS Bull."
      },
      {
        "cds": "Sib.Mat.Zh.",
        "inspire": "Sib.Mat.Zh."
      },
      {
        "cds": "Angew. Chem. Int. Ed.",
        "inspire": "Angew.Chem.Int.Ed."
      },
      {
        "cds": "J. Knot Theory Ramif.",
        "inspire": "J.Knot Theor.Ramifications"
      },
      {
        "cds": "J. Classif.",
        "inspire": "J.Classif."
      },
      {
        "cds": "Sol. Phys.",
        "inspire": "Solar Phys."
      },
      {
        "cds": "Astrophys. Lett. Commun.",
        "inspire": "Astrophys.Lett.Commun."
      },
      {
        "cds": "Phil.Trans.Roy.Soc.Lond.",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "Eur. Phys. J. Appl. Phys.",
        "inspire": "Eur.Phys.J.Appl.Phys."
      },
      {
        "cds": "Stud.Hist.Phil.Sci.",
        "inspire": "Stud.Hist.Phil.Sci."
      },
      {
        "cds": "Ann. Math.",
        "inspire": "Annals Math."
      },
      {
        "cds": "Metrika",
        "inspire": "Metrika"
      },
      {
        "cds": "Opt. Acta",
        "inspire": "Opt.Acta"
      },
      {
        "cds": "Annalen Phys.",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "J. Opt. Soc. Am.",
        "inspire": "J.Opt.Soc.Am."
      },
      {
        "cds": "Nuovo Cim.",
        "inspire": "Riv.Nuovo Cim."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "ACM Trans. Math. Softw.",
        "inspire": "ACM Trans.Math.Software"
      },
      {
        "cds": "J. Algebra",
        "inspire": "J.Algebra"
      },
      {
        "cds": "Proc. R. Soc. Lond.",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "J. Differ. Geom.",
        "inspire": "J.Diff.Geom."
      },
      {
        "cds": "Sci.Rep.Nat Tsing Hua Univ.Ser.A",
        "inspire": "Sci.Rep.Nat Tsing Hua Univ.Ser.A"
      },
      {
        "cds": "Physica",
        "inspire": "Physica"
      },
      {
        "cds": "J.Group Theor.Phys.",
        "inspire": "J.Group Theor.Phys."
      },
      {
        "cds": "Laser Photonics. Rev.",
        "inspire": "Laser Photonics Rev."
      },
      {
        "cds": "Prog. Theor. Phys., Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "J. Phys.",
        "inspire": "J.Phys.A"
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Proc. IEEE",
        "inspire": "IEEE Proc."
      },
      {
        "cds": "Yad. Fiz.",
        "inspire": "Yad.Fiz."
      },
      {
        "cds": "Icarus",
        "inspire": "Icarus"
      },
      {
        "cds": "Algorithmica",
        "inspire": "Algorithmica"
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Prib.Tekh.Eksp.",
        "inspire": "Prib.Tekh.Eksp."
      },
      {
        "cds": "Radiocarbon",
        "inspire": "Radiocarbon"
      },
      {
        "cds": "Astron. Astrophys. Suppl. Ser.",
        "inspire": "Astron.Astrophys.Suppl.Ser."
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Nuovo Cimento",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "Illinois J. Math.",
        "inspire": "Ill.J.Math."
      },
      {
        "cds": "Nucl. Phys.",
        "inspire": "Nucl.Phys."
      },
      {
        "cds": "Bull.London Math.Soc.",
        "inspire": "Bull.London Math.Soc."
      },
      {
        "cds": "Aust. J. Phys.",
        "inspire": "Austral.J.Phys."
      },
      {
        "cds": "Meteorit. Planet. Sci.",
        "inspire": "Meteoritics"
      },
      {
        "cds": "Rev. Mod. Astron.",
        "inspire": "Rev.Mod.Astron."
      },
      {
        "cds": "Int.J.Rad.Phys.Chem.",
        "inspire": "Int.J.Rad.Phys.Chem."
      },
      {
        "cds": "SIAM J. Appl. Math.",
        "inspire": "SIAM J.Appl.Math."
      },
      {
        "cds": "Astron. Zh.",
        "inspire": "Astron.Zh."
      },
      {
        "cds": "Oper.Theor.Advan.Appl.",
        "inspire": "Oper.Theor.Advan.Appl."
      },
      {
        "cds": "J.Roy.Statist.Soc.A",
        "inspire": "J.Roy.Statist.Soc.B"
      },
      {
        "cds": "Nippon Butsuri Gakkaishi",
        "inspire": "Nippon Butsuri Gakkaishi"
      },
      {
        "cds": "Nuovo Cimento",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "Sov. Phys. - Dokl.",
        "inspire": "Sov.Phys.Dokl."
      },
      {
        "cds": "J. Comput. Chem.",
        "inspire": "J.Comput.Chem."
      },
      {
        "cds": "Dokl.Akad.Nauk Ser.Fiz.",
        "inspire": "Dokl.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Nuovo Cimento",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "J. Phys. E",
        "inspire": "J.Phys.E"
      },
      {
        "cds": "Jpn. J. Math.",
        "inspire": "Math.Jap."
      },
      {
        "cds": "Acta Philos.Fenn.",
        "inspire": "Acta Philos.Fenn."
      },
      {
        "cds": "Fiz. Elem. Chastits At. Yadra",
        "inspire": "Fiz.Elem.Chast.Atom.Yadra"
      },
      {
        "cds": "Nucl. Fusion",
        "inspire": "Nucl.Fusion"
      },
      {
        "cds": "Philips Tech. Rev.",
        "inspire": "Philips Tech.Rev."
      },
      {
        "cds": "J.Stat.Mech.",
        "inspire": "J.Stat.Mech."
      },
      {
        "cds": "Teor. Mat. Fiz.",
        "inspire": "Theor.Math.Phys."
      },
      {
        "cds": "Intermetallics",
        "inspire": "Intermetallics"
      },
      {
        "cds": "Phys. Rev. Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "Measurement",
        "inspire": "Measurement (London)"
      },
      {
        "cds": "Math. Notes",
        "inspire": "Math.Notes"
      },
      {
        "cds": "J. Mater. Res.",
        "inspire": "J.Mater.Res."
      },
      {
        "cds": "Ann. Rev. Nucl. Part. Phys.",
        "inspire": "Complete"
      },
      {
        "cds": "Adv. Phys.",
        "inspire": "Adv.Phys."
      },
      {
        "cds": "Libertas Math.",
        "inspire": "Libertas Math."
      },
      {
        "cds": "Zap. Nauchnykh Semin. POMI",
        "inspire": "Zap.Nauchn.Semin."
      },
      {
        "cds": "Solid State Commun.",
        "inspire": "Solid State Commun."
      },
      {
        "cds": "Fiz.Mnogochast.Sistem",
        "inspire": "Fiz.Mnogochast.Sistem"
      },
      {
        "cds": "J. Low Temp. Phys.",
        "inspire": "J.Low.Temp.Phys."
      },
      {
        "cds": "Laser Phys. Lett.",
        "inspire": "Laser Phys.Lett."
      },
      {
        "cds": "Radiofizika",
        "inspire": "Izv.Vuz.Radiofiz."
      },
      {
        "cds": "IEEE Photonics Technol. Lett.",
        "inspire": "IEEE Photonics Tech.Lett."
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Rev. Math. Phys.",
        "inspire": "Rev.Math.Phys."
      },
      {
        "cds": "Markov Processes Relat. Fields",
        "inspire": "Markov Process. Related Fields"
      },
      {
        "cds": "Aust. J. Phys.",
        "inspire": "Austral.J.Phys."
      },
      {
        "cds": "J. Stat. Phys.",
        "inspire": "J.Statist.Phys."
      },
      {
        "cds": "Proc. Natl. Acad. Sci. U. S. A.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "Commun. Algebra",
        "inspire": "Commun.Algebra"
      },
      {
        "cds": "Proc.Indian Natl.Sci.Acad.",
        "inspire": "Proc.Indian Natl.Sci.Acad."
      },
      {
        "cds": "Rev.R.Acad.Cien.Exactas Fis.Nat.Ser.A Mat.",
        "inspire": "Rev.R.Acad.Cien.Exactas Fis.Nat.Ser.A Mat."
      },
      {
        "cds": "IEEE Trans. Automat. Control",
        "inspire": "IEEE Trans.Automatic Control"
      },
      {
        "cds": "Riv.Mat.Univ.Parma",
        "inspire": "Riv.Mat.Univ.Parma"
      },
      {
        "cds": "J. Phys. Soc. Jpn.",
        "inspire": "J.Phys.Soc.Jap."
      },
      {
        "cds": "Proc.Phys.Math.Soc.Jap.",
        "inspire": "Proc.Phys.Math.Soc.Jap."
      },
      {
        "cds": "Riv.Nuovo Cim.",
        "inspire": "Riv.Nuovo Cim."
      },
      {
        "cds": "Result.Math.",
        "inspire": "Result.Math."
      },
      {
        "cds": "Phys. Bull.",
        "inspire": "Phys.Bull."
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "Annu. Rev. Phys. Chem.",
        "inspire": "Ann.Rev.Phys.Chem."
      },
      {
        "cds": "Highlights Astron.",
        "inspire": "Highlights Astron."
      },
      {
        "cds": "J.Phil.Logic",
        "inspire": "J.Phil.Logic"
      },
      {
        "cds": "Rev. Colomb. Mat.",
        "inspire": "Rev.Colomb.Mat."
      },
      {
        "cds": "Nonlinearity",
        "inspire": "Nonlinearity"
      },
      {
        "cds": "Sov.Phys.JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Scand.Actuarial J.",
        "inspire": "Scand.Actuarial J."
      },
      {
        "cds": "Trans.Jap.Inst.Met.",
        "inspire": "Trans.Jap.Inst.Met."
      },
      {
        "cds": "Radiochim. Acta",
        "inspire": "Radiochim.Acta"
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Geophys.Res.Lett.",
        "inspire": "Geophys.Res.Lett."
      },
      {
        "cds": "Annu. Rev. Astron. Astrophys.",
        "inspire": "Ann.Rev.Astron.Astrophys."
      },
      {
        "cds": "Nonlinear Process. Geophys.",
        "inspire": "Nonlin.Proc.Geophys."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals N.Y.Acad.Sci."
      },
      {
        "cds": "Kon.Ned.Akad.Wetensch.Proc.",
        "inspire": "Kon.Ned.Akad.Wetensch.Proc."
      },
      {
        "cds": "Rev. Sci. Instrum.",
        "inspire": "Rev.Sci.Instrum."
      },
      {
        "cds": "Astrophys. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Leningrad Math.J.",
        "inspire": "Leningrad Math.J."
      },
      {
        "cds": "Lect. Notes Pure Appl. Math.",
        "inspire": "Lect.Notes Pure Appl.Math."
      },
      {
        "cds": "Chem.Scripta",
        "inspire": "Chem.Scripta"
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "J. Fluid Mech.",
        "inspire": "J.Fluid Mech."
      },
      {
        "cds": "J. Phys.: Condens. Matter",
        "inspire": "J.Phys.Condens.Matter"
      },
      {
        "cds": "J. Opt. Soc. Am. B",
        "inspire": "J.Opt.Soc.Am."
      },
      {
        "cds": "Proc.Utah.Acad.Sci.",
        "inspire": "Proc.Utah.Acad.Sci."
      },
      {
        "cds": "Adv. Comput.",
        "inspire": "Adv.Comput."
      },
      {
        "cds": "Sov. Astron. Lett.",
        "inspire": "Sov.Astron.Lett."
      },
      {
        "cds": "Czechoslov. Math. J.",
        "inspire": "Czech.Math.J."
      },
      {
        "cds": "High Energ.Chem.",
        "inspire": "High Energ.Chem."
      },
      {
        "cds": "At. Data",
        "inspire": "Atom.Data"
      },
      {
        "cds": "Sov. J. Nucl. Phys.",
        "inspire": "Sov.J.Nucl.Phys."
      },
      {
        "cds": "J. Stat. Phys.",
        "inspire": "J.Statist.Phys."
      },
      {
        "cds": "Acta Arith.",
        "inspire": "Acta Arith."
      },
      {
        "cds": "J.Quant.Spectrosc.Radiat.Trans.",
        "inspire": "J.Quant.Spectrosc.Radiat.Trans."
      },
      {
        "cds": "Phys. Life Rev.",
        "inspire": "Phys.Life Rev."
      },
      {
        "cds": "Sov. Phys. - JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "New Sci.",
        "inspire": "New Sci."
      },
      {
        "cds": "Ann. Sci. Ec. Norm. Supér.",
        "inspire": "Annales Sci.Ecole Norm.Sup."
      },
      {
        "cds": "IEEE Trans. Signal Process.",
        "inspire": "IEEE Trans.Signal Process."
      },
      {
        "cds": "Metall. Trans.",
        "inspire": "Metall.Trans."
      },
      {
        "cds": "J.Br.Interplanet.Soc.",
        "inspire": "J.Br.Interplanet.Soc."
      },
      {
        "cds": "Acta Inform.",
        "inspire": "Acta Inform."
      },
      {
        "cds": "J. Lond. Math. Soc.",
        "inspire": "Bull.London Math.Soc."
      },
      {
        "cds": "Liet.TSR Mokslu Akad.Darbai (Ser.B)",
        "inspire": "Liet.TSR Mokslu Akad.Darbai (Ser.B)"
      },
      {
        "cds": "SIAM J. Math. Anal.",
        "inspire": "SIAM J.Math.Anal."
      },
      {
        "cds": "J. R. Astron. Soc. Can.",
        "inspire": "J.Roy.Astron.Soc.Canada"
      },
      {
        "cds": "Spacetime Subst.",
        "inspire": "Spacetime and Substance"
      },
      {
        "cds": "Nucleic Acids Res.",
        "inspire": "Nucleic Acids Res."
      },
      {
        "cds": "J.Phys.Oceanogr.",
        "inspire": "J.Phys.Oceanogr."
      },
      {
        "cds": "SIAM J.Sci.Statist.Comput.",
        "inspire": "SIAM J.Comput."
      },
      {
        "cds": "Math. Res. Lett.",
        "inspire": "Math.Res.Lett."
      },
      {
        "cds": "Eur. Phys. J. C",
        "inspire": "Eur.Phys.J."
      },
      {
        "cds": "Mem.Roy.Astron.Soc.",
        "inspire": "Mem.Roy.Astron.Soc."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals N.Y.Acad.Sci."
      },
      {
        "cds": "Measurement (London)",
        "inspire": "Measurement (London)"
      },
      {
        "cds": "Trans. Camb. Philos. Soc.",
        "inspire": "Trans.Cambridge Phil.Soc."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Adv. Chem. Phys.",
        "inspire": "Adv.Chem.Phys."
      },
      {
        "cds": "J. Am. Math. Soc.",
        "inspire": "J.Am.Math.Soc."
      },
      {
        "cds": "Acta Phys. Sin.",
        "inspire": "Acta Phys.Sin."
      },
      {
        "cds": "Int. J. Bifurc. Chaos",
        "inspire": "Int.J.Bifurc.Chaos"
      },
      {
        "cds": "Lett. Nuovo Cimento",
        "inspire": "Lett.Nuovo Cim."
      },
      {
        "cds": "J. Am. Chem. Soc.",
        "inspire": "J.Am.Chem.Soc."
      },
      {
        "cds": "Philos. Trans. R. Soc. Lond., A",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "Acta Cryst.",
        "inspire": "Acta Cryst."
      },
      {
        "cds": "J. Low Temp. Phys.",
        "inspire": "J.Low.Temp.Phys."
      },
      {
        "cds": "Comput. Phys. Rep.",
        "inspire": "Comput.Phys.Rept."
      },
      {
        "cds": "Appl. Phys. Lett.",
        "inspire": "Appl.Phys.Lett."
      },
      {
        "cds": "Opt. Commun.",
        "inspire": "Opt.Commun."
      },
      {
        "cds": "J.Org.Chem",
        "inspire": "J.Org.Chem"
      },
      {
        "cds": "Opt. Eng.",
        "inspire": "Opt.Eng."
      },
      {
        "cds": "Adv. Radiat. Chem.",
        "inspire": "Adv.Radiat.Chem."
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "Fuel",
        "inspire": "Fuel"
      },
      {
        "cds": "Physica Medica",
        "inspire": "Phys.Medica"
      },
      {
        "cds": "J. Astrophys. Astron.",
        "inspire": "J.Astrophys.Astron."
      },
      {
        "cds": "Funkt.Anal.Pril.",
        "inspire": "Funkt.Anal.Pril."
      },
      {
        "cds": "J. Vac. Sci. Technol. A",
        "inspire": "J.Vac.Sci.Tech."
      },
      {
        "cds": "Fusion Technol.",
        "inspire": "Fusion Tech."
      },
      {
        "cds": "At.Energ.",
        "inspire": "Sov.Atom.Energ."
      },
      {
        "cds": "J. Symb. Log.",
        "inspire": "J.Symbolic Logic"
      },
      {
        "cds": "Electron.Power",
        "inspire": "Electron.Power"
      },
      {
        "cds": "IEEE Trans. Magn.",
        "inspire": "IEEE Trans.Magnetics"
      },
      {
        "cds": "J. Mol. Spectrosc.",
        "inspire": "J.Molec.Spectrosc."
      },
      {
        "cds": "Eur.Phys.J.",
        "inspire": "Eur.Phys.J."
      },
      {
        "cds": "J. Appl. Math. Mech.",
        "inspire": "J.Math.Mech."
      },
      {
        "cds": "Helv. Phys. Acta",
        "inspire": "Helv.Phys.Acta"
      },
      {
        "cds": "Bul.Acad.Sti.Rep.Moldova (Fiz.Teh.)",
        "inspire": "Bul.Acad.Sti.Rep.Moldova (Fiz.Teh.)"
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Phys. Rev. Spec. Top. Accel. Beams",
        "inspire": "Phys.Rev.ST Accel.Beams"
      },
      {
        "cds": "Sov.Phys.Cryst.",
        "inspire": "Sov.Phys.Cryst."
      },
      {
        "cds": "Behaviormetrika",
        "inspire": "Behaviormetrika"
      },
      {
        "cds": "J. Geod. Soc. Jpn.",
        "inspire": "J.Geodet.Soc.Jap."
      },
      {
        "cds": "IEEE Trans.Med.Imag.",
        "inspire": "IEEE Trans.Med.Imag."
      },
      {
        "cds": "IEEE Comput. Sci. Eng.",
        "inspire": "IEEE Comput.Sci.Eng."
      },
      {
        "cds": "Carbon",
        "inspire": "Carbon"
      },
      {
        "cds": "Bull.Sci.Math.",
        "inspire": "Bull.Sci.Math."
      },
      {
        "cds": "Kratk Soobshch. Fiz.",
        "inspire": "Kratk.Soobshch.Fiz."
      },
      {
        "cds": "Comput.Artif.Intell.",
        "inspire": "Comput.Artif.Intell."
      },
      {
        "cds": "Analysis",
        "inspire": "Analysis"
      },
      {
        "cds": "Mem.Soc.It.Sc.(Dei XL)",
        "inspire": "Mem.Soc.It.Sc.(Dei XL)"
      },
      {
        "cds": "Mod. Phys. Lett. B",
        "inspire": "Mod.Phys.Lett."
      },
      {
        "cds": "Mod. Phys. Lett. A",
        "inspire": "Mod.Phys.Lett."
      },
      {
        "cds": "Linear Algebra Appl.",
        "inspire": "Linear Algebra Appl."
      },
      {
        "cds": "Pion Nucleon Newsl.",
        "inspire": "PiN Newslett."
      },
      {
        "cds": "Astron. Zh.",
        "inspire": "Astron.Zh."
      },
      {
        "cds": "J. Opt. Soc. Am.",
        "inspire": "J.Opt.Soc.Am."
      },
      {
        "cds": "Comment. Math. Helv.",
        "inspire": "Comment.Math.Helv."
      },
      {
        "cds": "IEEE Trans. Magn.",
        "inspire": "IEEE Trans.Magnetics"
      },
      {
        "cds": "ACM Trans. Math. Softw.",
        "inspire": "ACM Trans.Math.Software"
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "J. Appl. Mech. Tech. Phys.",
        "inspire": "J.Appl.Mech.Tech.Phys."
      },
      {
        "cds": "Monogr. Mat.",
        "inspire": "Monogr.Mat."
      },
      {
        "cds": "IEEE Trans. Antennas Propag.",
        "inspire": "IEEE Trans.Anten.Propag."
      },
      {
        "cds": "J. Cosmol. Astropart. Phys.",
        "inspire": "JCAP"
      },
      {
        "cds": "HEP-PH",
        "inspire": "HEP-PH"
      },
      {
        "cds": "Sov.Phys.Usp.",
        "inspire": "Sov.Phys.Usp."
      },
      {
        "cds": "Nagoya Math. J.",
        "inspire": "Nagoya Math.J."
      },
      {
        "cds": "Dokl. Phys.",
        "inspire": "Dokl.Phys."
      },
      {
        "cds": "Condens. Matter Phys.",
        "inspire": "Condensed Matter Phys."
      },
      {
        "cds": "Izv.Fiz.Inst.Bulg.Akad.Nauk.",
        "inspire": "Izv.Fiz.Inst.Bulg.Akad.Nauk."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "J. Phys. Lett.",
        "inspire": "J.Phys.Lett."
      },
      {
        "cds": "J. Algebr. Comb.",
        "inspire": "J.Algebr.Comb."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Syst. Control Lett.",
        "inspire": "Syst.Control Lett."
      },
      {
        "cds": "Quantum Semiclass. Opt.",
        "inspire": "Quant.Semiclass.Opt."
      },
      {
        "cds": "Phys.Rev.ST Accel.Beams",
        "inspire": "Phys.Rev.ST Accel.Beams"
      },
      {
        "cds": "Few Body Syst.Suppl.",
        "inspire": "Few Body Syst.Suppl."
      },
      {
        "cds": "Chaos Solitons Fractals",
        "inspire": "Chaos Solitons Fractals"
      },
      {
        "cds": "Commun. Pure Appl. Math.",
        "inspire": "Commun.Pure Appl.Math."
      },
      {
        "cds": "Chin. Astron. Astrophys.",
        "inspire": "Chin.Astron.Astrophys."
      },
      {
        "cds": "Onde Electr.",
        "inspire": "Onde.Elec."
      },
      {
        "cds": "Bull. Soc. Math. Fr.",
        "inspire": "Bull.Soc.Math.Fr."
      },
      {
        "cds": "Math. Z.",
        "inspire": "Math.Z."
      },
      {
        "cds": "FEBS Lett.",
        "inspire": "Febs Lett."
      },
      {
        "cds": "Inorg.Nucl.Chem.Lett.",
        "inspire": "Inorg.Nucl.Chem.Lett."
      },
      {
        "cds": "J. Differ. Geom.",
        "inspire": "J.Diff.Geom."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "J. Am. Stat. Assoc.",
        "inspire": "J.Am.Statist.Assoc."
      },
      {
        "cds": "MagnetoHydrodyn.",
        "inspire": "Magnetohydrodynamics"
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "At. Data Nucl. Data Tables",
        "inspire": "Atom.Data Nucl.Data Tabl."
      },
      {
        "cds": "Waves Random Media",
        "inspire": "Waves Random Media"
      },
      {
        "cds": "Appl.Laser",
        "inspire": "Appl.Laser"
      },
      {
        "cds": "Nucl.Med.Biol.",
        "inspire": "Nucl.Med.Biol."
      },
      {
        "cds": "SIAM J. Numer. Anal.",
        "inspire": "SIAM J.Numer.Anal."
      },
      {
        "cds": "Electron. Lett.",
        "inspire": "Electron.Lett."
      },
      {
        "cds": "Zastos.Mat.",
        "inspire": "Zastos.Mat."
      },
      {
        "cds": "Astron. Soc. Pac. Conf. Proc.",
        "inspire": "ASP Conf.Ser."
      },
      {
        "cds": "Acta Astron.",
        "inspire": "Acta Astron."
      },
      {
        "cds": "J. Phys. Chem. Solids",
        "inspire": "J.Phys.Chem.Solids"
      },
      {
        "cds": "Part. Accel.",
        "inspire": "Part.Accel."
      },
      {
        "cds": "Phys. Lett.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "J. Struct. Biol.",
        "inspire": "J.Struct.Biol."
      },
      {
        "cds": "Phys. Usp.",
        "inspire": "Phys.Usp."
      },
      {
        "cds": "Proc. Steklov Inst. Math.",
        "inspire": "Proc.Steklov Inst.Math."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Am. Inst. Phys. Conf. Proc.",
        "inspire": "AIP Conf.Proc."
      },
      {
        "cds": "Uspekhi Fiz. Nauk",
        "inspire": "Usp.Fiz.Nauk"
      },
      {
        "cds": "J. Cryst. Growth",
        "inspire": "J.Cryst.Growth"
      },
      {
        "cds": "Gravit. Cosmol., Suppl.",
        "inspire": "Grav.Cosmol.Suppl."
      },
      {
        "cds": "Instrum. Exp. Tech.",
        "inspire": "Instrum.Exp.Tech."
      },
      {
        "cds": "At. Data Nucl. Data Tables",
        "inspire": "Atom.Data Nucl.Data Tabl."
      },
      {
        "cds": "J. Opt.",
        "inspire": "J.Optics"
      },
      {
        "cds": "Bull. Astron. Soc. India",
        "inspire": "Bull.Astron.Soc.India"
      },
      {
        "cds": "Phys. Scr.",
        "inspire": "Phys.Scripta"
      },
      {
        "cds": "Am.Statist.",
        "inspire": "Am.Statist."
      },
      {
        "cds": "J.Nonlin.Sci.",
        "inspire": "J.Nonlin.Sci."
      },
      {
        "cds": "Phys.Lett.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Electron. Commun. Probab.",
        "inspire": "Electron.Commun.Probab."
      },
      {
        "cds": "Comput. Stand. Interfaces",
        "inspire": "Comput.Stand.Interfaces"
      },
      {
        "cds": "J. Acoust. Soc. Am.",
        "inspire": "J.Acoust.Soc.Am."
      },
      {
        "cds": "J. Differ. Geom.",
        "inspire": "J.Diff.Geom."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Lect. Notes Math.",
        "inspire": "Lect.Notes Math."
      },
      {
        "cds": "Nonlinear Differ.Eq.Appl.",
        "inspire": "Nonlinear Differ.Eq.Appl."
      },
      {
        "cds": "J. Oper. Theory",
        "inspire": "J.Operator Theor."
      },
      {
        "cds": "Math. Z.",
        "inspire": "Math.Z."
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Proc. Am. Math. Soc.",
        "inspire": "Proc.Am.Math.Soc."
      },
      {
        "cds": "Astron. Nachr.",
        "inspire": "Astron.Nachr."
      },
      {
        "cds": "Ann. Inst. Stat. Math.",
        "inspire": "Annals Inst.Stat.Math."
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Polymer",
        "inspire": "Polymer"
      },
      {
        "cds": "J. Object Oriented Program.",
        "inspire": "J.Object-Oriented program."
      },
      {
        "cds": "Phys. Lett.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Proc. Natl. Acad. Sci. U. S. A.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "Ann.Rev.Psych.",
        "inspire": "Ann.Rev.Psych."
      },
      {
        "cds": "Naturwissenschaften",
        "inspire": "Naturwiss."
      },
      {
        "cds": "J. Polym. Sci. B",
        "inspire": "J.Polymer Sci."
      },
      {
        "cds": "Few Body Syst.",
        "inspire": "Few Body Syst."
      },
      {
        "cds": "Math. USSR Izvestiya",
        "inspire": "Math.USSR Izv."
      },
      {
        "cds": "Izv.Ross.Akad.Nauk Ser.Fiz.",
        "inspire": "Izv.Ross.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "J.Photochem.Photobiol.",
        "inspire": "J.Photochem.Photobiol."
      },
      {
        "cds": "Ferroelectrics",
        "inspire": "Ferroelectrics"
      },
      {
        "cds": "Comput. Stat. Data Anal.",
        "inspire": "Comput.Stat.Data Anal."
      },
      {
        "cds": "Neutron News",
        "inspire": "Neutron News"
      },
      {
        "cds": "Ann. Phys. (Les Ulis)",
        "inspire": "Annales Phys.(France)"
      },
      {
        "cds": "Ukr.Mat.Zh.",
        "inspire": "Ukr.Mat.Zh."
      },
      {
        "cds": "Phys.Rept.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "IEEE Trans.Info.Theor.",
        "inspire": "IEEE Trans.Info.Theor."
      },
      {
        "cds": "Appl. Phys. Lett.",
        "inspire": "Appl.Phys.Lett."
      },
      {
        "cds": "Ann. Glob. Anal. Geom.",
        "inspire": "Annals Global Anal.Geom."
      },
      {
        "cds": "Enseign.Math.",
        "inspire": "Enseign.Math."
      },
      {
        "cds": "J. Geomagn. Geoelectr.",
        "inspire": "J.Geomag.Geoelectr.Japan"
      },
      {
        "cds": "Commun. Anal. Geom.",
        "inspire": "Commun.Anal.Geom."
      },
      {
        "cds": "Inform. Proc. Lett.",
        "inspire": "Inform.Proc.Lett."
      },
      {
        "cds": "Artif. Intell.",
        "inspire": "Artificial Intel."
      },
      {
        "cds": "Sov. J. Supercond. Temp.",
        "inspire": "Fiz.Nizk.Temp."
      },
      {
        "cds": "Annu. Rev. Nucl. Part. Sci.",
        "inspire": "Complete"
      },
      {
        "cds": "New J. Phys.",
        "inspire": "New J.Phys."
      },
      {
        "cds": "IEEE Microw. Guid. Wave Lett.",
        "inspire": "IEEE Microw.Guid.Wave Lett."
      },
      {
        "cds": "Doc.Math.J.DMV",
        "inspire": "Doc.Math.J.DMV"
      },
      {
        "cds": "Learn. Publ.",
        "inspire": "Learn.Publ."
      },
      {
        "cds": "Numer. Math.",
        "inspire": "Numer.Math."
      },
      {
        "cds": "J. Phys. G",
        "inspire": "J.Phys.G"
      },
      {
        "cds": "Caribbean J.Sci.Math.",
        "inspire": "Caribbean J.Sci.Math."
      },
      {
        "cds": "J. Plasma Fusion Res.",
        "inspire": "J.Plasma Fusion Res."
      },
      {
        "cds": "Spectrochimica Acta, A",
        "inspire": "Spectrochim.Acta A"
      },
      {
        "cds": "Algebr. Geom. Topol.",
        "inspire": "Algebr.Geom.Topol."
      },
      {
        "cds": "J.Roy.Astron.Soc.Canada",
        "inspire": "J.Roy.Astron.Soc.Canada"
      },
      {
        "cds": "Astrophys. Lett. Commun.",
        "inspire": "Astrophys.Lett.Commun."
      },
      {
        "cds": "Nonlin.Phenom.Complex Syst.",
        "inspire": "Nonlin.Phenom.Complex Syst."
      },
      {
        "cds": "Commun. Part. Differ. Equ.",
        "inspire": "Commun.Part.Diff.Eq."
      },
      {
        "cds": "Constr. Approx.",
        "inspire": "Constructive Approx."
      },
      {
        "cds": "Nucleus",
        "inspire": "Nucleus"
      },
      {
        "cds": "Asian J. Math.",
        "inspire": "Asian J.Math"
      },
      {
        "cds": "Bull.Acad.Polon.Sci.(Phys.Astron.)",
        "inspire": "Bull.Acad.Polon.Sci.(Phys.Astron.)"
      },
      {
        "cds": "Sci.Tech.Libraries",
        "inspire": "Sci.Tech.Libraries"
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "Microproc.Microprog.",
        "inspire": "Microproc.Microprog."
      },
      {
        "cds": "Diff.Urav.",
        "inspire": "Diff.Urav."
      },
      {
        "cds": "Numer. Math.",
        "inspire": "Numer.Math."
      },
      {
        "cds": "Phys.Met.Metallogr.",
        "inspire": "Phys.Met.Metallogr."
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "ALG-GEOM",
        "inspire": "ALG-GEOM"
      },
      {
        "cds": "Int.J.Supercomput.Appl.",
        "inspire": "Int.J.Supercomput.Appl."
      },
      {
        "cds": "Sens. Actuators",
        "inspire": "Sens.Actuators"
      },
      {
        "cds": "Fundam. Cosm. Phys.",
        "inspire": "Fund.Cosmic Phys."
      },
      {
        "cds": "Mon. Weather Rev.",
        "inspire": "Mon.Weather Rev."
      },
      {
        "cds": "Analysis (Oldenburg)",
        "inspire": "Analysis (Oldenburg)"
      },
      {
        "cds": "Mathematica J.",
        "inspire": "Mathematica J."
      },
      {
        "cds": "Ark.Mat.",
        "inspire": "Ark.Mat."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "J. Eng. Math.",
        "inspire": "J.Eng.Math."
      },
      {
        "cds": "Alg. Group Geom.",
        "inspire": "Alg.Groups Geom."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Astron. J.",
        "inspire": "Astron.J."
      },
      {
        "cds": "Izv.Ross.Akad.Nauk.",
        "inspire": "Izv.Ross.Akad.Nauk."
      },
      {
        "cds": "Physicalia",
        "inspire": "Physicalia Mag."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Glasnik Mat.",
        "inspire": "Glasnik Mat."
      },
      {
        "cds": "Astron.Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Proc. Astron. Soc. Pac.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "Chemtron.",
        "inspire": "Chemtron."
      },
      {
        "cds": "Stoch.Proc.Appl.",
        "inspire": "Stoch.Proc.Appl."
      },
      {
        "cds": "Ann. Henri Poincaré",
        "inspire": "Annales Poincare Phys.Theor."
      },
      {
        "cds": "Biochem.",
        "inspire": "Biochem."
      },
      {
        "cds": "MRS Symp. Proc.",
        "inspire": "Mater.Res.Soc.Symp.Proc."
      },
      {
        "cds": "Algebr. Represent. Theory",
        "inspire": "Algebr.Represent.Theory"
      },
      {
        "cds": "Publ. Astron. Soc. Jpn.",
        "inspire": "Publ.Astron.Soc.Jap."
      },
      {
        "cds": "Prog. Nucl. Energy",
        "inspire": "Prog.Nucl.Energy"
      },
      {
        "cds": "Int.J.Phys.",
        "inspire": "Int.J.Phys."
      },
      {
        "cds": "Annu. Rev. Earth Planet. Sci.",
        "inspire": "Ann.Rev.Earth Planet.Sci."
      },
      {
        "cds": "IEEE Trans. Pattern Anal. Mach. Intell.",
        "inspire": "IEEE Trans.Pattern Anal.Machine Intell."
      },
      {
        "cds": "Phys. Rev. Spec. Top. Accel. Beams",
        "inspire": "Phys.Rev.ST Accel.Beams"
      },
      {
        "cds": "Z. Angew. Math. Mech.",
        "inspire": "Z.Angew.Math.Mech."
      },
      {
        "cds": "Adv. Phys.",
        "inspire": "Adv.Phys."
      },
      {
        "cds": "Ann. Stat.",
        "inspire": "Annals Statist."
      },
      {
        "cds": "Ann. Phys. (San Diego)",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Adv. Colloid Interface Sci.",
        "inspire": "Adv.Colloid Interface Sci."
      },
      {
        "cds": "Chin. J. Phys.",
        "inspire": "Chin.J.Phys."
      },
      {
        "cds": "Bull.Res.Coun.Israel.(A.Chem.)",
        "inspire": "Bull.Res.Coun.Israel.(A.Chem.)"
      },
      {
        "cds": "Verh. Deutsch. Phys. Ges",
        "inspire": "Verh.Phys.Gesell."
      },
      {
        "cds": "Q. J. Math.",
        "inspire": "Quart.J.Math.Oxford Ser."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Astron. Astrophys. Suppl. Ser.",
        "inspire": "Astron.Astrophys.Suppl.Ser."
      },
      {
        "cds": "Astron.Astrophys.Rev.",
        "inspire": "Astron.Astrophys.Rev."
      },
      {
        "cds": "Prikl.Mat.Mekh.",
        "inspire": "Prikl.Mat.Mekh."
      },
      {
        "cds": "Math. Methods Appl. Sci.",
        "inspire": "Math.Methods Appl.Sci."
      },
      {
        "cds": "J. Stat. Phys.",
        "inspire": "J.Statist.Phys."
      },
      {
        "cds": "Bull. Aust. Math. Soc.",
        "inspire": "Bull.Austral.Math.Soc."
      },
      {
        "cds": "BIT",
        "inspire": "Bit"
      },
      {
        "cds": "Sov.J.Nucl.Phys.",
        "inspire": "Sov.J.Nucl.Phys."
      },
      {
        "cds": "J. High Energy Phys.",
        "inspire": "JHEP"
      },
      {
        "cds": "Appl. Opt.",
        "inspire": "Appl.Opt."
      },
      {
        "cds": "Nuovo Cimento",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "Bulg. J. Phys.",
        "inspire": "Bulg.J.Phys."
      },
      {
        "cds": "Annals Glaciol.",
        "inspire": "Annals Glaciol."
      },
      {
        "cds": "J.Noncryst.Solids",
        "inspire": "J.Noncryst.Solids"
      },
      {
        "cds": "Radio Sci.",
        "inspire": "Radio Sci."
      },
      {
        "cds": "Prog. Theor. Phys., Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "Adv.Astronaut.Sci.",
        "inspire": "Adv.Astronaut.Sci."
      },
      {
        "cds": "Springer Ser.Opt.Sci.",
        "inspire": "Springer Ser.Opt.Sci."
      },
      {
        "cds": "J. Geophys. Res.",
        "inspire": "J.Geophys.Res."
      },
      {
        "cds": "Acta Phys. Austriaca",
        "inspire": "Acta Phys.Austriaca"
      },
      {
        "cds": "J. Biol. Phys.",
        "inspire": "J.Biol.Phys."
      },
      {
        "cds": "Funct. Anal. Appl.",
        "inspire": "Funct.Anal.Appl."
      },
      {
        "cds": "Philos. Mag. Lett.",
        "inspire": "Phil.Mag.Lett."
      },
      {
        "cds": "IEEE Trans. Electromagn. Compat.",
        "inspire": "IEEE Trans.Electromagn.Compat."
      },
      {
        "cds": "Tokyo J. Math.",
        "inspire": "Tokyo J.Math."
      },
      {
        "cds": "Prib. Tekh. Eksp.",
        "inspire": "Prib.Tekh.Eksp."
      },
      {
        "cds": "Econ.Geol.",
        "inspire": "Econ.Geol."
      },
      {
        "cds": "Phys. Rev.",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "Publ. Astron. Soc. Jpn.",
        "inspire": "Publ.Astron.Soc.Jap."
      },
      {
        "cds": "Part. World",
        "inspire": "Part.World"
      },
      {
        "cds": "Comput. Rev.",
        "inspire": "Comput.Rev."
      },
      {
        "cds": "Muon Cat.Fusion",
        "inspire": "Muon Cat.Fusion"
      },
      {
        "cds": "Phys. Lett.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Biochem.Biophys.Res.Commun.",
        "inspire": "Biochem.Biophys.Res.Commun."
      },
      {
        "cds": "Mater.Sci.Eng.",
        "inspire": "Mater.Sci.Technol."
      },
      {
        "cds": "Ann. Rev. Fluid. Mech.",
        "inspire": "Ann.Rev.Fluid Mech."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Rend. Circ. Mat. Palermo",
        "inspire": "Rend.Circ.Mat.Palermo"
      },
      {
        "cds": "Sov.J.Low.Tem.Phys.",
        "inspire": "Sov.J.Low.Tem.Phys."
      },
      {
        "cds": "Phys. Rep., C",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Comput. Sci. Eng.",
        "inspire": "Comput.Sci.Eng."
      },
      {
        "cds": "J. R. Stat. Soc.",
        "inspire": "J.Roy.Statist.Soc.B"
      },
      {
        "cds": "J. Exp. Theor. Phys.",
        "inspire": "Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Onde.Elec.",
        "inspire": "Onde.Elec."
      },
      {
        "cds": "J. Number Theory",
        "inspire": "J.Number Theor."
      },
      {
        "cds": "J. Phys. Soc. Jpn.",
        "inspire": "J.Phys.Soc.Jap."
      },
      {
        "cds": "Trans.Amer.Soc.Mech.Eng.Appl.Mech.",
        "inspire": "Trans.Amer.Soc.Mech.Eng.Appl.Mech."
      },
      {
        "cds": "Acta Phys. Pol.",
        "inspire": "Acta Phys.Polon."
      },
      {
        "cds": "New J. Phys.",
        "inspire": "New J.Phys."
      },
      {
        "cds": "Izv.Vuz.Fiz.",
        "inspire": "Izv.Vuz.Fiz."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Physiol. Rev.",
        "inspire": "Physiol.Rev."
      },
      {
        "cds": "Ettore Majorana Int. Sci. Ser., Phys. Sci.",
        "inspire": "Ettore Majorana Int.Sci.Ser.Phys.Sci."
      },
      {
        "cds": "Nagoya Math. J.",
        "inspire": "Nagoya Math.J."
      },
      {
        "cds": "Appl. Sci. Res.",
        "inspire": "Appl.Sci.Res."
      },
      {
        "cds": "Astronews.com",
        "inspire": "Astronews"
      },
      {
        "cds": "Proc. Am. Math. Soc.",
        "inspire": "Proc.Am.Math.Soc."
      },
      {
        "cds": "Proc. Lond. Math. Soc.",
        "inspire": "Proc.Lond.Math.Soc."
      },
      {
        "cds": "Inf. Syst.",
        "inspire": "Inf.Syst."
      },
      {
        "cds": "Biol. Cybern.",
        "inspire": "Biol.Cyber."
      },
      {
        "cds": "Zh.Tekh.Fiz.",
        "inspire": "Zh.Tekh.Fiz."
      },
      {
        "cds": "Rev. Roum. Phys.",
        "inspire": "Rom.J.Phys."
      },
      {
        "cds": "Phys. Essays",
        "inspire": "Phys.Essays"
      },
      {
        "cds": "Bull.Lebedev Phys.Inst.",
        "inspire": "Bull.Lebedev Phys.Inst."
      },
      {
        "cds": "EOS Trans.Am.Geophys.Union",
        "inspire": "EOS Trans.Am.Geophys.Union"
      },
      {
        "cds": "Kinet.Catal.",
        "inspire": "Kinet.Catal."
      },
      {
        "cds": "Adv. Appl. Math.",
        "inspire": "Adv.Appl.Math."
      },
      {
        "cds": "SIAM J. Contr. Optim.",
        "inspire": "SIAM J.Control Optim."
      },
      {
        "cds": "Russ.J.Math.Phys.",
        "inspire": "Russ.J.Math.Phys."
      },
      {
        "cds": "Bull.Astron.Inst.Czech.",
        "inspire": "Bull.Astron.Inst.Czech."
      },
      {
        "cds": "Probl. Inf. Transm.",
        "inspire": "Prob.Peredachi Info."
      },
      {
        "cds": "Angew. Chem.",
        "inspire": "Angew.Chem."
      },
      {
        "cds": "Astropart. Phys.",
        "inspire": "Astropart.Phys."
      },
      {
        "cds": "J. Radioanal. Nucl. Chem.",
        "inspire": "J.Radioanal.Nucl.Chem."
      },
      {
        "cds": "J. Exp. Theor. Phys.",
        "inspire": "Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Astrophys.J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Phys. Scr.",
        "inspire": "Phys.Scripta"
      },
      {
        "cds": "High Energy Phys. Nucl. Phys.",
        "inspire": "High Energy Phys.Nucl.Phys."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "J. Phys.",
        "inspire": "J.Phys.A"
      },
      {
        "cds": "Proc.Indian Acad.Sci.",
        "inspire": "Proc.Indian Acad.Sci."
      },
      {
        "cds": "J. Phys.",
        "inspire": "J.Phys.I(France)"
      },
      {
        "cds": "Bol.A.A.Astron.",
        "inspire": "Bol.A.A.Astron."
      },
      {
        "cds": "Mem. Am. Math. Soc.",
        "inspire": "Mem.Am.Math.Soc."
      },
      {
        "cds": "Wiss.Ber.AEG Telefunken",
        "inspire": "Wiss.Ber.AEG Telefunken"
      },
      {
        "cds": "Log Analyst",
        "inspire": "Log Analyst"
      },
      {
        "cds": "Math. Res. Lett.",
        "inspire": "Math.Res.Lett."
      },
      {
        "cds": "Acta Phys. Sin.",
        "inspire": "Acta Phys.Sin."
      },
      {
        "cds": "J.Theor.Comput.Chem.",
        "inspire": "J.Theor.Comput.Chem."
      },
      {
        "cds": "Phys. Part. Nucl.",
        "inspire": "Phys.Part.Nucl."
      },
      {
        "cds": "Proc. Inst. Electr. Eng.",
        "inspire": "IEEE Proc."
      },
      {
        "cds": "Prog. Quantum Electron.",
        "inspire": "Prog.Quant.Electron."
      },
      {
        "cds": "Phys. Part. Nucl. Lett.",
        "inspire": "Phys.Part.Nucl.Lett."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Nucl.Phys.",
        "inspire": "Nucl.Phys."
      },
      {
        "cds": "Prog.Elem.Part.Cosmic Ray Phys.",
        "inspire": "Prog.Elem.Part.Cosmic Ray Phys."
      },
      {
        "cds": "Z. Astrophys.",
        "inspire": "Z.Astrophys."
      },
      {
        "cds": "Comments Astrophys.",
        "inspire": "Comments Astrophys."
      },
      {
        "cds": "Math. Japon.",
        "inspire": "Math.Jap."
      },
      {
        "cds": "SIAM J. Discrete Math.",
        "inspire": "SIAM J.Discrete Math."
      },
      {
        "cds": "Solid State Commun.",
        "inspire": "Solid State Commun."
      },
      {
        "cds": "Sib. Math. J.",
        "inspire": "Sib.Math.J."
      },
      {
        "cds": "Mach. Learn.",
        "inspire": "Machine Learning"
      },
      {
        "cds": "Colloid Polym. Sci.",
        "inspire": "Colloid Polym.Sci."
      },
      {
        "cds": "Adv. Appl. Clifford Algebras",
        "inspire": "Adv.Appl.Clifford Algebras"
      },
      {
        "cds": "Isotopenpraxis",
        "inspire": "Isotopenpraxis"
      },
      {
        "cds": "Phys. Rev. Focus",
        "inspire": "Phys.Rev.Focus"
      },
      {
        "cds": "IEEE Trans. Electron. Comput.",
        "inspire": "IEEE Trans.Electron.Comput."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Rep. Prog. Phys.",
        "inspire": "Rept.Prog.Phys."
      },
      {
        "cds": "Adv. Nucl. Phys.",
        "inspire": "Adv.Nucl.Phys."
      },
      {
        "cds": "AIP Conf. Proc.",
        "inspire": "AIP Conf.Proc."
      },
      {
        "cds": "Int. J. Electron.",
        "inspire": "Int.J.Electron."
      },
      {
        "cds": "Commun. Nonlinear Sci. Numer. Simul.",
        "inspire": "Commun.Nonlinear Sci.Numer.Simul."
      },
      {
        "cds": "Acc. Chem. Res.",
        "inspire": "Accounts Chem.Res."
      },
      {
        "cds": "J.Mater.",
        "inspire": "J.Mater."
      },
      {
        "cds": "Kong.Dan.Vid.Sel.Mat.Fys.Med.",
        "inspire": "Kong.Dan.Vid.Sel.Mat.Fys.Med."
      },
      {
        "cds": "Fundam. Cosm. Phys.",
        "inspire": "Fund.Cosmic Phys."
      },
      {
        "cds": "Phys. Rep.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Phys. Rep.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Nature",
        "inspire": "Nature"
      },
      {
        "cds": "Z. Naturforsch.",
        "inspire": "Z.Naturforsch."
      },
      {
        "cds": "Russ. Math. Surv.",
        "inspire": "Russ.Math.Surveys"
      },
      {
        "cds": "Bull. Am. Astron. Soc.",
        "inspire": "Bull.Am.Astron.Soc."
      },
      {
        "cds": "Acta Phys. Pol.",
        "inspire": "Acta Phys.Polon."
      },
      {
        "cds": "Int. Astron. Union Symp.",
        "inspire": "IAU Symp."
      },
      {
        "cds": "Automatica",
        "inspire": "Automatica"
      },
      {
        "cds": "Trans. Camb. Philos. Soc.",
        "inspire": "Trans.Cambridge Phil.Soc."
      },
      {
        "cds": "Comput. Phys. Rep.",
        "inspire": "Comput.Phys.Rept."
      },
      {
        "cds": "J. Phys. III",
        "inspire": "J.Phys.III"
      },
      {
        "cds": "Duke Math. J.",
        "inspire": "Duke Math.J."
      },
      {
        "cds": "Z. Elektrochem.",
        "inspire": "Z.Elektrochem."
      },
      {
        "cds": "Publ. Astron. Soc. Pac.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "Acta Radiol.Ther.Phys.Biol.",
        "inspire": "Acta Radiol.Ther.Phys.Biol."
      },
      {
        "cds": "Sov.Math.Dokl.",
        "inspire": "Sov.Math.Dokl."
      },
      {
        "cds": "Tech. Phys.",
        "inspire": "Tech.Phys."
      },
      {
        "cds": "J. Comput. Phys.",
        "inspire": "J.Comput.Phys."
      },
      {
        "cds": "Bull. Am. Math. Soc.",
        "inspire": "Bull.Am.Math.Soc."
      },
      {
        "cds": "Uch.Zapiski Kazan Univ.",
        "inspire": "Uch.Zapiski Kazan Univ."
      },
      {
        "cds": "At. Energy Rev.",
        "inspire": "At.Energy Rev."
      },
      {
        "cds": "Sci. Am.",
        "inspire": "Sci.Am."
      },
      {
        "cds": "J.Multiv.Anal.",
        "inspire": "J.Multiv.Anal."
      },
      {
        "cds": "Astron. J.",
        "inspire": "Astron.J."
      },
      {
        "cds": "J. Geom. Phys.",
        "inspire": "J.Geom.Phys."
      },
      {
        "cds": "Proc.Roy.Soc.Lond.",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "Chin. J. Astron. Astrophys.",
        "inspire": "Chin.J.Astron.Astrophys."
      },
      {
        "cds": "Linear Algebra Appl.",
        "inspire": "Linear Algebra Appl."
      },
      {
        "cds": "J. Eur. Math. Soc.",
        "inspire": "J.Eur.Math.Soc"
      },
      {
        "cds": "Czechoslov. J. Phys.",
        "inspire": "Czech.J.Phys."
      },
      {
        "cds": "Int. J. Numer. Methods Fluids",
        "inspire": "Int. J. Numer. Methods Fluids"
      },
      {
        "cds": "J.Comput.Syst.Sci.",
        "inspire": "J.Comput.Syst.Sci."
      },
      {
        "cds": "Rev. Mod. Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "Fundam. Cosm. Phys.",
        "inspire": "Fund.Cosmic Phys."
      },
      {
        "cds": "Sci. Sinica",
        "inspire": "Sci.Sin."
      },
      {
        "cds": "Biotechnics",
        "inspire": "Biotechnics"
      },
      {
        "cds": "Rep. Math. Phys.",
        "inspire": "Rept.Math.Phys."
      },
      {
        "cds": "Curr. Opin. Colloid. Interface Sci.",
        "inspire": "Curr.Opin.Colloid Interface Sci."
      },
      {
        "cds": "J. Sci. Comput.",
        "inspire": "J.Sci.Comput."
      },
      {
        "cds": "Egypt. J. Solids",
        "inspire": "Egypt.J.Solids"
      },
      {
        "cds": "Mon.Not.Roy.Astron.Soc.",
        "inspire": "Mem.Roy.Astron.Soc."
      },
      {
        "cds": "Adv. Stud. Pure Math.",
        "inspire": "Adv.Stud.Pure Math."
      },
      {
        "cds": "J.Radioanal.Chem.",
        "inspire": "J.Radioanal.Chem."
      },
      {
        "cds": "Int. J. Mod. Phys. C",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Int. J. Mod. Phys. B",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Int. J. Mod. Phys. A",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Laser Phys.",
        "inspire": "Laser Phys."
      },
      {
        "cds": "Vestn.Mosk.Univ.Fiz.Astron.",
        "inspire": "Vestn.Mosk.Univ.Fiz.Astron."
      },
      {
        "cds": "Nuovo Cimento, Suppl.",
        "inspire": "Nuovo Cim.Suppl."
      },
      {
        "cds": "Math. Phys. Anal. Geom.",
        "inspire": "Math.Phys.Anal.Geom."
      },
      {
        "cds": "Genet.Res.",
        "inspire": "Genet.Res."
      },
      {
        "cds": "Numer. Algorithms",
        "inspire": "Num. Algorithms"
      },
      {
        "cds": "New Astron.",
        "inspire": "New Astron."
      },
      {
        "cds": "Maple Tech.Newslett.",
        "inspire": "Maple Tech.Newslett."
      },
      {
        "cds": "ESA Spec. Publ.",
        "inspire": "ESA Spec.Publ."
      },
      {
        "cds": "IEEE Trans. Neural Netw.",
        "inspire": "IEEE Trans.Neural Networks"
      },
      {
        "cds": "Rev.Union Mat.Argentina",
        "inspire": "Rev.Union Mat.Argentina"
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "New Astron. Rev.",
        "inspire": "New Astron.Rev."
      },
      {
        "cds": "Eur. J. Phys.",
        "inspire": "Eur.J.Phys."
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Annu. Rev. Astron. Astrophys.",
        "inspire": "Ann.Rev.Astron.Astrophys."
      },
      {
        "cds": "Clim.Change",
        "inspire": "Clim.Change"
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Dokl.Akad.Nauk Arm.SSR",
        "inspire": "Dokl.Akad.Nauk Arm.SSR"
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "ASME J. Basic Eng.",
        "inspire": "J.Basic Eng."
      },
      {
        "cds": "J.Comput.Assist.Tomogr.",
        "inspire": "J.Comput.Assist.Tomogr."
      },
      {
        "cds": "IEEE Trans. Audio Electroacoust.",
        "inspire": "IEEE Trans.Audio Electroacoust."
      },
      {
        "cds": "Trends Genet.",
        "inspire": "Trends Genet."
      },
      {
        "cds": "Berkeley Lab Res. Rev.",
        "inspire": "LBL Res. Rev."
      },
      {
        "cds": "Solar Energy Mater.Solar Cells",
        "inspire": "Solar Energy Mater.Solar Cells"
      },
      {
        "cds": "J.Indian Math.Soc.",
        "inspire": "J.Indian Math.Soc."
      },
      {
        "cds": "K-Theory",
        "inspire": "K Theory"
      },
      {
        "cds": "J. Opt. B",
        "inspire": "J.Opt.B Quant.Semiclass.Opt."
      },
      {
        "cds": "Electromagnetics",
        "inspire": "Electromagnetics"
      },
      {
        "cds": "Muon Cat.Fusion",
        "inspire": "Muon Cat.Fusion"
      },
      {
        "cds": "Publ. Res. Inst. Math. Sci.",
        "inspire": "Publ.Res.Inst.Math.Sci.Kyoto"
      },
      {
        "cds": "Astron.Astrophys.Suppl.Ser.",
        "inspire": "Astron.Astrophys.Trans."
      },
      {
        "cds": "Sov.J.Numer.Anal.Math.Modeling",
        "inspire": "Sov.J.Numer.Anal.Math.Modeling"
      },
      {
        "cds": "Lect. Notes Phys.",
        "inspire": "Lect.Notes Phys."
      },
      {
        "cds": "Int. J. Mod. Phys.",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "JHEP",
        "inspire": "JHEP"
      },
      {
        "cds": "Genshikaku-Kenkyu",
        "inspire": "Genshikaku Kenkyu"
      },
      {
        "cds": "Z. Naturforsch.",
        "inspire": "Z.Naturforsch."
      },
      {
        "cds": "Isr. J. Math.",
        "inspire": "Isr.J.Math."
      },
      {
        "cds": "Med.Radiol.",
        "inspire": "Med.Radiol."
      },
      {
        "cds": "Sov. Sci. Rev., A",
        "inspire": "Sov.Sci.Rev."
      },
      {
        "cds": "J. Lightwave Technol.",
        "inspire": "J.Lightwave Tech."
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc. Lett.",
        "inspire": "Mon.Not.Roy.Astron.Soc.Lett."
      },
      {
        "cds": "Rend. Sem. Mat. Univ. Polit. Torino",
        "inspire": "Rend.Sem.Mat.Univ.Politec.Torino"
      },
      {
        "cds": "Phys. Part. Nucl.",
        "inspire": "Phys.Part.Nucl."
      },
      {
        "cds": "Selecta Math.",
        "inspire": "Selecta Math."
      },
      {
        "cds": "Theor.Probab.Appl.",
        "inspire": "Theor.Probab.Appl."
      },
      {
        "cds": "Recent Res. Dev. Astron. Astrophys.",
        "inspire": "Recent Res.Dev.Astron.Astrophys."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Int. J. Neural Syst.",
        "inspire": "Int.J.Neural Syst."
      },
      {
        "cds": "Comments Condens. Matter Phys.",
        "inspire": "Comments Condens.Matter Phys."
      },
      {
        "cds": "Springer Tracts Mod. Phys.",
        "inspire": "Springer Tracts Mod.Phys."
      },
      {
        "cds": "Proc. R. Soc. Lond.",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "Theor. Comput. Fluid Dyn.",
        "inspire": "Theor.Comput.Fluid Dyn."
      },
      {
        "cds": "Phys. Scr.",
        "inspire": "Phys.Scripta"
      },
      {
        "cds": "Phys. At. Nucl.",
        "inspire": "Phys.Atom.Nucl."
      },
      {
        "cds": "Mater. Chem. Phys.",
        "inspire": "Mater.Chem.Phys."
      },
      {
        "cds": "Publ. Mat. Debrecen",
        "inspire": "Publ.Math.Debrecen"
      },
      {
        "cds": "Physica",
        "inspire": "Physica"
      },
      {
        "cds": "Info.Control",
        "inspire": "Info.Control"
      },
      {
        "cds": "Adv. Phys.",
        "inspire": "Adv.Phys."
      },
      {
        "cds": "Proc. R. Irish Acad.",
        "inspire": "Proc.Roy.Irish Acad.(Sect.A)"
      },
      {
        "cds": "Ukr.Fiz.Zh.(Ukr.Ed.)",
        "inspire": "Ukr.Fiz.Zh.(Ukr.Ed.)"
      },
      {
        "cds": "Quant.Fractals Complex Syst.",
        "inspire": "Quant.Fractals Complex Syst."
      },
      {
        "cds": "Phys. Lett.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Tôhoku Math. J.",
        "inspire": "Tohoku Math.J."
      },
      {
        "cds": "K-Theory",
        "inspire": "K Theory"
      },
      {
        "cds": "J. Chem. Phys.",
        "inspire": "J.Chem.Phys."
      },
      {
        "cds": "J. Appl. Crystallogr.",
        "inspire": "J.Appl.Cryst."
      },
      {
        "cds": "Astrophysics",
        "inspire": "Astrophysics"
      },
      {
        "cds": "Eur. J. Solid State Inorg. Chem.",
        "inspire": "Eur.J.Solid State Inorg.Chem."
      },
      {
        "cds": "Proc. SPIE",
        "inspire": "Proc.SPIE Int.Soc.Opt.Eng."
      },
      {
        "cds": "Dokl. Akad. Nauk SSSR",
        "inspire": "Dokl.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Neuron",
        "inspire": "Neuron"
      },
      {
        "cds": "Trud.Mosk.Mat.Obshch.",
        "inspire": "Trud.Mosk.Mat.Obshch."
      },
      {
        "cds": "J. Mol. Spectrosc.",
        "inspire": "J.Molec.Spectrosc."
      },
      {
        "cds": "Fusion Technol.",
        "inspire": "Fusion Tech."
      },
      {
        "cds": "Annu. Rev. Nucl. Part. Sci.",
        "inspire": "Complete"
      },
      {
        "cds": "Z.Anal.Anw.",
        "inspire": "Z.Anal.Anw."
      },
      {
        "cds": "Trans.Metall.Soc.AIME",
        "inspire": "Trans.Metall.Soc.AIME"
      },
      {
        "cds": "Am. J. Phys.",
        "inspire": "Am.J.Phys."
      },
      {
        "cds": "Ann. Fis.",
        "inspire": "Anales Fis."
      },
      {
        "cds": "Int. J. Mod. Phys. C",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Int. J. Mod. Phys. B",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Bell Syst. Tech. J.",
        "inspire": "Bell Syst.Tech.J."
      },
      {
        "cds": "Adv. Electron. Electron Phys.",
        "inspire": "Adv.Electron.Electron Phys."
      },
      {
        "cds": "Theor. Math. Phys.",
        "inspire": "Theor.Math.Phys."
      },
      {
        "cds": "JETP Lett.",
        "inspire": "JETP Lett."
      },
      {
        "cds": "Brown Boveri Mitt.",
        "inspire": "Brown Boveri Mitt."
      },
      {
        "cds": "Comput. Commun. Rev.",
        "inspire": "Comput.Commun.Rev."
      },
      {
        "cds": "BYTE",
        "inspire": "Byte"
      },
      {
        "cds": "Publ.Astron.Soc.Pac.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "IEEE Spectr.",
        "inspire": "IEEE Spectrum"
      },
      {
        "cds": "Houston J.Math.",
        "inspire": "Houston J.Math."
      },
      {
        "cds": "Diam. Relat. Mater.",
        "inspire": "Diam.Rel.Mater."
      },
      {
        "cds": "J. Phys. Soc. Jpn.",
        "inspire": "J.Phys.Soc.Jap."
      },
      {
        "cds": "Mod.Phys.Lett.",
        "inspire": "Mod.Phys.Lett."
      },
      {
        "cds": "Acta Astron.",
        "inspire": "Acta Astron."
      },
      {
        "cds": "Chin.Sci.Bull.",
        "inspire": "Chin.Sci.Bull."
      },
      {
        "cds": "Izv.Akad.Nauk SSSR (Neorg.Mater.)",
        "inspire": "Izv.Akad.Nauk SSSR (Neorg.Mater.)"
      },
      {
        "cds": "Radiokhimiya",
        "inspire": "Radiokhimiya"
      },
      {
        "cds": "Bull. Astron. Soc. India",
        "inspire": "Bull.Astron.Soc.India"
      },
      {
        "cds": "Sci. Am.",
        "inspire": "Sci.Am."
      },
      {
        "cds": "Jpn. J. Math.",
        "inspire": "Jap.J.Math."
      },
      {
        "cds": "Q.Appl.Math.",
        "inspire": "Q.Appl.Math."
      },
      {
        "cds": "Comput. Graph.",
        "inspire": "Comput.Graphics"
      },
      {
        "cds": "Lett.Nuovo Cim.",
        "inspire": "Lett.Nuovo Cim."
      },
      {
        "cds": "Philos. Sci.",
        "inspire": "Phil.Sci."
      },
      {
        "cds": "Nucl. Phys. News",
        "inspire": "Nucl.Phys.News"
      },
      {
        "cds": "Theor. Comput. Sci.",
        "inspire": "Theor.Comput.Sci."
      },
      {
        "cds": "J.Radiat.Res.",
        "inspire": "J.Radiat.Res."
      },
      {
        "cds": "Acta Photonica Sin.",
        "inspire": "Acta Photonica Sin."
      },
      {
        "cds": "Kybernetika",
        "inspire": "Kybernetika"
      },
      {
        "cds": "J. Appl. Crystallogr.",
        "inspire": "J.Appl.Cryst."
      },
      {
        "cds": "Kristallografiya",
        "inspire": "Kristallografiya"
      },
      {
        "cds": "Sov. Phys. Acoust.",
        "inspire": "Sov.Phys.-Acoust."
      },
      {
        "cds": "Nuovo Cimento, Suppl.",
        "inspire": "Nuovo Cim.Suppl."
      },
      {
        "cds": "Appl. Math.",
        "inspire": "Applications Math."
      },
      {
        "cds": "J. Theor. Biol.",
        "inspire": "J.Theor.Biol."
      },
      {
        "cds": "Gemini",
        "inspire": "Gemini"
      },
      {
        "cds": "Nature",
        "inspire": "Nature"
      },
      {
        "cds": "Neural Comput.",
        "inspire": "Neural Comput."
      },
      {
        "cds": "Am. Sci.",
        "inspire": "Am.Sci."
      },
      {
        "cds": "Acustika",
        "inspire": "Acustica"
      },
      {
        "cds": "Philos. Trans. R. Soc. Lond.",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "SIAM J. Numer. Anal.",
        "inspire": "SIAM J.Numer.Anal."
      },
      {
        "cds": "Publ. Math. IHES",
        "inspire": "Inst.Hautes Etudes Sci.Publ.Math."
      },
      {
        "cds": "Monatsber.Deutsch.Akad.Wiss.Berlin",
        "inspire": "Monatsber.Deutsch.Akad.Wiss.Berlin"
      },
      {
        "cds": "Proc. R. Soc. Edinburgh",
        "inspire": "Proc.Roy.Soc.Edinburgh"
      },
      {
        "cds": "Ergod.Theor.Dynam.Syst.",
        "inspire": "Ergod.Theor.Dynam.Syst."
      },
      {
        "cds": "Am. Math. Mon.",
        "inspire": "Am.Math.Mon."
      },
      {
        "cds": "Comput.Geosci.",
        "inspire": "Comput.Geosci."
      },
      {
        "cds": "J. Sci. Instrum.",
        "inspire": "J.Sci.Instrum."
      },
      {
        "cds": "Comments At. Mol. Phys.",
        "inspire": "Comments Atom.Mol.Phys."
      },
      {
        "cds": "Compt.Rend.Math.",
        "inspire": "Compt.Rend.Math."
      },
      {
        "cds": "Astron. Nachr.",
        "inspire": "Astron.Nachr."
      },
      {
        "cds": "J. Exp. Theor. Phys.",
        "inspire": "Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Phys. Lett.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Ann. Inst. Fourier",
        "inspire": "Annales Inst.Fourier"
      },
      {
        "cds": "J. Geom. Phys.",
        "inspire": "J.Geom.Phys."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Acta Astron.",
        "inspire": "Acta Astron."
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "Stat. Sci.",
        "inspire": "Statist.Sci."
      },
      {
        "cds": "PiN Newslett.",
        "inspire": "PiN Newslett."
      },
      {
        "cds": "Solar Phys.",
        "inspire": "Solar Phys."
      },
      {
        "cds": "Ann. Math.",
        "inspire": "Annals Math."
      },
      {
        "cds": "Sov. Sci. Rev. A Phys.",
        "inspire": "Sov.Sci.Rev."
      },
      {
        "cds": "Adv. Appl. Math.",
        "inspire": "Adv.Appl.Math."
      },
      {
        "cds": "Econometrica",
        "inspire": "Econometrica"
      },
      {
        "cds": "J.Phys.G",
        "inspire": "J.Phys.G"
      },
      {
        "cds": "Discrete Math.",
        "inspire": "Discrete Mathematics"
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "Opt. Spektrosk.",
        "inspire": "Opt.Spektrosk."
      },
      {
        "cds": "Proc. Astron. Soc. Jpn.",
        "inspire": "Publ.Astron.Soc.Jap."
      },
      {
        "cds": "Funct. Anal. Appl.",
        "inspire": "Funkt.Anal.Pril."
      },
      {
        "cds": "Mater. Lett.",
        "inspire": "Mater.Lett."
      },
      {
        "cds": "Prog.Theor.Phys.Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "Prog.Theor.Phys.Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "Supercond. Sci. Technol.",
        "inspire": "Supercond.Sci.Technol."
      },
      {
        "cds": "Theor. Comput. Fluid Dyn.",
        "inspire": "Theor.Comput.Fluid Dyn."
      },
      {
        "cds": "Ann. Inst. Henri Poincaré, Probab. Stat.",
        "inspire": "Ann.Inst.H.Poincare Probab.Statist."
      },
      {
        "cds": "CMS Bull.",
        "inspire": "CMS Bull."
      },
      {
        "cds": "Astrophys. Lett.",
        "inspire": "Astrophys.Lett."
      },
      {
        "cds": "Amer.Miner.",
        "inspire": "Amer.Miner."
      },
      {
        "cds": "Polyhedron",
        "inspire": "Polyhedron"
      },
      {
        "cds": "Nieuw Arch.Wis.",
        "inspire": "Nieuw Arch.Wis."
      },
      {
        "cds": "J.Meca.Theor.Appl.",
        "inspire": "J.Meca.Theor.Appl."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Phys. Rev. Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "Int. J. Evol.",
        "inspire": "Evolution"
      },
      {
        "cds": "Alg.Anal.",
        "inspire": "Alg.Anal."
      },
      {
        "cds": "Phys. Teach.",
        "inspire": "Phys.Teacher"
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "J.Jap.Assoc.Philos.Sci.",
        "inspire": "J.Jap.Assoc.Philos.Sci."
      },
      {
        "cds": "Prog. Mater. Sci.",
        "inspire": "Prog.Mater.Sci."
      },
      {
        "cds": "Izv. Ran., Ser. Fiz.",
        "inspire": "Izv.Ross.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Bull. Belg. Math. Soc.",
        "inspire": "Bull.Soc.Math.Belg."
      },
      {
        "cds": "Lett.Nuovo Cim.",
        "inspire": "Lett.Nuovo Cim."
      },
      {
        "cds": "Probab. Surv.",
        "inspire": "Probab.Surv."
      },
      {
        "cds": "Ann. Geophys.",
        "inspire": "Ann.Geophys."
      },
      {
        "cds": "Publ.Astron.Obs.Belgrade",
        "inspire": "Publ.Astron.Obs.Belgrade"
      },
      {
        "cds": "Appl. Opt.",
        "inspire": "Appl.Opt."
      },
      {
        "cds": "Radiat. Res.",
        "inspire": "Rad.Res."
      },
      {
        "cds": "ChemPhysChem",
        "inspire": "ChemPhysChem"
      },
      {
        "cds": "Discrete Math.",
        "inspire": "Discrete Mathematics"
      },
      {
        "cds": "Synth. Met.",
        "inspire": "Synth.Metals"
      },
      {
        "cds": "Elektroteknikeren",
        "inspire": "Elektroteknikeren"
      },
      {
        "cds": "Adv. Sov. Math.",
        "inspire": "Adv.Sov.Math."
      },
      {
        "cds": "IBM J. Res. Dev.",
        "inspire": "IBM J.Res.Dev."
      },
      {
        "cds": "Probl. Inf. Transm.",
        "inspire": "Probl.Info.Transm."
      },
      {
        "cds": "Found. Phys. Lett.",
        "inspire": "Found.Phys.Lett."
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "J.Optim.Th.Appl.",
        "inspire": "J.Optim.Th.Appl."
      },
      {
        "cds": "Geophys. Astrophys. Fluid Dyn.",
        "inspire": "Geophys.Astrophys.Fluid Dynamics"
      },
      {
        "cds": "Scand.J.Stat.",
        "inspire": "Scand.J.Stat."
      },
      {
        "cds": "Phys. Status Solidi",
        "inspire": "Phys.Status Solidi"
      },
      {
        "cds": "Commun.Sternberg Astron.Inst.",
        "inspire": "Commun.Sternberg Astron.Inst."
      },
      {
        "cds": "Acoust.Phys.",
        "inspire": "Acoust.Phys."
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "J. Res. Natl. Inst. Stand. Technol.",
        "inspire": "J.Res.Natl.Bur.Stand."
      },
      {
        "cds": "Trans.Inst.Phys. Estonian Acad.Sci.",
        "inspire": "Trans.Inst.Phys. Estonian Acad.Sci."
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "J. Mol. Spectrosc.",
        "inspire": "J.Molec.Spectrosc."
      },
      {
        "cds": "Cell",
        "inspire": "Cell"
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "Stud.Math.",
        "inspire": "Stud.Math."
      },
      {
        "cds": "Annu. Rev. Astron. Astrophys.",
        "inspire": "Ann.Rev.Astron.Astrophys."
      },
      {
        "cds": "SIAM Rev.",
        "inspire": "SIAM Rev."
      },
      {
        "cds": "Funkt.Anal.Pril.",
        "inspire": "Funkt.Anal.Pril."
      },
      {
        "cds": "Bull. Am. Astron. Soc.",
        "inspire": "Bull.Am.Astron.Soc."
      },
      {
        "cds": "Acta Phys. Austriaca",
        "inspire": "Acta Phys.Austriaca"
      },
      {
        "cds": "Nauch.Dokl.Vyssh.Shk.Fiz.Mat.Nauk",
        "inspire": "Nauch.Dokl.Vyssh.Shk.Fiz.Mat.Nauk"
      },
      {
        "cds": "J.Appl.Meteor.",
        "inspire": "J.Appl.Meteor."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Isr. J. Chem.",
        "inspire": "Israel J.Chem."
      },
      {
        "cds": "J. Magn. Magn. Mater.",
        "inspire": "J.Magn.Magn.Mater."
      },
      {
        "cds": "J.Br.Interplanet.Soc.",
        "inspire": "J.Br.Interplanet.Soc."
      },
      {
        "cds": "J. Differ. Geom.",
        "inspire": "J.Diff.Geom."
      },
      {
        "cds": "Acta Astrophys.Sin.",
        "inspire": "Acta Astrophys.Sin."
      },
      {
        "cds": "Geophys. Res. Lett.",
        "inspire": "Geophys.Res.Lett."
      },
      {
        "cds": "Math.USSR Sb.",
        "inspire": "Math.USSR Sb."
      },
      {
        "cds": "Can. J. Phys.",
        "inspire": "Can.J.Phys."
      },
      {
        "cds": "Microw. Opt. Technol. Lett.",
        "inspire": "Microw.Opt.Technol.Lett."
      },
      {
        "cds": "SIGSAM Bull.",
        "inspire": "Sigsam Bull."
      },
      {
        "cds": "Complex Variables Theor.Appl.",
        "inspire": "Complex Variables Theor.Appl."
      },
      {
        "cds": "Z.Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Synth. Met.",
        "inspire": "Synth.Metals"
      },
      {
        "cds": "Annals Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Phys. Rep.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "IEEE Trans. Comput.",
        "inspire": "IEEE Trans.Comput."
      },
      {
        "cds": "J. Mol. Evol.",
        "inspire": "J.Molec.Evol."
      },
      {
        "cds": "J. Business",
        "inspire": "J.Business"
      },
      {
        "cds": "ACM Trans. Math. Softw.",
        "inspire": "ACM Trans.Math.Software"
      },
      {
        "cds": "Arabian.J.Sci.Eng.",
        "inspire": "Arabian.J.Sci.Eng."
      },
      {
        "cds": "Austral.J.Phys.",
        "inspire": "Austral.J.Phys."
      },
      {
        "cds": "Acta Phys. Slovaca",
        "inspire": "Acta Phys.Slov."
      },
      {
        "cds": "J. Symb. Log.",
        "inspire": "J.Symbolic Logic"
      },
      {
        "cds": "Ann. Appl. Probab.",
        "inspire": "Annals Appl.Probab."
      },
      {
        "cds": "Am. Math. Soc. Transl.",
        "inspire": "Am.Math.Soc.Transl."
      },
      {
        "cds": "Adv. Space Res.",
        "inspire": "Adv.Space Res."
      },
      {
        "cds": "J. Phys. A",
        "inspire": "J.Phys.A"
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Geom. Funct. Anal.",
        "inspire": "Geom.Funct.Anal."
      },
      {
        "cds": "CERN Cour.",
        "inspire": "CERN Cour."
      },
      {
        "cds": "Adv. Theor. Math. Phys.",
        "inspire": "Adv.Theor.Math.Phys."
      },
      {
        "cds": "Rev. Mod. Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "Supercomputer",
        "inspire": "Supercomputer"
      },
      {
        "cds": "J. Appl. Phys.",
        "inspire": "J.Appl.Phys."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Proc. R. Soc. Lond., A",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "Itogi Nauki Tekhn.Sovrem.Problemy Matem.",
        "inspire": "Itogi Nauki Tekhn.Sovrem.Problemy Matem."
      },
      {
        "cds": "Acta Phys. Pol.",
        "inspire": "Acta Phys.Polon."
      },
      {
        "cds": "J.Molec.Struc.",
        "inspire": "J.Molec.Struc."
      },
      {
        "cds": "Adv.Ser.Direct.High Energy Phys.",
        "inspire": "Adv.Ser.Direct.High Energy Phys."
      },
      {
        "cds": "Astrobiology",
        "inspire": "Astrobiology"
      },
      {
        "cds": "Fields Inst. Commun.",
        "inspire": "Fields Inst.Commun."
      },
      {
        "cds": "SIAM Rev.",
        "inspire": "SIAM Rev."
      },
      {
        "cds": "IEEE Trans. Parallel Distrib. Syst.",
        "inspire": "IEEE Trans.Parallel Distrib.Syst."
      },
      {
        "cds": "Math. Phys. Electron. J.",
        "inspire": "Math.Phys.Electron.J."
      },
      {
        "cds": "Phys. Scr.",
        "inspire": "Phys.Scripta"
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Phys.Rept.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Boll.Soc.Ital.Fis.Nuova Ser.",
        "inspire": "Boll.Soc.Ital.Fis.Nuova Ser."
      },
      {
        "cds": "Zh.Tekh.Fiz.",
        "inspire": "Zh.Tekh.Fiz."
      },
      {
        "cds": "Electron. J. Comb.",
        "inspire": "Electron. J. Comb."
      },
      {
        "cds": "Phys. Bull.",
        "inspire": "Phys.Bull."
      },
      {
        "cds": "J. Inorg. Nucl. Chem.",
        "inspire": "J.Inorg.Nucl.Chem."
      },
      {
        "cds": "Biol. Cybern.",
        "inspire": "Biol.Cyber."
      },
      {
        "cds": "Potential Anal.",
        "inspire": "Potential Anal."
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Trans.Roy.Soc.Edin.",
        "inspire": "Trans.Roy.Soc.Edin."
      },
      {
        "cds": "Int. J. Mod. Phys.",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Sov. Phys. - Tech. Phys.",
        "inspire": "Tech.Phys."
      },
      {
        "cds": "Adv.Atom.Molec.Phys.",
        "inspire": "Adv.Atom.Molec.Phys."
      },
      {
        "cds": "Prog. Elem. Part. Cosm. Ray Phys.",
        "inspire": "Prog.Elem.Part.Cosmic Ray Phys."
      },
      {
        "cds": "IEEE Trans. Image Process.",
        "inspire": "IEEE Trans.Image Process."
      },
      {
        "cds": "Rev. Mod. Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "Am. J. Math.",
        "inspire": "Am.J.Math."
      },
      {
        "cds": "ASTRO-PH",
        "inspire": "ASTRO-PH"
      },
      {
        "cds": "Optics News",
        "inspire": "Optics News"
      },
      {
        "cds": "Quart. J. Pure Appl. Math.",
        "inspire": "Pure Appl.Math."
      },
      {
        "cds": "Inverse Probl.",
        "inspire": "Inverse Prob."
      },
      {
        "cds": "Opt. Lett.",
        "inspire": "Opt.Lett."
      },
      {
        "cds": "Adv. Nucl. Phys.",
        "inspire": "Adv.Nucl.Phys."
      },
      {
        "cds": "Naturwissenschaften",
        "inspire": "Naturwiss."
      },
      {
        "cds": "Fibonacci Quart.",
        "inspire": "Fibonacci Q."
      },
      {
        "cds": "Ann.Rev.Earth Planet.Sci.",
        "inspire": "Ann.Rev.Earth Planet.Sci."
      },
      {
        "cds": "Rev. Mod. Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Sov. Phys. - Uspekhi",
        "inspire": "Sov.Phys.Usp."
      },
      {
        "cds": "Int. Math. Res. Not.",
        "inspire": "Int.Math.Res.Not."
      },
      {
        "cds": "Electron.Design",
        "inspire": "Electron.Design"
      },
      {
        "cds": "Rev.Mex.Astron.Astrofis.",
        "inspire": "Rev.Mex.Astron.Astrofis."
      },
      {
        "cds": "Uspekhi Fiz. Nauk",
        "inspire": "Usp.Fiz.Nauk"
      },
      {
        "cds": "Phys. Rev. D",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "J. Inst. Math. Appl.",
        "inspire": "J.Inst.Math.Appl."
      },
      {
        "cds": "J. Korean Astron. Soc.",
        "inspire": "J.Korean Astron.Soc."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Mem. Konan Univ., Sci. Ser.",
        "inspire": "Mem. Konan Univ., Sci. Ser."
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "D Lib Mag.",
        "inspire": "D Lib Magazine"
      },
      {
        "cds": "Arch. Sci.",
        "inspire": "Arch.Sci.Phys.Nat."
      },
      {
        "cds": "AIP Conf. Proc.",
        "inspire": "AIP Conf.Proc."
      },
      {
        "cds": "Adv. Appl. Mech.",
        "inspire": "Adv.Appl.Mech."
      },
      {
        "cds": "Comments Nucl. Part. Phys.",
        "inspire": "Comments Nucl.Part.Phys."
      },
      {
        "cds": "Nucleic Acids Res.",
        "inspire": "Nucleic Acids Res."
      },
      {
        "cds": "Adv. Appl. Probab.",
        "inspire": "Adv.Appl.Probab."
      },
      {
        "cds": "Bull.Allahabad Math.Soc.",
        "inspire": "Bull. Allahabad Math. Soc."
      },
      {
        "cds": "Nucl. Phys.",
        "inspire": "Nucl.Phys."
      },
      {
        "cds": "Int.J.Appl.Radiat.Isotopes",
        "inspire": "Int.J.Appl.Radiat.Isotopes"
      },
      {
        "cds": "Am. Math. Mon.",
        "inspire": "Am.Math.Mon."
      },
      {
        "cds": "Elektronik",
        "inspire": "Elektronik"
      },
      {
        "cds": "Denkschr. Schweiz. Naturforsch. Ges.",
        "inspire": "Denkschr. Schweiz. Naturforsch. Ges."
      },
      {
        "cds": "Europhys. Lett.",
        "inspire": "Europhys.Lett."
      },
      {
        "cds": "Izmer. Tekh.",
        "inspire": "Izmer. Tekh."
      },
      {
        "cds": "Summa Bras.Math.",
        "inspire": "Summa Bras.Math."
      },
      {
        "cds": "Phys. Rev.",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "Rev.Mod.Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "J. Chem. Phys.",
        "inspire": "J.Chem.Phys."
      },
      {
        "cds": "Physicalia",
        "inspire": "Physicalia Mag."
      },
      {
        "cds": "Ann. Nucl. Sci. Eng.",
        "inspire": "Ann.Nucl.Sci.Eng."
      },
      {
        "cds": "Dokl. Akad. Nauk SSSR",
        "inspire": "Dokl.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "J. Am. Stat. Assoc.",
        "inspire": "J.Am.Statist.Assoc."
      },
      {
        "cds": "Meas. Tech.",
        "inspire": "Meas.Tech."
      },
      {
        "cds": "Wu Li Hsueh Pao",
        "inspire": "Wu Li Hsueh Pao"
      },
      {
        "cds": "Kexue Zazhi",
        "inspire": "Kexue Zazhi"
      },
      {
        "cds": "Can. J. Chem.",
        "inspire": "Can.J.Chem."
      },
      {
        "cds": "Comput. Stand. Interfaces",
        "inspire": "Comput.Stand.Interfaces"
      },
      {
        "cds": "J.Approx.Theor.",
        "inspire": "J.Approx.Theor."
      },
      {
        "cds": "Differ. Geom. Appl.",
        "inspire": "Differ.Geom.Appl."
      },
      {
        "cds": "Solid State Phys.",
        "inspire": "Solid State Phys."
      },
      {
        "cds": "Chem. Phys.",
        "inspire": "Chem.Phys."
      },
      {
        "cds": "Fortschr. Phys.",
        "inspire": "Fortsch.Phys."
      },
      {
        "cds": "Technometrics",
        "inspire": "Technometrics"
      },
      {
        "cds": "Bull.Lebedev Phys.Inst.",
        "inspire": "Bull.Lebedev Phys.Inst."
      },
      {
        "cds": "Few-Body Syst., Suppl.",
        "inspire": "Few Body Syst.Suppl."
      },
      {
        "cds": "Duke Math. J.",
        "inspire": "Duke Math.J."
      },
      {
        "cds": "Commun. ACM",
        "inspire": "Commun.ACM"
      },
      {
        "cds": "J.Radio Res.Lab.",
        "inspire": "J.Radio Res.Lab."
      },
      {
        "cds": "J.Res.Natl.Bur.Stand.",
        "inspire": "J.Res.Natl.Bur.Stand."
      },
      {
        "cds": "Math. Res. Lett.",
        "inspire": "Math.Res.Lett."
      },
      {
        "cds": "J.Info.Proc.",
        "inspire": "J.Info.Proc."
      },
      {
        "cds": "Australasian J.Philos.",
        "inspire": "Australasian J.Philos."
      },
      {
        "cds": "Astronomy",
        "inspire": "Astronomy"
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "J. Am. Soc. Inf. Sci.",
        "inspire": "J.Am.Soc.Inf.Sci."
      },
      {
        "cds": "Pramana - J. Phys.",
        "inspire": "Pramana"
      },
      {
        "cds": "J. Comput. Neurosci.",
        "inspire": "J.Comput.Neurosci."
      },
      {
        "cds": "Proc. Phys. Soc. Lond., A",
        "inspire": "Proc.Phys.Soc."
      },
      {
        "cds": "Hyperfine Interact.",
        "inspire": "Hyperfine Interact."
      },
      {
        "cds": "Phys. At. Nucl.",
        "inspire": "Phys.Atom.Nucl."
      },
      {
        "cds": "Publ. Astron. Soc. Aust.",
        "inspire": "Proc.Astron.Soc.Austral."
      },
      {
        "cds": "Sov. J. Nucl. Phys.",
        "inspire": "Sov.J.Nucl.Phys."
      },
      {
        "cds": "J.Glaciol.",
        "inspire": "J.Glaciol."
      },
      {
        "cds": "Math. USSR Izvestiya",
        "inspire": "Math.USSR Izv."
      },
      {
        "cds": "Phys.Chem.Chem.Phys.",
        "inspire": "Phys.Chem.Chem.Phys."
      },
      {
        "cds": "Science",
        "inspire": "Science"
      },
      {
        "cds": "J. Geom. Anal.",
        "inspire": "J.Geom.Anal."
      },
      {
        "cds": "Publ. Astron. Soc. Pac.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "Int. J. Mod. Phys. B",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Int. J. Mod. Phys. A",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Lett. Math. Phys.",
        "inspire": "Lett.Math.Phys."
      },
      {
        "cds": "Int. J. Mod. Phys. D",
        "inspire": "Int.J.Mass Spectr.Ion Process."
      },
      {
        "cds": "Rev. Mod. Astron.",
        "inspire": "Rev.Mod.Astron."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Br. J. Philos. Sci.",
        "inspire": "Brit.J.Phil.Sci."
      },
      {
        "cds": "J.Ration.Mech.Anal.",
        "inspire": "J.Ration.Mech.Anal."
      },
      {
        "cds": "Eur. Phys. J.",
        "inspire": "Eur.Phys.J."
      },
      {
        "cds": "J. Phys. Chem. Ref. Data",
        "inspire": "J.Phys.Chem.Ref.Data"
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "Magn. Reson. Med.",
        "inspire": "Magn.Reson.Med."
      },
      {
        "cds": "J. Comput. Chem.",
        "inspire": "J.Comput.Chem."
      },
      {
        "cds": "Int. Rev. Phys. Chem.",
        "inspire": "Int.Rev.Phys.Chem."
      },
      {
        "cds": "J.Anal.Math.",
        "inspire": "J.Anal.Math."
      },
      {
        "cds": "Proc. K. Ned. Akad. Wet.",
        "inspire": "Kon.Ned.Akad.Wetensch.Proc."
      },
      {
        "cds": "Few-Body Syst.",
        "inspire": "Few Body Syst."
      },
      {
        "cds": "COND-MAT",
        "inspire": "COND-MAT"
      },
      {
        "cds": "Invest.Radiol.",
        "inspire": "Invest.Radiol."
      },
      {
        "cds": "Sov.Phys.J.",
        "inspire": "Russ.Phys.J"
      },
      {
        "cds": "Southeast Asian Bull. Math.",
        "inspire": "Southeast Asian Bull.Math."
      },
      {
        "cds": "Trans.Am.Soc.Mech.Eng.",
        "inspire": "Trans.Am.Soc.Mech.Eng."
      },
      {
        "cds": "Int. J. Mod. Phys.",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Lect. Notes Math.",
        "inspire": "Lect.Notes Math."
      },
      {
        "cds": "St. Petersburg Math. J.",
        "inspire": "St.Petersburg Math.J."
      },
      {
        "cds": "Appl. Opt.",
        "inspire": "Appl.Opt."
      },
      {
        "cds": "Proc. Combust. Inst.",
        "inspire": "Proc.Combust.Inst."
      },
      {
        "cds": "Moscow Univ.Math.Bull.",
        "inspire": "Moscow Univ.Math.Bull."
      },
      {
        "cds": "Sov. Phys. - JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Philos. Trans. R. Soc.",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "Dokl.Akad.Nauk Uzb.SSR",
        "inspire": "Dokl.Akad.Nauk Uzb.SSR"
      },
      {
        "cds": "Astron. Nachr.",
        "inspire": "Astron.Nachr."
      },
      {
        "cds": "Publ. Astron. Soc. Pac.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "Mitt. Astron. Ges.",
        "inspire": "Mitt. Astron. Ges."
      },
      {
        "cds": "J. Less Common Met.",
        "inspire": "J.Less Common Met."
      },
      {
        "cds": "Adv. Math.",
        "inspire": "Adv.Math."
      },
      {
        "cds": "J.Automat.Reas.",
        "inspire": "J.Automat.Reas."
      },
      {
        "cds": "Int. J. Numer. Anal. Methods Geomech.",
        "inspire": "Int.J.Numer.Anal.Methods Geomech."
      },
      {
        "cds": "Adv. Space Res.",
        "inspire": "Adv.Space Res."
      },
      {
        "cds": "Parallel Comput.",
        "inspire": "Parallel Comput."
      },
      {
        "cds": "Bulg. J. Phys.",
        "inspire": "Bulg.J.Phys."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals N.Y.Acad.Sci."
      },
      {
        "cds": "Austral.N.Zeal.Phys.",
        "inspire": "Austral.N.Zeal.Phys."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "J. Nucl. Mater.",
        "inspire": "J.Nucl.Mater."
      },
      {
        "cds": "Proc. IRE",
        "inspire": "Proc.Ire."
      },
      {
        "cds": "New Astron.",
        "inspire": "New Astron."
      },
      {
        "cds": "Math. Ann.",
        "inspire": "Math.Ann."
      },
      {
        "cds": "J. R. Stat. Soc., B",
        "inspire": "J.Roy.Statist.Soc.B"
      },
      {
        "cds": "Not.Amer.Math.Soc.",
        "inspire": "Not.Amer.Math.Soc."
      },
      {
        "cds": "Razved.Okhr.Nedr.",
        "inspire": "Razved.Okhr.Nedr."
      },
      {
        "cds": "J. Astrophys. Astron.",
        "inspire": "J.Astrophys.Astron."
      },
      {
        "cds": "Plasma Phys. Control. Fusion",
        "inspire": "Plasma Phys.Control.Fusion"
      },
      {
        "cds": "Bull. Soc. Math. Fr.",
        "inspire": "Bull.Soc.Math.Fr."
      },
      {
        "cds": "Appl. Mech. Rev.",
        "inspire": "Appl.Mech.Rev."
      },
      {
        "cds": "Biochem. J.",
        "inspire": "Biochem.J."
      },
      {
        "cds": "Contemp. Math.",
        "inspire": "Contemp.Math."
      },
      {
        "cds": "Mat. Model.",
        "inspire": "Mat.Model."
      },
      {
        "cds": "Quimica Nova",
        "inspire": "Quimica Nova"
      },
      {
        "cds": "Nucl.Data Sec.A",
        "inspire": "Nucl.Data Sec.A"
      },
      {
        "cds": "Ann. Phys. (San Diego)",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Ann. Inst. Henri Poincaré",
        "inspire": "Annales Poincare Phys.Theor."
      },
      {
        "cds": "Lect. Notes Math.",
        "inspire": "Lect.Notes Math."
      },
      {
        "cds": "Sol. Phys.",
        "inspire": "Solar Phys."
      },
      {
        "cds": "Serdica",
        "inspire": "Serdica"
      },
      {
        "cds": "Proc. Natl. Acad. Sci. U. S. A.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "Nuovo Cimento, Lett.",
        "inspire": "Lett.Nuovo Cim."
      },
      {
        "cds": "Prog. Cryst. Growth Charact. Mater.",
        "inspire": "Prog.Cryst.Growth Charact.Mater."
      },
      {
        "cds": "J. Am. Math. Soc.",
        "inspire": "J.Am.Math.Soc."
      },
      {
        "cds": "Z.Math.Phys.",
        "inspire": "Z.Math.Phys."
      },
      {
        "cds": "IEEE Trans. Evol. Comput.",
        "inspire": "IEEE Trans.Evol.Comput."
      },
      {
        "cds": "Numer. Math.",
        "inspire": "Numer.Math."
      },
      {
        "cds": "Am.Phys.Teach.",
        "inspire": "Am.Phys.Teach."
      },
      {
        "cds": "Elem.Part.Res.J.",
        "inspire": "Elem.Part.Res.J."
      },
      {
        "cds": "Z. Angew. Math. Phys.",
        "inspire": "Z.Angew.Math.Phys."
      },
      {
        "cds": "J.Contemp.Phys.",
        "inspire": "J.Contemp.Phys."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Mem.Coll.Sci.Univ.Kyoto (Series A Math.)",
        "inspire": "Mem.Coll.Sci.Univ.Kyoto (Series A Math.)"
      },
      {
        "cds": "IEEE Trans. Inf. Theory",
        "inspire": "IEEE Trans.Info.Theor."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Bull.Astron.Inst.Netherlands",
        "inspire": "Bull.Astron.Inst.Netherlands"
      },
      {
        "cds": "Bull.Am.Math.Soc.",
        "inspire": "Bull.Am.Math.Soc."
      },
      {
        "cds": "Proc. Natl. Acad. Sci. U. S. A.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "Ultramicroscopy",
        "inspire": "Ultramicroscopy"
      },
      {
        "cds": "IEEE Microw. Wirel. Compon. Lett.",
        "inspire": "IEEE Microw.Wirel.Compon.Lett."
      },
      {
        "cds": "Trud.Inst.Fiz. Akad.Nauk Eston.SSR",
        "inspire": "Trud.Inst.Fiz. Akad.Nauk Eston.SSR"
      },
      {
        "cds": "Superlattices Microstruct.",
        "inspire": "Superlattices Microstruct."
      },
      {
        "cds": "Phys. Z Sowjetunion",
        "inspire": "Phys.Z.Sowjetunion"
      },
      {
        "cds": "J. Math. Soc. Jpn.",
        "inspire": "J.Math.Soc.Jap."
      },
      {
        "cds": "Bull. Russ. Acad. Sci. Phys.",
        "inspire": "Bull.Russ.Acad.Sci.Phys."
      },
      {
        "cds": "Smart Mater. Struct.",
        "inspire": "Smart Mater.Struct."
      },
      {
        "cds": "Earth Moon Planets",
        "inspire": "Earth Moon Planets"
      },
      {
        "cds": "Quantum Electron.",
        "inspire": "Quant.Electron."
      },
      {
        "cds": "Arch.Ration.Mech.Anal.",
        "inspire": "Arch.Ration.Mech.Anal."
      },
      {
        "cds": "Surv. Differ. Geom.",
        "inspire": "Surveys Diff.Geom."
      },
      {
        "cds": "J. Physiol.",
        "inspire": "J.Physiol."
      },
      {
        "cds": "J. Electron Spectrosc. Relat. Phenom.",
        "inspire": "J.Electron Spectrosc.Rel.Phenom."
      },
      {
        "cds": "Philips Res.Rept.",
        "inspire": "Philips Res.Rept."
      },
      {
        "cds": "J. Pure Appl. Algebra",
        "inspire": "J.Pure Appl.Algebra"
      },
      {
        "cds": "J.Res.Natl.Bur.Stand.B Math.Sci.",
        "inspire": "J.Res.Natl.Bur.Stand.B Math.Sci."
      },
      {
        "cds": "Cluster Comput.",
        "inspire": "Cluster Comput."
      },
      {
        "cds": "Mat.Modelirovanie",
        "inspire": "Mat.Modelirovanie"
      },
      {
        "cds": "Sov. Math. - Dokl.",
        "inspire": "Sov.Math.Dokl."
      },
      {
        "cds": "J. High Energy Phys.",
        "inspire": "JHEP"
      },
      {
        "cds": "Rev. Mod. Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "Prog. Solid State Chem.",
        "inspire": "Prog.Solid State.Chem."
      },
      {
        "cds": "Comments Solid State Phys.",
        "inspire": "Comments Solid State Phys."
      },
      {
        "cds": "Philos. Trans. R. Soc. Lond., A",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "Pac. J. Math.",
        "inspire": "Pacific J.Math."
      },
      {
        "cds": "Astron. Soc. Pac. Conf. Proc.",
        "inspire": "ASP Conf.Ser."
      },
      {
        "cds": "Proc.Japan Acad.",
        "inspire": "Proc.Japan Acad."
      },
      {
        "cds": "Europhys. Lett.",
        "inspire": "Europhys.Lett."
      },
      {
        "cds": "Ann. Rev. Phys. Chem.",
        "inspire": "Ann.Rev.Phys.Chem."
      },
      {
        "cds": "Symp.Math.",
        "inspire": "Symp.Math."
      },
      {
        "cds": "Proc.Am.Phil.Soc.",
        "inspire": "Proc.Am.Phil.Soc."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Giornale Astron.",
        "inspire": "Giornale Astron."
      },
      {
        "cds": "At. Energ. (USSR)",
        "inspire": "At.Energ."
      },
      {
        "cds": "J.Electr.Electron.Eng.Aust.",
        "inspire": "J.Electr.Electron.Eng.Aust."
      },
      {
        "cds": "Math. Tables Aids Comput.",
        "inspire": "Math.Tables Aids Comput."
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Appl. Energy",
        "inspire": "Appl.Energy"
      },
      {
        "cds": "AIP Conf. Proc.",
        "inspire": "AIP Conf.Proc."
      },
      {
        "cds": "Phys. World",
        "inspire": "Phys.World"
      },
      {
        "cds": "JETP Lett.",
        "inspire": "JETP Lett."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "BioSyst.",
        "inspire": "Biosyst."
      },
      {
        "cds": "Eur. J. Phys.",
        "inspire": "Eur.J.Phys."
      },
      {
        "cds": "USSR Comput.Math.Math.Phys.",
        "inspire": "USSR Comput.Math.Math.Phys."
      },
      {
        "cds": "Adv. Magn. Reson.",
        "inspire": "Adv.Magn.Reson."
      },
      {
        "cds": "Found.Phys.",
        "inspire": "Found.Phys."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Funkt.Anal.Pril.",
        "inspire": "Funkt.Anal.Pril."
      },
      {
        "cds": "Ann. Eugenics",
        "inspire": "Annals Eugen."
      },
      {
        "cds": "Commun.Stat.",
        "inspire": "Commun.Stat."
      },
      {
        "cds": "Astron. Astrophys. Suppl. Ser.",
        "inspire": "Astron.Astrophys.Suppl.Ser."
      },
      {
        "cds": "Fluid Mech.Sov.Res.",
        "inspire": "Fluid Mech.Sov.Res."
      },
      {
        "cds": "Adv. Protein Chem.",
        "inspire": "Adv.Prot.Chem."
      },
      {
        "cds": "Lect. Notes Phys.",
        "inspire": "Lect.Notes Phys."
      },
      {
        "cds": "Adv.X-Ray Spectros.",
        "inspire": "Universitas"
      },
      {
        "cds": "Europhys. Lett.",
        "inspire": "Europhys.Lett."
      },
      {
        "cds": "Phys. Rev. Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "J. Opt. Soc. Am. B",
        "inspire": "J.Opt.Soc.Am."
      },
      {
        "cds": "J. Opt. Soc. Am. A",
        "inspire": "J.Opt.Soc.Am."
      },
      {
        "cds": "Rev. Phys. Appl.",
        "inspire": "Rev.Phys.Appl."
      },
      {
        "cds": "IEEE Trans. Microw. Theory Tech.",
        "inspire": "IEEE Trans.Microwave Theor.Tech."
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "IEEE Trans. Microw. Theory Tech.",
        "inspire": "IEEE Trans.Microwave Theor.Tech."
      },
      {
        "cds": "Theor. Math. Phys.",
        "inspire": "Theor.Math.Phys."
      },
      {
        "cds": "Chin.J.Comput.Phys.",
        "inspire": "Chin.J.Comput.Phys."
      },
      {
        "cds": "Teor. Mat. Fiz.",
        "inspire": "Teor.Mat.Fiz."
      },
      {
        "cds": "Int.J.Math.Educ.Sci.Tech.",
        "inspire": "Int.J.Math.Educ.Sci.Tech."
      },
      {
        "cds": "Monatsh. Math.",
        "inspire": "Mon.Math."
      },
      {
        "cds": "Proc.Nat.Acad.Sci.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "J. Artif. Intell. Res.",
        "inspire": "J.Artif.Intell.Res."
      },
      {
        "cds": "New J. Phys.",
        "inspire": "New J.Phys."
      },
      {
        "cds": "Nous",
        "inspire": "Nous"
      },
      {
        "cds": "Comput.Design",
        "inspire": "Comput.Design"
      },
      {
        "cds": "Elec.Trans.Numer.Anal.",
        "inspire": "ETNA"
      },
      {
        "cds": "J. Philos. Log.",
        "inspire": "J.Phil.Logic"
      },
      {
        "cds": "Int. J. Control",
        "inspire": "Int.J.Control"
      },
      {
        "cds": "NUCL-TH",
        "inspire": "NUCL-TH"
      },
      {
        "cds": "Fresenius J. Anal. Chem.",
        "inspire": "Fresenius' J.Anal.Chem."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Rend. Lincei",
        "inspire": "Rend. Acc. Naz. Lincei"
      },
      {
        "cds": "Proc. Camb. Philos. Soc.",
        "inspire": "Proc.Cambridge Phil.Soc."
      },
      {
        "cds": "Q.Rev.Biol.",
        "inspire": "Q.Rev.Biol."
      },
      {
        "cds": "J. Biol. Chem.",
        "inspire": "J.Biol.Chem."
      },
      {
        "cds": "J. Soviet Math.",
        "inspire": "J.Sov.Math."
      },
      {
        "cds": "Fusion Eng. Des.",
        "inspire": "Fusion Eng.Des."
      },
      {
        "cds": "Theor. Comput. Sci.",
        "inspire": "Theor.Comput.Sci."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Ann. Eugenics",
        "inspire": "Annals Eugen."
      },
      {
        "cds": "High Energy Phys.Nucl.Phys.",
        "inspire": "High Energy Phys.Nucl.Phys."
      },
      {
        "cds": "Topol. Appl.",
        "inspire": "Topology Appl."
      },
      {
        "cds": "Chem. Eur. J.",
        "inspire": "Chem.Eur.J."
      },
      {
        "cds": "Quantum Semiclass. Opt.",
        "inspire": "Quant.Semiclass.Opt."
      },
      {
        "cds": "ACM Trans. Graph.",
        "inspire": "ACM Trans.Graph."
      },
      {
        "cds": "Nucl. Phys. A",
        "inspire": "Nucl.Phys."
      },
      {
        "cds": "Annals Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Comment. Math. Helv.",
        "inspire": "Comment.Math.Helv."
      },
      {
        "cds": "Sugaku",
        "inspire": "Sugaku"
      },
      {
        "cds": "Spectrosc.",
        "inspire": "Spectrosc."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Astron. J.",
        "inspire": "Astron.J."
      },
      {
        "cds": "Ann.Univ.Turku Ser.AL",
        "inspire": "Ann.Univ.Turku Ser.AL"
      },
      {
        "cds": "At. Data Nucl. Data Tables",
        "inspire": "Atom.Data Nucl.Data Tabl."
      },
      {
        "cds": "Eur.J.Nucl.Med.",
        "inspire": "Eur.J.Nucl.Med."
      },
      {
        "cds": "Chem. Mater.",
        "inspire": "Chem.Mater."
      },
      {
        "cds": "Astron. Astrophys. Rev.",
        "inspire": "Astron.Astrophys.Rev."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "C. R. Acad. Sci. Paris",
        "inspire": "Compt.Rend.Math."
      },
      {
        "cds": "J. Grid Comput.",
        "inspire": "J.Grid Comput."
      },
      {
        "cds": "Bull. Astron. Soc. India",
        "inspire": "Bull.Astron.Soc.India"
      },
      {
        "cds": "Izv.Vuz.Mat.",
        "inspire": "Izv.Vuz.Mat."
      },
      {
        "cds": "Fiz. Elem. Chastits At. Yadra",
        "inspire": "Fiz.Elem.Chast.Atom.Yadra"
      },
      {
        "cds": "Frascati Phys.Ser.",
        "inspire": "Frascati Phys.Ser."
      },
      {
        "cds": "Astrophys.J.Suppl.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Publ. Astron. Soc. Jpn.",
        "inspire": "Publ.Astron.Soc.Jap."
      },
      {
        "cds": "Sitzungsber.Akad.Wiss.Wien.Math.(Naturwiss.Klasse)",
        "inspire": "Sitzungsber.Akad.Wiss.Wien.Math.(Naturwiss.Klasse)"
      },
      {
        "cds": "Astron. J.",
        "inspire": "Astron.J."
      },
      {
        "cds": "Surf. Sci.",
        "inspire": "Surf.Sci."
      },
      {
        "cds": "J.Cell.Compar.Physiol.",
        "inspire": "J.Cell.Compar.Physiol."
      },
      {
        "cds": "Acta Mater.",
        "inspire": "Acta Mater."
      },
      {
        "cds": "J. Quant. Spectrosc. Radiat. Transf",
        "inspire": "J.Quant.Spectrosc.Radiat.Trans."
      },
      {
        "cds": "J. Polym. Sci.",
        "inspire": "J.Polymer Sci."
      },
      {
        "cds": "Comment. Phys.-Math.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "AGF Jahresheft",
        "inspire": "AGF Jahresheft"
      },
      {
        "cds": "Geom. Dedicata",
        "inspire": "Geom.Dedicata"
      },
      {
        "cds": "Rev.Opt.",
        "inspire": "Rev.Opt."
      },
      {
        "cds": "J. Differ. Geom.",
        "inspire": "J.Diff.Geom."
      },
      {
        "cds": "Ann. Math.",
        "inspire": "Annals Math."
      },
      {
        "cds": "Arch.Hist.Ex.Sci.",
        "inspire": "Arch.Hist.Ex.Sci."
      },
      {
        "cds": "Mem.Fac.Sci.Kyoto (Ser.A Phys.Astrophys.Geophys.Chem.)",
        "inspire": "Mem.Fac.Sci.Kyoto (Ser.A Phys.Astrophys.Geophys.Chem.)"
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Adv. Part. Phys.",
        "inspire": "Adv.Part.Phys."
      },
      {
        "cds": "Astron. J.",
        "inspire": "Astron.J."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Astropart. Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Infrared Phys. Technol.",
        "inspire": "Infrared Phys.Tech."
      },
      {
        "cds": "Ann. Stat.",
        "inspire": "Annals Statist."
      },
      {
        "cds": "Magyar Fiz.Foly.",
        "inspire": "Magyar Fiz.Foly."
      },
      {
        "cds": "Nova Acta Leopoldina",
        "inspire": "Nova Acta Leopoldina"
      },
      {
        "cds": "Chin. J. Astron. Astrophys.",
        "inspire": "Chin.J.Astron.Astrophys."
      },
      {
        "cds": "Electronics",
        "inspire": "Electronics"
      },
      {
        "cds": "Adv. Space Res.",
        "inspire": "Adv.Space Res."
      },
      {
        "cds": "Vestn.Leningrad.Gos.Univ.",
        "inspire": "Vestn.Leningrad.Gos.Univ."
      },
      {
        "cds": "Appl. Spectrosc.",
        "inspire": "Appl.Spectrosc."
      },
      {
        "cds": "Nagare",
        "inspire": "Nagare"
      },
      {
        "cds": "Adv. Math.",
        "inspire": "Adv.Math."
      },
      {
        "cds": "Adv. Theor. Math. Phys.",
        "inspire": "Adv.Theor.Math.Phys."
      },
      {
        "cds": "C. R. Acad. Sci., 4",
        "inspire": "C.R.Acad.Sci.IV Phys.Astrophys."
      },
      {
        "cds": "Stud.Geophys.Geod.",
        "inspire": "Stud.Geophys.Geod."
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Microchem. J.",
        "inspire": "Microchem.J."
      },
      {
        "cds": "Pure Appl. Opt.",
        "inspire": "Pure Appl.Opt."
      },
      {
        "cds": "J.Plasma Phys.",
        "inspire": "J.Plasma Phys."
      },
      {
        "cds": "Int.J.Mod.Phys.",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Acta Math. Sin.",
        "inspire": "Acta Math.Sin."
      },
      {
        "cds": "Phys.Rev.",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "Biol.Cyber.",
        "inspire": "Biol.Cyber."
      },
      {
        "cds": "Publ.Dominion Astrophys.Obs.",
        "inspire": "Publ.Dominion Astrophys.Obs."
      },
      {
        "cds": "Bussei Kenkyu",
        "inspire": "Bussei Kenkyu"
      },
      {
        "cds": "Proc. Phys. Soc.",
        "inspire": "Proc.Phys.Soc."
      },
      {
        "cds": "Appl.Sci.Res.(B)",
        "inspire": "Appl.Sci.Res.(B)"
      },
      {
        "cds": "Magn.Ghidrodin.",
        "inspire": "Magn.Ghidrodin."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Russ. Math. Surv.",
        "inspire": "Usp.Mat.Nauk"
      },
      {
        "cds": "J. Cluster Sci.",
        "inspire": "J.Cluster Sci."
      },
      {
        "cds": "J.Virology",
        "inspire": "J.Virology"
      },
      {
        "cds": "J. Stat. Phys.",
        "inspire": "J.Statist.Phys."
      },
      {
        "cds": "St.Petersburg Math.J.",
        "inspire": "St.Petersburg Math.J."
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Nuovo Cimento, Lett.",
        "inspire": "Lett.Nuovo Cim."
      },
      {
        "cds": "J. Stat. Plan. Inference",
        "inspire": "J.Stat.Plan.Inference"
      },
      {
        "cds": "Nucl.Data Sec.B",
        "inspire": "Nucl.Data Sec.B"
      },
      {
        "cds": "J.Geodyn.",
        "inspire": "J.Geodyn."
      },
      {
        "cds": "Mon.Math.",
        "inspire": "Mon.Math."
      },
      {
        "cds": "Appl.Statist.",
        "inspire": "Appl.Statist."
      },
      {
        "cds": "Math. Syst. Theor.",
        "inspire": "Math.Syst.Theor."
      },
      {
        "cds": "Izv.Akad.Nauk Uzb.SSR",
        "inspire": "Izv.Akad.Nauk Uzb.SSR"
      },
      {
        "cds": "Structure",
        "inspire": "Structure"
      },
      {
        "cds": "J. Parallel Distrib. Comput.",
        "inspire": "J.Parallel Distrib.Comput."
      },
      {
        "cds": "Yad. Fiz.",
        "inspire": "Yad.Fiz."
      },
      {
        "cds": "Astron. Zh.",
        "inspire": "Astron.Zh."
      },
      {
        "cds": "Ergodic Theory Dyn. Syst.",
        "inspire": "Ergod.Theor.Dynam.Syst."
      },
      {
        "cds": "Geophys. J. Int.",
        "inspire": "Geophys.J.Int."
      },
      {
        "cds": "Pion Nucleon Newsl.",
        "inspire": "PiN Newslett."
      },
      {
        "cds": "Astron. Astrophys. Lett.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Nucl. Fusion",
        "inspire": "Nucl.Fusion"
      },
      {
        "cds": "Geophys. Astrophys. Fluid Dyn.",
        "inspire": "Geophys.Astrophys.Fluid Dynamics"
      },
      {
        "cds": "Bull.Acad.Pol.Sci.Ser.Sci.Math.Astron.Phys.",
        "inspire": "Bull.Acad.Pol.Sci.Ser.Sci.Math.Astron.Phys."
      },
      {
        "cds": "Opt. Mater.",
        "inspire": "Opt.Mater."
      },
      {
        "cds": "Ann. Sci. Ec. Norm. Supér.",
        "inspire": "Annales Sci.Ecole Norm.Sup."
      },
      {
        "cds": "Adv. Eng. Softw.",
        "inspire": "Adv. Eng. Softw."
      },
      {
        "cds": "J. Math. Anal. Appl.",
        "inspire": "J.Math.Anal.Appl."
      },
      {
        "cds": "A. Appl. Physiol.",
        "inspire": "J.Appl.Physiol."
      },
      {
        "cds": "Hand. Phys.",
        "inspire": "Handb.Phys."
      },
      {
        "cds": "Sci. Sinica",
        "inspire": "Sci.Sin."
      },
      {
        "cds": "Inverse Probl.",
        "inspire": "Inverse Prob."
      },
      {
        "cds": "Mem. Soc. Astron. Ital.",
        "inspire": "Mem.Soc.Ast.It."
      },
      {
        "cds": "Banach Cent. Publ.",
        "inspire": "Banach Center Publ."
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Comments Astrophys. Space Phys.",
        "inspire": "Comments Astrophys. Space Phys."
      },
      {
        "cds": "HEP-TH",
        "inspire": "HEP-TH"
      },
      {
        "cds": "Mater.Sci.Eng.",
        "inspire": "Mater.Sci.Eng."
      },
      {
        "cds": "Astron. Soc. Pac. Conf. Proc.",
        "inspire": "ASP Conf.Ser."
      },
      {
        "cds": "J. Differ. Equ.",
        "inspire": "J.Diff.Eq."
      },
      {
        "cds": "Comput.-Aided Geom. Des.",
        "inspire": "Comput. Aided Geom. Des."
      },
      {
        "cds": "Sov.Phys.",
        "inspire": "Sov.Phys."
      },
      {
        "cds": "Geom. Dedicata",
        "inspire": "Geom.Dedicata"
      },
      {
        "cds": "Indag.Math.",
        "inspire": "Indag.Math."
      },
      {
        "cds": "Proc. Natl. Acad. Sci. U. S. A.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "Phys.Rev.",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "J.Neurosci.Methods",
        "inspire": "J.Neurosci.Methods"
      },
      {
        "cds": "J. Phys. A",
        "inspire": "J.Phys.A"
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Mol.Mater.",
        "inspire": "Mol.Mater."
      },
      {
        "cds": "Trud.Tbil.Mat.Inst.",
        "inspire": "Trud.Tbil.Mat.Inst."
      },
      {
        "cds": "SIGACT News",
        "inspire": "Sigact News"
      },
      {
        "cds": "J. Phys., C",
        "inspire": "J.Phys.C"
      },
      {
        "cds": "J. Comput. Appl. Math.",
        "inspire": "J.Comput.Appl.Math."
      },
      {
        "cds": "Adv. Solid State Phys.",
        "inspire": "Adv.Solid State Phys."
      },
      {
        "cds": "Collect. Phenom.",
        "inspire": "Collect.Phenom."
      },
      {
        "cds": "J. Cosmol. Astropart. Phys.",
        "inspire": "JCAP"
      },
      {
        "cds": "Synchrotron Radiat.News",
        "inspire": "Synchrotron Radiat.News"
      },
      {
        "cds": "Comments At. Mol. Phys.",
        "inspire": "Comments Atom.Mol.Phys."
      },
      {
        "cds": "Solid State Phys.",
        "inspire": "Solid State Phys."
      },
      {
        "cds": "Rept.Prog.Phys.",
        "inspire": "Rept.Prog.Phys."
      },
      {
        "cds": "Prog. Theor. Phys., Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "Rev. Math. Phys.",
        "inspire": "Rev.Math.Phys."
      },
      {
        "cds": "Scientometrics",
        "inspire": "Scientometrics"
      },
      {
        "cds": "Appl. Numer. Math.",
        "inspire": "Appl.Numer.Math."
      },
      {
        "cds": "Proc. Natl. Acad. Sci. U. S. A.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "Semigroup Forum",
        "inspire": "Semigroup Forum"
      },
      {
        "cds": "IEEE Power Eng. Rev.",
        "inspire": "IEEE Power Eng.Rev."
      },
      {
        "cds": "Prog. Theor. Phys., Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "Adv.Neural Inf.Process.Syst.",
        "inspire": "Adv.Neural Inf.Process.Syst."
      },
      {
        "cds": "Commun. Algebra",
        "inspire": "Commun.Algebra"
      },
      {
        "cds": "Phys. Rep., C",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Biometrics",
        "inspire": "Biometrics"
      },
      {
        "cds": "J. Mod. Opt.",
        "inspire": "J.Mod.Opt."
      },
      {
        "cds": "Z. Phys. C",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Comment.Pont.Acad.Sci.",
        "inspire": "Comment.Pont.Acad.Sci."
      },
      {
        "cds": "Ultrasound Med. Biol.",
        "inspire": "Ultrasound Med.Biol."
      },
      {
        "cds": "J. Soviet Math.",
        "inspire": "J.Sov.Math."
      },
      {
        "cds": "Izv. Akad. Nauk Fiz.",
        "inspire": "Izv.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "Eur. Phys. J. Spec. Top.",
        "inspire": "Eur.Phys.J.ST"
      },
      {
        "cds": "Cryst. Growth Des.",
        "inspire": "Cryst.Growth Des."
      },
      {
        "cds": "Opt. Commun.",
        "inspire": "Opt.Commun."
      },
      {
        "cds": "J. ACM Assoc. Comput. Mach.",
        "inspire": "J.Assoc.Comput.Machinery"
      },
      {
        "cds": "Geokhim.",
        "inspire": "Geokhim."
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "Proc. R. Soc. Lond., A",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "Quat.Res.",
        "inspire": "Quat.Res."
      },
      {
        "cds": "Indian J. Phys. B",
        "inspire": "Indian J.Phys.B"
      },
      {
        "cds": "Proc. Astron. Soc. Pac.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "Indian J. Phys. A",
        "inspire": "Indian J.Phys.A"
      },
      {
        "cds": "JINR Rapid Commun.",
        "inspire": "JINR Rapid Commun."
      },
      {
        "cds": "J.Appl.Phil.",
        "inspire": "J.Appl.Phil."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Appl. Stat.",
        "inspire": "Appl.Statist."
      },
      {
        "cds": "Physiol. Meas.",
        "inspire": "Physiol.Meas."
      },
      {
        "cds": "Colloq. Math.",
        "inspire": "Colloq.Math."
      },
      {
        "cds": "Astron. Rep.",
        "inspire": "Astron.Rep."
      },
      {
        "cds": "Concepts Magn. Res.",
        "inspire": "Concepts Magn.Res."
      },
      {
        "cds": "Ann. Henri Poincaré",
        "inspire": "Annales Henri Poincare"
      },
      {
        "cds": "Fiz.Elem.Chast.Atom.Yadra",
        "inspire": "Fiz.Elem.Chast.Atom.Yadra"
      },
      {
        "cds": "J. Phys. Chem. Solids",
        "inspire": "J.Phys.Chem.Solids"
      },
      {
        "cds": "J. Algebra",
        "inspire": "J.Algebra"
      },
      {
        "cds": "IEEE Electron Device Lett.",
        "inspire": "IEEE Electron. Dev. Lett."
      },
      {
        "cds": "J. Bacteriol.",
        "inspire": "J.Bacteriol."
      },
      {
        "cds": "J.Ecole Poly.",
        "inspire": "J.Ecole Poly."
      },
      {
        "cds": "Stat. Probab. Lett.",
        "inspire": "Statist.Probab.Lett."
      },
      {
        "cds": "Rev. Mod. Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "Langmuir",
        "inspire": "Langmuir"
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Publ. Res. Inst. Math. Sci.",
        "inspire": "Publ.Res.Inst.Math.Sci.Kyoto"
      },
      {
        "cds": "Nuovo Cimento, Lett.",
        "inspire": "Lett.Nuovo Cim."
      },
      {
        "cds": "Discrete Appl. Math.",
        "inspire": "Discrete Appl.Math."
      },
      {
        "cds": "J. Comput. Phys.",
        "inspire": "J.Comput.Phys."
      },
      {
        "cds": "J. High Energy Phys.",
        "inspire": "JHEP"
      },
      {
        "cds": "Gott.Nachr.",
        "inspire": "Gott.Nachr."
      },
      {
        "cds": "Catal. Lett.",
        "inspire": "Catal.Lett."
      },
      {
        "cds": "J.Philos",
        "inspire": "J.Philos"
      },
      {
        "cds": "PiN Newslett.",
        "inspire": "PiN Newslett."
      },
      {
        "cds": "Norsk.Vid.Selsk.Forh.",
        "inspire": "Norsk.Vid.Selsk.Forh."
      },
      {
        "cds": "Represent. Theory",
        "inspire": "Represent. Theory"
      },
      {
        "cds": "Yad.Fiz.",
        "inspire": "Yad.Fiz."
      },
      {
        "cds": "J.Roy.Statist.Soc.A",
        "inspire": "J.Roy.Statist.Soc.B"
      },
      {
        "cds": "Appl. Mech. Rev.",
        "inspire": "Appl.Mech.Rev."
      },
      {
        "cds": "Phys. Part. Nucl.",
        "inspire": "Phys.Part.Nucl."
      },
      {
        "cds": "Publ. Astron. Soc. Aust.",
        "inspire": "Publ.Astron.Soc.Austral."
      },
      {
        "cds": "Astron. Lett.",
        "inspire": "Astron.Lett."
      },
      {
        "cds": "SIAM J. Sci. Comput.",
        "inspire": "SIAM J.Sci.Comput."
      },
      {
        "cds": "Sci.Cult.",
        "inspire": "Sci.Cult."
      },
      {
        "cds": "Pac. J. Math.",
        "inspire": "Pacific J.Math."
      },
      {
        "cds": "Eur.J.Combinatorics",
        "inspire": "Eur.J.Combinatorics"
      },
      {
        "cds": "Astron.Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Trans.Amer.Nucl.Soc.",
        "inspire": "Trans.Amer.Nucl.Soc."
      },
      {
        "cds": "Abh.Braunschweig.Wiss.Ges.",
        "inspire": "Abh.Braunschweig.Wiss.Ges."
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Appl.Phys.Lett."
      },
      {
        "cds": "IEEE Trans. Plasma Sci.",
        "inspire": "IEEE Trans.Plasma Sci."
      },
      {
        "cds": "Stoch.Anal.Appl.",
        "inspire": "Stoch.Anal.Appl."
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Int.J.Math.Math.Sci.",
        "inspire": "Int.J.Math.Math.Sci."
      },
      {
        "cds": "Ann. Inst. Fourier",
        "inspire": "Annales Inst.Fourier"
      },
      {
        "cds": "Dialectica",
        "inspire": "Dialectica"
      },
      {
        "cds": "Trans. Am. Math. Soc.",
        "inspire": "Trans.Am.Math.Soc."
      },
      {
        "cds": "Nonlinear Dyn.",
        "inspire": "Nonlinear Dyn."
      },
      {
        "cds": "Rocky Mt. J. Math.",
        "inspire": "Rocky Mt.J.Math."
      },
      {
        "cds": "Probab. Theory Relat. Fields",
        "inspire": "Probab.Theor.Related Fields"
      },
      {
        "cds": "Funct. Anal. Appl.",
        "inspire": "Funct.Anal.Appl."
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Phil.Trans.Roy.Soc.Lond.",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "Coll. Res. Libr.",
        "inspire": "Coll.Res.Libr."
      },
      {
        "cds": "Astron.Express",
        "inspire": "Astron.Express"
      },
      {
        "cds": "Proc. IEEE",
        "inspire": "IEEE Proc."
      },
      {
        "cds": "Arch. Appl. Mech.",
        "inspire": "Arch.Appl.Mech."
      },
      {
        "cds": "IEEE J. Quantum Electron.",
        "inspire": "IEEE J.Quant.Electron."
      },
      {
        "cds": "ESA Bull.",
        "inspire": "ESA Bull."
      },
      {
        "cds": "Acta Mater.",
        "inspire": "Acta Mater."
      },
      {
        "cds": "Exp.Tech.Phys.",
        "inspire": "Exp.Tech.Phys."
      },
      {
        "cds": "J. Magn. Reson.",
        "inspire": "J.Magn.Resonance"
      },
      {
        "cds": "Astrophys. Space Sci.",
        "inspire": "Astrophys.Space Sci."
      },
      {
        "cds": "Opt. Spectrosc.",
        "inspire": "Opt.Spectrosc."
      },
      {
        "cds": "Proc. R. Soc. Edinburgh",
        "inspire": "Proc.Roy.Soc.Edinburgh"
      },
      {
        "cds": "Phys. - Uspekhi",
        "inspire": "Phys.Usp."
      },
      {
        "cds": "J. Supercomput.",
        "inspire": "J.Supercomput."
      },
      {
        "cds": "J. Appl. Spectrosc.",
        "inspire": "J.Appl.Spectrosc."
      },
      {
        "cds": "Bull. Am. Math. Soc.",
        "inspire": "Bull.Am.Math.Soc."
      },
      {
        "cds": "Phys.Rev.Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "J. Lond. Math. Soc.",
        "inspire": "Bull.London Math.Soc."
      },
      {
        "cds": "Sov.Astron.",
        "inspire": "Sov.Astron."
      },
      {
        "cds": "Sitzungsber.Preuss.Akad.Wiss.Berlin (Math.Phys.)",
        "inspire": "Sitzungsber.Preuss.Akad.Wiss.Berlin (Math.Phys.)"
      },
      {
        "cds": "Analysis",
        "inspire": "Analysis"
      },
      {
        "cds": "Conceptus",
        "inspire": "Conceptus"
      },
      {
        "cds": "Prace Mat.- Fiz.",
        "inspire": "Prace Mat.Fiz."
      },
      {
        "cds": "Bull.Cancer",
        "inspire": "Bull.Cancer"
      },
      {
        "cds": "Math. Res. Lett.",
        "inspire": "Math.Res.Lett."
      },
      {
        "cds": "Nano Lett.",
        "inspire": "Nano Lett."
      },
      {
        "cds": "J. Algebr. Comb.",
        "inspire": "J.Algebr.Comb."
      },
      {
        "cds": "Invent. Math.",
        "inspire": "Invent.Math."
      },
      {
        "cds": "J. Microelectromech. Syst.",
        "inspire": "J.Microelectromech.Syst."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Proc. Symp. Pure Math.",
        "inspire": "Proc.Symp.Pure Math."
      },
      {
        "cds": "J. Quantum Spectrosc. Radiat. Transf.",
        "inspire": "J.Quant.Spectrosc.Radiat.Trans."
      },
      {
        "cds": "Sky Telesc.",
        "inspire": "Sky Telesc."
      },
      {
        "cds": "Nucl.Technol.",
        "inspire": "Nucl.Technol."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Publ. Astron. Soc. Pac.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "J. Phys. Soc. Jpn.",
        "inspire": "J.Phys.Soc.Jap."
      },
      {
        "cds": "J. Nucl. Mater.",
        "inspire": "J.Nucl.Mater."
      },
      {
        "cds": "J.Hydronaut.",
        "inspire": "J.Hydronaut."
      },
      {
        "cds": "Astron. Astrophys. Suppl. Ser.",
        "inspire": "Astron.Astrophys.Suppl.Ser."
      },
      {
        "cds": "Quart. J. Roy. Astron. Soc.",
        "inspire": "Q.J.Roy.Astron.Soc."
      },
      {
        "cds": "Phys. Rep.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "J. Chem. Phys.",
        "inspire": "J.Chem.Phys."
      },
      {
        "cds": "Acta Numer.",
        "inspire": "Acta Numer."
      },
      {
        "cds": "Adv. High Energy Phys.",
        "inspire": "Adv.High Energy Phys."
      },
      {
        "cds": "Math.Proc.Cambridge Phil.Soc.",
        "inspire": "Math.Proc.Cambridge Phil.Soc."
      },
      {
        "cds": "Prog. Quantum Electron.",
        "inspire": "Prog.Quant.Electron."
      },
      {
        "cds": "Izv.Krym.Astrofiz.Obs.",
        "inspire": "Izv.Krym.Astrofiz.Obs."
      },
      {
        "cds": "Geology",
        "inspire": "Geology"
      },
      {
        "cds": "Math. Comput.",
        "inspire": "Math.Comput."
      },
      {
        "cds": "Ric. Sci.",
        "inspire": "Ric.Sci."
      },
      {
        "cds": "Radiochim. Acta",
        "inspire": "Radiochim.Acta"
      },
      {
        "cds": "Am. J. Math.",
        "inspire": "Am.J.Math."
      },
      {
        "cds": "Sky Telesc.",
        "inspire": "Sky Telesc."
      },
      {
        "cds": "Phys. At. Nucl.",
        "inspire": "Phys.Atom.Nucl."
      },
      {
        "cds": "Proc. Natl. Acad. Sci. U. S. A.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "IEEE Proc.",
        "inspire": "IEEE Proc."
      },
      {
        "cds": "Econom.J.",
        "inspire": "Econom.J."
      },
      {
        "cds": "Phys. Status Solidi",
        "inspire": "Phys.Status Solidi"
      },
      {
        "cds": "Acta Phys. Slovaca",
        "inspire": "Acta Phys.Slov."
      },
      {
        "cds": "Phys. Rep.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Phys. Rev.",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "J. Supercond.",
        "inspire": "J.Supercond."
      },
      {
        "cds": "J. Magn. Magn. Mater.",
        "inspire": "J.Magn.Magn.Mater."
      },
      {
        "cds": "Am. J. Math.",
        "inspire": "Am.J.Math."
      },
      {
        "cds": "Laser Chem.",
        "inspire": "Laser Chem."
      },
      {
        "cds": "Ann. Rev. Fluid. Mech.",
        "inspire": "Ann.Rev.Fluid Mech."
      },
      {
        "cds": "J. Soviet Math.",
        "inspire": "J.Sov.Math."
      },
      {
        "cds": "Opt. Commun.",
        "inspire": "Opt.Commun."
      },
      {
        "cds": "Kinam",
        "inspire": "Kinam"
      },
      {
        "cds": "Adv. Mater.",
        "inspire": "Adv.Mater."
      },
      {
        "cds": "Chin. J. Phys.",
        "inspire": "Chin.J.Phys."
      },
      {
        "cds": "Curr. Opin. Struct. Biol.",
        "inspire": "Curr.Opin.Struct.Biol."
      },
      {
        "cds": "Appl. Phys. Lett.",
        "inspire": "Appl.Phys.Lett."
      },
      {
        "cds": "Intersci.Monogr.Phys.Astron.",
        "inspire": "Intersci.Monogr.Phys.Astron."
      },
      {
        "cds": "Nucl. Tracks",
        "inspire": "Nucl.Tracks"
      },
      {
        "cds": "J. Pure Appl. Algebra",
        "inspire": "J.Pure Appl.Algebra"
      },
      {
        "cds": "Zh.Eksp.Teor.Fiz.",
        "inspire": "Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Astron. Astrophys. Suppl. Ser.",
        "inspire": "Astron.Astrophys.Suppl.Ser."
      },
      {
        "cds": "Chem. Phys. Lett.",
        "inspire": "Chem.Phys.Lett."
      },
      {
        "cds": "J. Korean Astron. Soc.",
        "inspire": "J.Korean Astron.Soc."
      },
      {
        "cds": "Arch. Math.",
        "inspire": "Arch.Math."
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "Isr. J. Math.",
        "inspire": "Isr.J.Math."
      },
      {
        "cds": "J. Organomet. Chem.",
        "inspire": "J.Organomet.Chem."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc. Lett.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "J. Electron. Publ.",
        "inspire": "J.Electron.Publ."
      },
      {
        "cds": "Electron. J. Comb.",
        "inspire": "Eur.J.Combinatorics"
      },
      {
        "cds": "Electron. Eng.",
        "inspire": "Electron.Eng."
      },
      {
        "cds": "EPJ Direct",
        "inspire": "Eur.Phys.J.direct"
      },
      {
        "cds": "Ann. Probab.",
        "inspire": "Annals Probab."
      },
      {
        "cds": "Yad.Fiz.",
        "inspire": "Yad.Fiz."
      },
      {
        "cds": "J.Gen.Chem.USSR",
        "inspire": "J.Gen.Chem.USSR"
      },
      {
        "cds": "J.Roy.Astron.Soc.Canada",
        "inspire": "J.Roy.Astron.Soc.Canada"
      },
      {
        "cds": "Phys. Rev. E",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "C.R.Acad.Sci.URSS",
        "inspire": "C.R.Acad.Sci.URSS"
      },
      {
        "cds": "Phys. Rev. Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "Data Min. Knowl. Discov.",
        "inspire": "Data Min.Knowl.Discov."
      },
      {
        "cds": "Phys. Rep.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Ann. Eugenics",
        "inspire": "Annals Eugen."
      },
      {
        "cds": "Nucl. Tracks Radiat. Meas.",
        "inspire": "Nucl.Tracks Radiat.Meas."
      },
      {
        "cds": "Int. Math. Res. Not.",
        "inspire": "Int.Math.Res.Not."
      },
      {
        "cds": "Banach Cent. Publ.",
        "inspire": "Banach Center Publ."
      },
      {
        "cds": "Rom. J. Phys.",
        "inspire": "Rom.J.Phys."
      },
      {
        "cds": "Synthese",
        "inspire": "Synthese"
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "Phys. Status Solidi",
        "inspire": "Phys.Status Solidi"
      },
      {
        "cds": "X-ray Spectrom.",
        "inspire": "X Ray Spectrom."
      },
      {
        "cds": "Surv. High Energy Phys.",
        "inspire": "Surveys High Energ.Phys."
      },
      {
        "cds": "IEEE Trans. Softw. Eng.",
        "inspire": "IEEE Trans.Software Eng."
      },
      {
        "cds": "Rev. Mod. Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "Bull. Am. Phys. Soc.",
        "inspire": "Bull.Am.Phys.Soc."
      },
      {
        "cds": "Am.Philos.Q.",
        "inspire": "Am.Philos.Q."
      },
      {
        "cds": "Stud. Appl. Math.",
        "inspire": "Stud.Appl.Math."
      },
      {
        "cds": "Funkcial.Ekvac.",
        "inspire": "Funkcial.Ekvac."
      },
      {
        "cds": "Biopolymers",
        "inspire": "Biopolymers"
      },
      {
        "cds": "Rev. Mod. Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Fiz. Tverdogo Tela",
        "inspire": "Fiz.Tverd.Tela"
      },
      {
        "cds": "Proc. R. Soc. Lond., A",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "IEEE Photonics Technol. Lett.",
        "inspire": "IEEE Photonics Tech.Lett."
      },
      {
        "cds": "Celest. Mech.",
        "inspire": "Celestial Mech."
      },
      {
        "cds": "Math. Mag.",
        "inspire": "Math.Mag."
      },
      {
        "cds": "Solid State Electron.",
        "inspire": "Solid State Electron."
      },
      {
        "cds": "Astron. Astrophys. Suppl. Ser.",
        "inspire": "Astron.Astrophys.Suppl.Ser."
      },
      {
        "cds": "J. Virol.",
        "inspire": "J.Virology"
      },
      {
        "cds": "Proc. Am. Math. Soc.",
        "inspire": "Proc.Am.Math.Soc."
      },
      {
        "cds": "Matematika",
        "inspire": "Matematika"
      },
      {
        "cds": "Proteins Struct. Funct. Genet.",
        "inspire": "Proteins Struct.Funct.Genet."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Proc. Symp. Pure Math.",
        "inspire": "Proc.Symp.Pure Math."
      },
      {
        "cds": "J.Guid.Control Dyn.",
        "inspire": "J.Guid.Control Dyn."
      },
      {
        "cds": "J. Differ. Geom.",
        "inspire": "J.Diff.Geom."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "IEEE Trans. Instrum. Meas.",
        "inspire": "IEEE Trans.Instrum.Measur."
      },
      {
        "cds": "Muon Catal. Fusion",
        "inspire": "Muon Cat.Fusion"
      },
      {
        "cds": "Exp. Astron.",
        "inspire": "Exper.Astron."
      },
      {
        "cds": "J.Cybern.",
        "inspire": "J.Cybern."
      },
      {
        "cds": "Astron. J.",
        "inspire": "Astron.J."
      },
      {
        "cds": "Fluid Dynamics Res.",
        "inspire": "Fluid Dynamics"
      },
      {
        "cds": "Proc. Phys. Soc. Lond.",
        "inspire": "Proc.Phys.Soc."
      },
      {
        "cds": "Chin. Phys.",
        "inspire": "Chin.Phys."
      },
      {
        "cds": "Anales Inst.Mat.",
        "inspire": "Anales Inst.Mat."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Sov. Phys. - JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Publ. Math. IHES",
        "inspire": "Inst.Hautes Etudes Sci.Publ.Math."
      },
      {
        "cds": "J.Eng.Ind.",
        "inspire": "J.Eng.Ind."
      },
      {
        "cds": "Math.Gazette",
        "inspire": "Math.Gazette"
      },
      {
        "cds": "Meteorit. Planet. Sci.",
        "inspire": "Meteorit.Planet.Sci."
      },
      {
        "cds": "Int.J.Neurosci.",
        "inspire": "Int.J.Neurosci."
      },
      {
        "cds": "Nucl.Phys.",
        "inspire": "Nucl.Phys."
      },
      {
        "cds": "Phys.Appl.",
        "inspire": "Phys.Appl."
      },
      {
        "cds": "C. R. Acad. Sci. Paris",
        "inspire": "Compt.Rend.Math."
      },
      {
        "cds": "Nucl.Phys.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Ultrasonics",
        "inspire": "Ultrasonics"
      },
      {
        "cds": "AIP Conf. Proc.",
        "inspire": "AIP Conf.Proc."
      },
      {
        "cds": "Chaos",
        "inspire": "Chaos Solitons Fractals"
      },
      {
        "cds": "IEEE Trans. Ind. Appl.",
        "inspire": "IEEE Trans.Indust.Appl."
      },
      {
        "cds": "Hadron. J.",
        "inspire": "Hadronic J."
      },
      {
        "cds": "Int. J. Mod. Phys.",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Bull.Inst.Chem.Res.Kyoto",
        "inspire": "Bull.Inst.Chem.Res.Kyoto"
      },
      {
        "cds": "Arch.Sci.(Geneva)",
        "inspire": "Arch.Sci.(Geneva)"
      },
      {
        "cds": "Organometallics",
        "inspire": "Organometallics"
      },
      {
        "cds": "Rom. Rep. Phys.",
        "inspire": "Rom.Rep.Phys."
      },
      {
        "cds": "Proc. R. Soc. Lond.",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "Am. J. Phys.",
        "inspire": "Am.J.Phys."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Specul. Sci. Technol.",
        "inspire": "Speculations Sci.Tech."
      },
      {
        "cds": "Int. J. Quantum Chem.",
        "inspire": "Int.J.Quant.Chem."
      },
      {
        "cds": "DG-GA",
        "inspire": "DG-GA"
      },
      {
        "cds": "Comments Plasma Phys.Contr.Fusion",
        "inspire": "Comments Plasma Phys.Contr.Fusion"
      },
      {
        "cds": "Phys. Part. Nucl. Lett.",
        "inspire": "Part.Nucl.Lett."
      },
      {
        "cds": "Int. J. Eng. Sci.",
        "inspire": "Int.J.Eng.Sci."
      },
      {
        "cds": "Philos. Mag.",
        "inspire": "Phil.Mag."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Acta Phys. Hung. New Ser. Heavy Ion Phys.",
        "inspire": "Acta Phys.Hung."
      },
      {
        "cds": "Rev. Mex. Fis.",
        "inspire": "Rev.Mex.Fis."
      },
      {
        "cds": "Nucl. Sci. Eng.",
        "inspire": "Nucl.Sci.Eng."
      },
      {
        "cds": "AIChE J.",
        "inspire": "Aiche J."
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "Chin. J. Phys.",
        "inspire": "Chin.J.Phys."
      },
      {
        "cds": "J. Lond. Math. Soc.",
        "inspire": "Bull.London Math.Soc."
      },
      {
        "cds": "Compos. Math.",
        "inspire": "Compos.Math."
      },
      {
        "cds": "Appl. Phys.",
        "inspire": "Appl.Phys."
      },
      {
        "cds": "J.Astrophys.Astron.",
        "inspire": "J.Astrophys.Astron."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "J. Appl. Phys.",
        "inspire": "J.Appl.Phys."
      },
      {
        "cds": "Combust. Flame",
        "inspire": "Combust.Flame"
      },
      {
        "cds": "Bull. Astron. Soc. India",
        "inspire": "Bull.Astron.Soc.India"
      },
      {
        "cds": "J. Colloid Interface Sci.",
        "inspire": "J.Coll.Interface Sci."
      },
      {
        "cds": "IEEE Trans. Educ.",
        "inspire": "IEEE Trans.Educ."
      },
      {
        "cds": "Trans.Roy.Soc.Lond.",
        "inspire": "Trans.Roy.Soc.Lond."
      },
      {
        "cds": "Can. Math. Bull.",
        "inspire": "Can.Math.Bull."
      },
      {
        "cds": "Sov.Astron.",
        "inspire": "Sov.Astron.Lett."
      },
      {
        "cds": "J. Mater. Sci.",
        "inspire": "J.Molec.Spectrosc."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Programmirovanie",
        "inspire": "Programmirovanie"
      },
      {
        "cds": "Publ. Astron. Soc. Aust.",
        "inspire": "Proc.Astron.Soc.Austral."
      },
      {
        "cds": "Quantum Opt.",
        "inspire": "Quant.Opt."
      },
      {
        "cds": "Sov.Sci.Rev.",
        "inspire": "Sov.Sci.Rev."
      },
      {
        "cds": "Seminar Sophus Lie",
        "inspire": "Seminar Sophus Lie"
      },
      {
        "cds": "Astron. Nachr.",
        "inspire": "Astron.Nachr."
      },
      {
        "cds": "Expo.Math.",
        "inspire": "Expo.Math."
      },
      {
        "cds": "Topol. Appl.",
        "inspire": "Topology Appl."
      },
      {
        "cds": "Bull.Soc.Roy.Sci.Liege",
        "inspire": "Bull.Soc.Roy.Sci.Liege"
      },
      {
        "cds": "Sov. Astron.",
        "inspire": "Sov.Astron.Lett."
      },
      {
        "cds": "Nuovo Cimento",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "Optik",
        "inspire": "Optik"
      },
      {
        "cds": "Los Alamos Sci.",
        "inspire": "Los Alamos Sci."
      },
      {
        "cds": "Radiat.Eff.Lett.",
        "inspire": "Radiat.Eff.Lett."
      },
      {
        "cds": "Endeavour",
        "inspire": "Endeavour"
      },
      {
        "cds": "Proc. Camb. Philos. Soc.",
        "inspire": "Proc.Cambridge Phil.Soc."
      },
      {
        "cds": "Nucl. Instrum. Methods Phys. Res.",
        "inspire": "Nucl.Instrum.Meth."
      },
      {
        "cds": "Banach Cent. Publ.",
        "inspire": "Banach Center Publ."
      },
      {
        "cds": "J.Phys.Soc.Jap.Suppl.",
        "inspire": "J.Phys.Soc.Jap.Suppl."
      },
      {
        "cds": "Frontiers. Phys.",
        "inspire": "Front.Phys."
      },
      {
        "cds": "J. Electromagn. Waves Appl.",
        "inspire": "J.Electromagn.Waves Appl."
      },
      {
        "cds": "J. Frankl. Inst.",
        "inspire": "J.Franklin Inst."
      },
      {
        "cds": "Annals Math.",
        "inspire": "Annals Math."
      },
      {
        "cds": "Bull. Am. Phys. Soc.",
        "inspire": "Bull.Am.Phys.Soc."
      },
      {
        "cds": "Comput. J.",
        "inspire": "Comput.J."
      },
      {
        "cds": "Proc. Camb. Philos. Soc.",
        "inspire": "Proc.Cambridge Phil.Soc."
      },
      {
        "cds": "Astron. Astrophys. Rev.",
        "inspire": "Astron.Astrophys.Rev."
      },
      {
        "cds": "Cosmos Hist.",
        "inspire": "Cosmos Hist."
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Scanning",
        "inspire": "Scanning"
      },
      {
        "cds": "Rheologica Acta",
        "inspire": "Rheologica Acta"
      },
      {
        "cds": "Adv.Math.Supp.",
        "inspire": "Adv.Math.Supp."
      },
      {
        "cds": "Chin. Phys. Lett.",
        "inspire": "Chin.Phys.Lett."
      },
      {
        "cds": "Astrophys.J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "J. Knot Theory Ramif.",
        "inspire": "J.Knot Theor.Ramifications"
      },
      {
        "cds": "Mem.Soc.Roy.Sci.Liege",
        "inspire": "Mem.Soc.Roy.Sci.Liege"
      },
      {
        "cds": "Phys. Earth planet. Inter.",
        "inspire": "Phys.Earth Planet.Interiors"
      },
      {
        "cds": "Wiss.Fortschr.",
        "inspire": "Wiss.Fortschr."
      },
      {
        "cds": "Commun. Algebra",
        "inspire": "Commun.Algebra"
      },
      {
        "cds": "Europhys. Lett.",
        "inspire": "Europhys.Lett."
      },
      {
        "cds": "J. Appl. Math. Mech.",
        "inspire": "J.Appl.Math.Mech."
      },
      {
        "cds": "Fizika",
        "inspire": "Fizika"
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "J. Alloy. Compd.",
        "inspire": "J.Alloys Compounds"
      },
      {
        "cds": "Math. Sbornik",
        "inspire": "Mat.Sbornik"
      },
      {
        "cds": "CAMAC Bull.",
        "inspire": "CAMAC Bull."
      },
      {
        "cds": "Quantum Semiclass. Opt.",
        "inspire": "Quant.Semiclass.Opt."
      },
      {
        "cds": "Fortschr. Phys.",
        "inspire": "Fortsch.Phys."
      },
      {
        "cds": "Med. Phys.",
        "inspire": "Med.Phys."
      },
      {
        "cds": "Sov.J.Contemp.Math.Anal.",
        "inspire": "Sov.J.Contemp.Math.Anal."
      },
      {
        "cds": "Z. Angew. Phys.",
        "inspire": "Z.Angew.Phys."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Sol. Syst. Res.",
        "inspire": "Sol.Syst.Res."
      },
      {
        "cds": "Phys. At. Nucl.",
        "inspire": "Phys.Atom.Nucl."
      },
      {
        "cds": "Acta Chem.Scand.",
        "inspire": "Acta Chem.Scand."
      },
      {
        "cds": "Russ. Math. Surv.",
        "inspire": "Russ.Math.Surveys"
      },
      {
        "cds": "Nuovo Cimento, Lett.",
        "inspire": "Lett.Nuovo Cim."
      },
      {
        "cds": "Kodai Math. J.",
        "inspire": "Kodai Math.J."
      },
      {
        "cds": "Prog. Opt.",
        "inspire": "Prog.Opt"
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Compos. Math.",
        "inspire": "Compos.Math."
      },
      {
        "cds": "Sci. China",
        "inspire": "Sci.Sin."
      },
      {
        "cds": "Astron. J.",
        "inspire": "Astron.J."
      },
      {
        "cds": "Ariadne",
        "inspire": "Ariadne"
      },
      {
        "cds": "Adv. At. Mol. Phys.",
        "inspire": "Adv.Atom.Molec.Phys."
      },
      {
        "cds": "Planet. Space Sci.",
        "inspire": "Planet.Space Sci."
      },
      {
        "cds": "J. Approx. Theory",
        "inspire": "J.Approx.Theor."
      },
      {
        "cds": "Nuovo Cimento, Riv.",
        "inspire": "Riv.Nuovo Cim."
      },
      {
        "cds": "Phys. Rep.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Class.Quant.Grav.",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Period.Polytech.Phys.Nucl.Sci.",
        "inspire": "Period.Polytech.Phys.Nucl.Sci."
      },
      {
        "cds": "Future Gener. Comput. Syst.",
        "inspire": "Future Gener.Comput.Syst."
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "Exper.Neurol.Supp.",
        "inspire": "Exper.Neurol.Supp."
      },
      {
        "cds": "Phys. At. Nucl.",
        "inspire": "Phys.Atom.Nucl."
      },
      {
        "cds": "Nuovo Cim.",
        "inspire": "Riv.Nuovo Cim."
      },
      {
        "cds": "Cah. Phys.",
        "inspire": "Cahiers Phys."
      },
      {
        "cds": "Phys. Unserer Zeit",
        "inspire": "Phys.Unserer Zeit"
      },
      {
        "cds": "Rev.Fis.Apl.Instrum.",
        "inspire": "Rev.Fis.Apl.Instrum."
      },
      {
        "cds": "Addendum-ibid.",
        "inspire": "Addendum-ibid."
      },
      {
        "cds": "Am. Math. Soc. Transl.",
        "inspire": "Trans.Am.Math.Soc."
      },
      {
        "cds": "Computer",
        "inspire": "Computer"
      },
      {
        "cds": "Can. J. Math.",
        "inspire": "Can.J.Math."
      },
      {
        "cds": "New J. Phys.",
        "inspire": "New J.Phys."
      },
      {
        "cds": "Czechoslov. J. Phys. B",
        "inspire": "Czech.J.Phys."
      },
      {
        "cds": "Complexity",
        "inspire": "Complexity"
      },
      {
        "cds": "Not. Am. Math. Soc.",
        "inspire": "Not.Amer.Math.Soc."
      },
      {
        "cds": "Radiat. Environ. Biophys.",
        "inspire": "Radiat.Environ.Biophys."
      },
      {
        "cds": "Appl. Supercond.",
        "inspire": "Appl.Supercond."
      },
      {
        "cds": "J. Phys. Chem. Solids",
        "inspire": "J.Phys.Chem.Solids"
      },
      {
        "cds": "J. Mol. Spectrosc.",
        "inspire": "J.Molec.Spectrosc."
      },
      {
        "cds": "Proc. Phys. Soc. Lond.",
        "inspire": "Proc.Phys.Soc."
      },
      {
        "cds": "Proc. Phys. Math. Soc. Jpn.",
        "inspire": "Proc.Phys.Math.Soc.Jap."
      },
      {
        "cds": "Philos. Mag.",
        "inspire": "Phil.Mag."
      },
      {
        "cds": "Sov. Tech. Phys. Lett.",
        "inspire": "Sov.Tech.Phys.Lett."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Can. J. Math.",
        "inspire": "Can.J.Math."
      },
      {
        "cds": "J.Comp.Physiol.",
        "inspire": "J.Comp.Physiol."
      },
      {
        "cds": "Bull.Russ.Acad.Sci.Phys.",
        "inspire": "Bull.Russ.Acad.Sci.Phys."
      },
      {
        "cds": "Europhys. News",
        "inspire": "Europhys.News"
      },
      {
        "cds": "Topology",
        "inspire": "Topology"
      },
      {
        "cds": "Spaceflight",
        "inspire": "Spaceflight"
      },
      {
        "cds": "Adv. Electron. Electron Phys.",
        "inspire": "Adv.Electron.Electron Phys."
      },
      {
        "cds": "New Astron.",
        "inspire": "New Astron."
      },
      {
        "cds": "Nanotechnol.",
        "inspire": "Nanotechnol."
      },
      {
        "cds": "Arch.Math.Naturvidensk.",
        "inspire": "Arch.Math.Naturvidensk."
      },
      {
        "cds": "ANZIAM J.",
        "inspire": "ANZIAM J."
      },
      {
        "cds": "Cambridge Philos. Soc. Math. Proc.",
        "inspire": "Math.Proc.Cambridge Phil.Soc."
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "J. ACM Assoc. Comput. Mach.",
        "inspire": "J.Assoc.Comput.Machinery"
      },
      {
        "cds": "Mater.Glyas.Issled.",
        "inspire": "Mater.Glyas.Issled."
      },
      {
        "cds": "Chem.Eng.Sci.",
        "inspire": "Chem.Eng.Sci."
      },
      {
        "cds": "IEEE Trans. Magn.",
        "inspire": "IEEE Trans.Magnetics"
      },
      {
        "cds": "Phys. Rev. Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "Philos. Sci.",
        "inspire": "Phil.Sci."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Ann. Nucl. Energy",
        "inspire": "Annals Nucl.Energy"
      },
      {
        "cds": "Int. J. Geom. Meth. Mod. Phys.",
        "inspire": "Int.J.Geom.Meth.Mod.Phys."
      },
      {
        "cds": "Fiz. Elem. Chastits At. Yadra",
        "inspire": "Fiz.Elem.Chast.Atom.Yadra"
      },
      {
        "cds": "J. Synchrotron Radiat.",
        "inspire": "J.Synchrotron Radiat."
      },
      {
        "cds": "Russ. Math. Surv.",
        "inspire": "Russ.Math.Surveys"
      },
      {
        "cds": "Differ. Geom. Appl.",
        "inspire": "Differ.Geom.Appl."
      },
      {
        "cds": "Mod.Phys.Lett.",
        "inspire": "Mod.Phys.Lett."
      },
      {
        "cds": "J. Appl. Spectrosc.",
        "inspire": "J.Appl.Spectrosc."
      },
      {
        "cds": "Fortschr. Phys.",
        "inspire": "Fortsch.Phys."
      },
      {
        "cds": "JETP Lett.",
        "inspire": "JETP Lett."
      },
      {
        "cds": "J. Moscow Phys. Soc.",
        "inspire": "J.Moscow.Phys.Soc."
      },
      {
        "cds": "Annu. Rev. Astron. Astrophys.",
        "inspire": "Ann.Rev.Astron.Astrophys."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Int. Astron. Union Circ.",
        "inspire": "IAU Circ."
      },
      {
        "cds": "C. R. Acad. Bulg. Sci.",
        "inspire": "Compt.Rend.Acad.Sci.Bulg."
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Astrophys. Space Sci.",
        "inspire": "Astrophys.Space Sci."
      },
      {
        "cds": "SIAM J. Appl. Math.",
        "inspire": "SIAM J.Appl.Math."
      },
      {
        "cds": "Adv.Polymer Sci.",
        "inspire": "Adv.Polymer Sci."
      },
      { "cds": "Nucl. Instrum. Methods",
        "inspire": "Nucl.Instrum.Methods" },
      {
        "cds": "J.Molec.Evol.",
        "inspire": "J.Molec.Evol."
      },
      {
        "cds": "Microelectron. Reliab.",
        "inspire": "Microelectron. Reliab."
      },
      {
        "cds": "Int. J. Commun. Syst.",
        "inspire": "Int.J.Commun.Syst."
      },
      {
        "cds": "Prog. Theor. Phys., Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "J. Geophys. Res. A",
        "inspire": "J.Geophys.Res."
      },
      {
        "cds": "Arch.Philos.",
        "inspire": "Arch.Philos."
      },
      {
        "cds": "SIAM J. Numer. Anal.",
        "inspire": "SIAM J.Numer.Anal."
      },
      {
        "cds": "Synthetic Metals",
        "inspire": "Synth.Metals"
      },
      {
        "cds": "Aequat.Math.",
        "inspire": "Aequat.Math."
      },
      {
        "cds": "Rev. Mex. Fis.",
        "inspire": "Rev.Mex.Fis."
      },
      {
        "cds": "Phys. Rev.",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Math. Ann.",
        "inspire": "Math.Ann."
      },
      {
        "cds": "Am. Sci.",
        "inspire": "Am.Sci."
      },
      {
        "cds": "Adv. Cryog. Eng.",
        "inspire": "Adv.Cryog.Eng."
      },
      {
        "cds": "Spektrum Wiss.",
        "inspire": "Spektrum Wiss."
      },
      {
        "cds": "Arch. Hist. Exact Sci.",
        "inspire": "Arch.Hist.Ex.Sci."
      },
      {
        "cds": "ESA Spec. Publ.",
        "inspire": "ESA Spec.Publ."
      },
      {
        "cds": "IEEE Trans. Adv. Packag.",
        "inspire": "IEEE Trans.Adv.Packag."
      },
      {
        "cds": "Hadron. J., Suppl.",
        "inspire": "Hadronic J.Suppl."
      },
      {
        "cds": "Asymp.Anal.",
        "inspire": "Asymp.Anal."
      },
      {
        "cds": "Part. World",
        "inspire": "Part.World"
      },
      {
        "cds": "Dokl.Bulg.Akad.Nauk",
        "inspire": "Dokl.Bulg.Akad.Nauk"
      },
      {
        "cds": "Appl. Math. Optim.",
        "inspire": "Appl.Math.Optim."
      },
      {
        "cds": "Cambridge Philos. Soc. Math. Proc.",
        "inspire": "Math.Proc.Cambridge Phil.Soc."
      },
      {
        "cds": "FUNCT-AN",
        "inspire": "FUNCT-AN"
      },
      {
        "cds": "Fuzzy Sets Syst.",
        "inspire": "Fuzzy Sets Syst."
      },
      {
        "cds": "Kinet.Katal.",
        "inspire": "Kinet.Katal."
      },
      {
        "cds": "Methods Appl. Anal.",
        "inspire": "Methods Appl.Anal."
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "Rep. Math. Phys.",
        "inspire": "Rept.Math.Phys."
      },
      {
        "cds": "J. Appl. Phys.",
        "inspire": "J.Appl.Phys."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Bol. Soc. Mat. Mex.",
        "inspire": "Bol.Soc.Mat.Mexicana"
      },
      {
        "cds": "J. Geophys. Res.",
        "inspire": "J.Geophys.Res."
      },
      {
        "cds": "J.Appl.Probab.",
        "inspire": "J.Appl.Probab."
      },
      {
        "cds": "Environ. Sci. Technol.",
        "inspire": "Environ.Sci.Tech."
      },
      {
        "cds": "Phys. Fluids",
        "inspire": "Phys.Fluids"
      },
      {
        "cds": "Space Sci. Rev.",
        "inspire": "Space Sci.Rev."
      },
      {
        "cds": "Sov.Radiochem.",
        "inspire": "Sov.Radiochem."
      },
      {
        "cds": "Integral Equ. Oper. Theor.",
        "inspire": "Integral Equ. Oper. Theor."
      },
      {
        "cds": "J. Synchrotron Radiat.",
        "inspire": "J.Synchrotron Radiat."
      },
      {
        "cds": "Plasma Phys. Control. Fusion",
        "inspire": "Plasma Phys.Control.Fusion"
      },
      {
        "cds": "Phys. Bl.",
        "inspire": "Phys.Bl."
      },
      {
        "cds": "IEEE/ACM Trans. Netw.",
        "inspire": "IEEE/ACM Trans.Netw."
      },
      {
        "cds": "Yad. Fiz.",
        "inspire": "Yad.Fiz."
      },
      {
        "cds": "J. Chem. Soc. Faraday Trans.",
        "inspire": "J.Chem.Soc.Faraday Trans."
      },
      {
        "cds": "Mat. Fys. Medd. K. Dan. Vidensk. Selsk.",
        "inspire": "Kong.Dan.Vid.Sel.Mat.Fys.Med."
      },
      {
        "cds": "Biometrika",
        "inspire": "Biometrika"
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Uchuusen Kenkyuu",
        "inspire": "Uchuusen Kenkyuu"
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "J.Ultrasound Med.",
        "inspire": "J.Ultrasound Med."
      },
      {
        "cds": "Lect. Notes Comput. Sci.",
        "inspire": "Lect.Notes Comput.Sci."
      },
      {
        "cds": "Chaos Solitons Fractals",
        "inspire": "Chaos Solitons Fractals"
      },
      {
        "cds": "J. Opt. Technol.",
        "inspire": "J.Opt.Technol."
      },
      {
        "cds": "J. Differ. Equ.",
        "inspire": "J.Diff.Eq."
      },
      {
        "cds": "Part. Accel.",
        "inspire": "Part.Accel."
      },
      {
        "cds": "Rev. Mod. Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "J. Vac. Sci. Technol.",
        "inspire": "J.Vac.Sci.Tech."
      },
      {
        "cds": "Phys. Essays",
        "inspire": "Phys.Essays"
      },
      {
        "cds": "ICFA Beam Dyn. Newsl.",
        "inspire": "ICFA Beam Dyn.Newslett."
      },
      {
        "cds": "ICFA Instrum. Bull.",
        "inspire": "ICFA Instrum.Bull."
      },
      {
        "cds": "J. Phys. Soc. Jpn. Suppl.",
        "inspire": "J.Phys.Soc.Jap.Suppl."
      },
      {
        "cds": "Adv. Nucl. Phys.",
        "inspire": "Adv.Nucl.Phys."
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "Publ. Math. IHES",
        "inspire": "Inst.Hautes Etudes Sci.Publ.Math."
      },
      {
        "cds": "Vestn.Akad.Nauk SSSR",
        "inspire": "Vestn.Akad.Nauk SSSR"
      },
      {
        "cds": "Ann. Phys. (Les Ulis)",
        "inspire": "Annales Phys.(France)"
      },
      {
        "cds": "Ber. Bunsenges. Phys. Chem.",
        "inspire": "Ber.Bunsenges.Phys.Chem."
      },
      {
        "cds": "Phys. Rev.",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "Comput. J.",
        "inspire": "Comput.J."
      },
      {
        "cds": "SIGMA, Symmetry Integrability Geom. Methods Appl.",
        "inspire": "SIGMA"
      },
      {
        "cds": "IEEE J. Quantum Electron.",
        "inspire": "IEEE J.Quant.Electron."
      },
      {
        "cds": "Comput. Phys.",
        "inspire": "Comput.Phys."
      },
      {
        "cds": "Indust.Eng.Chem.",
        "inspire": "Indust.Eng.Chem."
      },
      {
        "cds": "Ann. Phys. (Les Ulis)",
        "inspire": "Annales Phys.(France)"
      },
      {
        "cds": "Naturwiss.Rundsch.",
        "inspire": "Naturwiss."
      },
      {
        "cds": "J. Cell Biol.",
        "inspire": "J.Cell Biol."
      },
      {
        "cds": "J. Phys. Radium",
        "inspire": "J.Phys.Radium"
      },
      {
        "cds": "Publ. Astron. Soc. Pac.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "Aequ. Math.",
        "inspire": "Aequat.Math."
      },
      {
        "cds": "Nuovo Cimento",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "Hyperfine Interact.",
        "inspire": "Hyperfine Interact."
      },
      {
        "cds": "J. Vac. Sci. Technol. A",
        "inspire": "J.Vac.Sci.Technol.A Vac.Surf.Films"
      },
      {
        "cds": "Opt. Commun.",
        "inspire": "Opt.Commun."
      },
      {
        "cds": "Surv. High Energy Phys.",
        "inspire": "Surveys High Energ.Phys."
      },
      {
        "cds": "Manuscr. Math.",
        "inspire": "Manuscr.Math."
      },
      {
        "cds": "Rept.Fac.Sci.Shizuoka Univ.",
        "inspire": "Rept.Fac.Sci.Shizuoka Univ."
      },
      {
        "cds": "Constr. Approx.",
        "inspire": "Constructive Approx."
      },
      {
        "cds": "Analyst",
        "inspire": "Analyst"
      },
      {
        "cds": "Nucl. Instrum. Methods Phys. Res., A",
        "inspire": "Nucl.Instrum.Meth."
      },
      {
        "cds": "Math. Sbornik",
        "inspire": "Mat.Sbornik"
      },
      {
        "cds": "Physics News India",
        "inspire": "Physics News India"
      },
      {
        "cds": "J.Math.Psychol.",
        "inspire": "J.Math.Psychol."
      },
      {
        "cds": "Proc.Roy.Irish Acad.(Sect.A)",
        "inspire": "Proc.Roy.Irish Acad.(Sect.A)"
      },
      {
        "cds": "Surf. Interface Anal.",
        "inspire": "Surf. Interface Anal."
      },
      {
        "cds": "Adv. Theor. Phys.",
        "inspire": "Adv.Theor.Phys."
      },
      {
        "cds": "Chin. Phys. Lett.",
        "inspire": "Chin.Phys.Lett."
      },
      {
        "cds": "N. Y. J. Math.",
        "inspire": "New York J.Math."
      },
      {
        "cds": "Num. Algorithms",
        "inspire": "Num. Algorithms"
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "Ann. Inst. Henri Poincaré",
        "inspire": "Ann.Inst.Henri Poincare"
      },
      {
        "cds": "Fluct. Noise Lett.",
        "inspire": "Fluct.Noise Lett."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "IEEE Trans. Nucl. Sci.",
        "inspire": "IEEE Trans.Nucl.Sci."
      },
      {
        "cds": "Geoexploration",
        "inspire": "Geoexploration"
      },
      {
        "cds": "J. Nonlinear Math. Phys.",
        "inspire": "J.Nonlin.Math.Phys."
      },
      {
        "cds": "Math. Comput.",
        "inspire": "Math.Comput."
      },
      {
        "cds": "Fluid Dyn. Res.",
        "inspire": "Fluid Dynamics Res."
      },
      {
        "cds": "Comments Nucl.Part.Phys.",
        "inspire": "Comments Nucl.Part.Phys."
      },
      {
        "cds": "New Astron.",
        "inspire": "New Astron."
      },
      {
        "cds": "Opt. Photonics News",
        "inspire": "Opt.Phontonics News"
      },
      {
        "cds": "J.Math.(Kyoto)",
        "inspire": "J.Math.(Kyoto)"
      },
      {
        "cds": "Ann. Fac. Sci. (Toulouse)",
        "inspire": "Annales Fac.Sci.Toulouse"
      },
      {
        "cds": "Int. J. Mass Spectrom.",
        "inspire": "Int.J.Mass Spectr.Ion Process."
      },
      {
        "cds": "Linux J.",
        "inspire": "Linux J."
      },
      {
        "cds": "Commun.Phys.",
        "inspire": "Commun.Phys."
      },
      {
        "cds": "Electron. Eng.",
        "inspire": "Electron.Eng."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Nuovo Cimento",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "J.Phys.(France) Suppl.",
        "inspire": "J.Phys.(France) Suppl."
      },
      {
        "cds": "Eur. J. Phys.",
        "inspire": "Eur.J.Phys."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "J.Astrophys.Astron."
      },
      {
        "cds": "ESA J.",
        "inspire": "ESA J."
      },
      {
        "cds": "Rev. Sci. Instrum.",
        "inspire": "Rev.Sci.Instrum."
      },
      {
        "cds": "Centaurus",
        "inspire": "Centaurus"
      },
      {
        "cds": "Instrum. Exp. Tech.",
        "inspire": "Instrum.Exp.Tech."
      },
      {
        "cds": "Lett. Math. Phys.",
        "inspire": "Lett.Math.Phys."
      },
      {
        "cds": "Radiat.Eff.Def.Solids",
        "inspire": "Radiat.Eff.Def.Solids"
      },
      {
        "cds": "Mater. Res. Bull.",
        "inspire": "Mater.Res.Bull."
      },
      {
        "cds": "Celest. Mech.",
        "inspire": "Celestial Mech."
      },
      {
        "cds": "Selecta Math.",
        "inspire": "Sel.Math.Sov."
      },
      {
        "cds": "Nucl. Phys. A",
        "inspire": "Nucl.Phys."
      },
      {
        "cds": "Mat.-Fys. Medd.",
        "inspire": "Kong.Dan.Vid.Sel.Mat.Fys.Med."
      },
      {
        "cds": "Kong.Dan.Vid.Sel.Mat.Fys.Med.",
        "inspire": "Kong.Dan.Vid.Sel.Mat.Fys.Med."
      },
      {
        "cds": "Astropart. Phys.",
        "inspire": "Astropart.Phys."
      },
      {
        "cds": "Z.Chem.",
        "inspire": "Z.Chem."
      },
      {
        "cds": "Phys. Plasmas",
        "inspire": "Phys.Plasmas"
      },
      {
        "cds": "Probl.Issl.Vsel.",
        "inspire": "Probl.Issl.Vsel."
      },
      {
        "cds": "Bull.Lebedev Phys.Inst.",
        "inspire": "Bull.Lebedev Phys.Inst."
      },
      {
        "cds": "Pure Appl. Chem.",
        "inspire": "Pure Appl.Chem."
      },
      {
        "cds": "J. Opt.",
        "inspire": "J.Optics"
      },
      {
        "cds": "Cell Biophys.",
        "inspire": "Cell Biophys."
      },
      {
        "cds": "Funct. Anal. Appl.",
        "inspire": "Funct.Anal.Appl."
      },
      {
        "cds": "RCA Rev.",
        "inspire": "RCA Rev."
      },
      {
        "cds": "MRS Bull.",
        "inspire": "MRS Bull."
      },
      {
        "cds": "Nucl.Sci.Appl.",
        "inspire": "Nucl.Sci.Appl."
      },
      {
        "cds": "Aust. J. Phys.",
        "inspire": "Austral.J.Phys."
      },
      {
        "cds": "The Lancet",
        "inspire": "Lancet"
      },
      {
        "cds": "Nucl. Instrum. Methods Phys. Res., A",
        "inspire": "Nucl.Instrum.Meth."
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "Mol. Phys.",
        "inspire": "Mol.Phys."
      },
      {
        "cds": "J. Phys. I",
        "inspire": "J.Phys.I(France)"
      },
      {
        "cds": "Astrofiz. (USSR)",
        "inspire": "Astrofiz."
      },
      {
        "cds": "J. Phys. B",
        "inspire": "J.Phys.B"
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "J. Phys. A",
        "inspire": "J.Phys.A"
      },
      {
        "cds": "J. Phys. D",
        "inspire": "J.Phys.D"
      },
      {
        "cds": "J. Phys. E",
        "inspire": "J.Phys.E"
      },
      {
        "cds": "Math.Today",
        "inspire": "Math.Today"
      },
      {
        "cds": "Astropart. Phys.",
        "inspire": "Astropart.Phys."
      },
      {
        "cds": "Astronomy",
        "inspire": "ASTRO-PH"
      },
      {
        "cds": "J. Phys. USSR",
        "inspire": "J.Phys.(USSR)"
      },
      {
        "cds": "Prog. Math.",
        "inspire": "Prog.Math."
      },
      {
        "cds": "Chem. Phys.",
        "inspire": "Chem.Phys."
      },
      {
        "cds": "Sov. Math. - Dokl.",
        "inspire": "Sov.Math.Dokl."
      },
      {
        "cds": "Sov.Sci.Rev.C.Math.Phys.Rev.",
        "inspire": "Sov.Sci.Rev.C.Math.Phys.Rev."
      },
      {
        "cds": "Cosmic Electrodyn.",
        "inspire": "Cosmic Electrodyn."
      },
      {
        "cds": "Biochim. Biophys. Acta",
        "inspire": "Biochim.Biophys.Acta"
      },
      {
        "cds": "Publ. Res. Inst. Math. Sci.",
        "inspire": "Publ.Res.Inst.Math.Sci.Kyoto"
      },
      {
        "cds": "Acta Biol.Hung.",
        "inspire": "Acta Biol.Hung."
      },
      {
        "cds": "Fizika",
        "inspire": "Fizika"
      },
      {
        "cds": "J.Br.Interplanet.Soc.",
        "inspire": "J.Br.Interplanet.Soc."
      },
      {
        "cds": "J.Atmos.Sci.",
        "inspire": "J.Atmos.Sci."
      },
      {
        "cds": "BioSystems",
        "inspire": "Biosyst."
      },
      {
        "cds": "Condens. Matter Theor.",
        "inspire": "Condensed Matter Theor."
      },
      {
        "cds": "Astron. Lett.",
        "inspire": "Astron.Lett."
      },
      {
        "cds": "Surv. High Energy Phys.",
        "inspire": "Surveys High Energ.Phys."
      },
      {
        "cds": "Fundam. Math.",
        "inspire": "Fund.Math."
      },
      {
        "cds": "Prog. Energy Combust. Sci.",
        "inspire": "Prog.Energy Combust.Sci."
      },
      {
        "cds": "Elektrokhim.",
        "inspire": "Elektrokhim."
      },
      {
        "cds": "Metrolog.",
        "inspire": "Metrologia"
      },
      {
        "cds": "Phys. - Dokl.",
        "inspire": "Phys.Dokl."
      },
      {
        "cds": "Acta Phys. Austriaca",
        "inspire": "Acta Phys.Austriaca"
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "J. Appl. Phys.",
        "inspire": "J.Appl.Phys."
      },
      {
        "cds": "J. Stat. Phys.",
        "inspire": "J.Statist.Phys."
      },
      {
        "cds": "Astron. Rep.",
        "inspire": "Astron.Rep."
      },
      {
        "cds": "J. Math. Biol.",
        "inspire": "J.Math.Biol."
      },
      {
        "cds": "Inorg. Chem.",
        "inspire": "Inorg.Chem."
      },
      {
        "cds": "Pure Appl. Opt.",
        "inspire": "Pure Appl.Opt."
      },
      {
        "cds": "Math. Ann.",
        "inspire": "Math.Ann."
      },
      {
        "cds": "Comput. Phys. Rep.",
        "inspire": "Comput.Phys.Rept."
      },
      {
        "cds": "Anales Acad.Bras.Cienc.",
        "inspire": "Bull.Atom.Sci."
      },
      {
        "cds": "Phys. Lett. B",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Proc. Natl. Acad. Sci. U. S. A.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "J.Phys.Conf.Ser.",
        "inspire": "J.Phys.Conf.Ser."
      },
      {
        "cds": "Asterisque",
        "inspire": "Asterisque"
      },
      {
        "cds": "Prib. Tekh. Eksp.",
        "inspire": "Prib.Tekh.Eksp."
      },
      {
        "cds": "Sov.Math.",
        "inspire": "Sov.Math."
      },
      {
        "cds": "Astrophys. Space Sci.",
        "inspire": "Astrophys.Space Sci."
      },
      {
        "cds": "J. Phys.",
        "inspire": "J.Phys.(France)"
      },
      {
        "cds": "J.Mind.Behav.",
        "inspire": "J.Mind.Behav."
      },
      {
        "cds": "Sky Telesc.",
        "inspire": "Sky Telesc."
      },
      {
        "cds": "Quantum Opt.",
        "inspire": "Quant.Opt."
      },
      {
        "cds": "J. R. Stat. Soc., B",
        "inspire": "J.Roy.Statist.Soc.B"
      },
      {
        "cds": "J. Opt. Soc. Am.",
        "inspire": "J.Opt.Soc.Am."
      },
      {
        "cds": "Nuovo Cimento, Lett.",
        "inspire": "Lett.Nuovo Cim."
      },
      {
        "cds": "Astrophys. Space Sci.",
        "inspire": "Astrophys.Space Sci."
      },
      {
        "cds": "Opt. Express",
        "inspire": "Opt.Express"
      },
      {
        "cds": "Contin. Mech. Thermodyn.",
        "inspire": "Contin.Mech.Thermodyn."
      },
      {
        "cds": "Hyperfine Interact.",
        "inspire": "Hyperfine Interact."
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Atompraxis",
        "inspire": "Atompraxis"
      },
      {
        "cds": "Astron. Astrophys. Rev.",
        "inspire": "Astron.Astrophys.Rev."
      },
      {
        "cds": "Bull. Spec. Astrophys. Obs.",
        "inspire": "Bull.Spec.Astrophys.Obs.N.Caucasus"
      },
      {
        "cds": "Topol. Appl.",
        "inspire": "Topology Appl."
      },
      {
        "cds": "Phys. Lett.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Electrochim. Acta",
        "inspire": "Electrochim.Acta"
      },
      {
        "cds": "Nature",
        "inspire": "Nature"
      },
      {
        "cds": "Vestn.Moskov.Univ.(Mat.Mekh.)",
        "inspire": "Vestn.Moskov.Univ.(Mat.Mekh.)"
      },
      {
        "cds": "Poverkhnost",
        "inspire": "Poverkhnost"
      },
      {
        "cds": "J. Stat. Phys.",
        "inspire": "J.Statist.Phys."
      },
      {
        "cds": "J. Br. Astron. Assoc.",
        "inspire": "J.Br.Astron.Assoc."
      },
      {
        "cds": "Pattern Recognition",
        "inspire": "Pattern Recognition"
      },
      {
        "cds": "Indian J. Phys.",
        "inspire": "Indian J.Phys."
      },
      {
        "cds": "Phys.Rept.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "J. Geophys. Res.",
        "inspire": "J.Geophys.Res."
      },
      {
        "cds": "Sci.Pap.Inst.Phys.Chem.Res.",
        "inspire": "Sci.Pap.Inst.Phys.Chem.Res."
      },
      {
        "cds": "Astrophys.J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "J.Mecan.",
        "inspire": "J.Mecan."
      },
      {
        "cds": "Vak.-Tech.",
        "inspire": "Vak.Tech."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Anales Acad.Bras.Cienc.",
        "inspire": "Bull.Atom.Sci."
      },
      {
        "cds": "Ann. Sci. Ec. Norm. Supér.",
        "inspire": "Annales Sci.Ecole Norm.Sup."
      },
      {
        "cds": "Izv.Akad.Nauk Arm.SSR Fiz.",
        "inspire": "Izv.Akad.Nauk Arm.SSR Fiz."
      },
      {
        "cds": "Selecta Math.",
        "inspire": "Sel.Math.Sov."
      },
      {
        "cds": "Texas Q.",
        "inspire": "Texas Q."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Bull.Can.Nucl.Soc.",
        "inspire": "Bull.Can.Nucl.Soc."
      },
      {
        "cds": "Port. Math.",
        "inspire": "Port.Math."
      },
      {
        "cds": "J. Algebra",
        "inspire": "J.Algebra"
      },
      {
        "cds": "Physiol.Plant.",
        "inspire": "Physiol.Plant."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Phys. Rev.",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "J.Molec.Electron.",
        "inspire": "J.Molec.Electron."
      },
      {
        "cds": "Philos. Trans. R. Soc. Lond., A",
        "inspire": "Trans.Cambridge Phil.Soc."
      },
      {
        "cds": "Eur. J. Mech. B",
        "inspire": "Eur.J.Mech.B Fluids"
      },
      {
        "cds": "Methods Comput. Phys.",
        "inspire": "Methods Comput.Phys."
      },
      {
        "cds": "Phys.Rept.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Immunity",
        "inspire": "Immunity"
      },
      {
        "cds": "J. Parallel Distrib. Comput.",
        "inspire": "J.Parallel Distrib.Comput."
      },
      {
        "cds": "Arch. Rational Mech. Analys.",
        "inspire": "Arch.Ration.Mech.Anal."
      },
      {
        "cds": "Int. J. Radiat. Biol.",
        "inspire": "Int.J.Radiat.Biol."
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Zap. Nauchnykh Semin. POMI",
        "inspire": "Zap.Nauchn.Semin."
      },
      {
        "cds": "Math. Ann.",
        "inspire": "Math.Ann."
      },
      {
        "cds": "Commun. Theor. Phys.",
        "inspire": "Commun.Theor.Phys."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Metall. Trans. A",
        "inspire": "Metall.Trans."
      },
      {
        "cds": "Vacuum",
        "inspire": "Vacuum"
      },
      {
        "cds": "Opt.Laser Tech.",
        "inspire": "Opt.Laser Tech."
      },
      {
        "cds": "Sci.Rept.Tohoku Univ.(Ser.A)",
        "inspire": "Sci.Rept.Tohoku Univ.(Ser.A)"
      },
      {
        "cds": "Nucl. Instrum. Methods Phys. Res., A",
        "inspire": "Nucl.Instrum.Meth."
      },
      {
        "cds": "Sci. China",
        "inspire": "Sci.Sin."
      },
      {
        "cds": "Diss.Math.",
        "inspire": "Diss.Math."
      },
      {
        "cds": "Astrophys. Space Sci. libr.",
        "inspire": "Astrophys.Space Sci.Libr."
      },
      {
        "cds": "Rev. Mex. Fis., Suppl.",
        "inspire": "Rev.Mex.Fis."
      },
      {
        "cds": "Eur. J. Phys.",
        "inspire": "Eur.J.Phys."
      },
      {
        "cds": "J.Math.Phys.Sci.",
        "inspire": "J.Math.Phys.Sci."
      },
      {
        "cds": "Appl. Stat.",
        "inspire": "Appl.Statist."
      },
      {
        "cds": "Bull. Am. Phys. Soc.",
        "inspire": "Bull.Am.Phys.Soc."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Teor. Mat. Fiz.",
        "inspire": "Teor.Mat.Fiz."
      },
      {
        "cds": "Braz. J. Phys.",
        "inspire": "Braz.J.Phys."
      },
      {
        "cds": "Izv.Akad.Nauk Ser.Fiz.",
        "inspire": "Izv.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "Celest. Mech.",
        "inspire": "Celestial Mech."
      },
      {
        "cds": "Electron. Lett.",
        "inspire": "Electron.Lett."
      },
      {
        "cds": "Fiz.Tidsskrift",
        "inspire": "Fiz.Tidsskrift"
      },
      {
        "cds": "Phys. Plasmas",
        "inspire": "Phys.Plasmas"
      },
      {
        "cds": "Phys. Teach.",
        "inspire": "Phys.Teacher"
      },
      {
        "cds": "Commun. ACM",
        "inspire": "Commun.ACM"
      },
      {
        "cds": "Bull. Am. Astron. Soc.",
        "inspire": "Bull.Am.Astron.Soc."
      },
      {
        "cds": "Lect. Notes Math.",
        "inspire": "Lect.Notes Math."
      },
      {
        "cds": "Environ. Sci. Technol.",
        "inspire": "Environ.Sci.Tech."
      },
      {
        "cds": "Indian J. Pure Appl. Math.",
        "inspire": "Indian J.Pure Appl.Math."
      },
      {
        "cds": "JETP Lett.",
        "inspire": "JETP Lett."
      },
      {
        "cds": "Chem. Biol.",
        "inspire": "Chem.Biol."
      },
      {
        "cds": "Philos. Trans. R. Soc.",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "Duke Math. J.",
        "inspire": "Duke Math.J."
      },
      {
        "cds": "Tectonophys.",
        "inspire": "Tectonophys."
      },
      {
        "cds": "Found. Phys.",
        "inspire": "Found.Phys."
      },
      {
        "cds": "Oyo Butsuri",
        "inspire": "Oyo Butsuri"
      },
      {
        "cds": "J. Fluid Mech.",
        "inspire": "J.Fluid Mech."
      },
      {
        "cds": "Nucl. Instrum. Methods Phys. Res.",
        "inspire": "Nucl.Instrum.Meth."
      },
      {
        "cds": "Int.J.Numer.Meth.Eng.",
        "inspire": "Int.J.Numer.Meth.Eng."
      },
      {
        "cds": "Bull. Am. Math. Soc.",
        "inspire": "Bull.Am.Math.Soc."
      },
      {
        "cds": "Nucl. Part. Phys.",
        "inspire": "Comments Nucl.Part.Phys."
      },
      {
        "cds": "J. Nonlinear Math. Phys.",
        "inspire": "J.Nonlin.Math.Phys."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Anal. Chim. Acta",
        "inspire": "Anal.Chim.Acta"
      },
      {
        "cds": "Sov.Astron.Lett.",
        "inspire": "Sov.Astron.Lett."
      },
      {
        "cds": "Prog. Theor. Phys., Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "Discret. Contin. Dyn. Syst.",
        "inspire": "Discrete Contin.Dyn.Syst."
      },
      {
        "cds": "Mol. Phys.",
        "inspire": "Mol.Phys."
      },
      {
        "cds": "Contemp. Math.",
        "inspire": "Contemp.Math."
      },
      {
        "cds": "J. Funct. Anal.",
        "inspire": "J.Funct.Anal."
      },
      {
        "cds": "J. Nucl. Energy",
        "inspire": "J.Nucl.Energy"
      },
      {
        "cds": "Ann. Math.",
        "inspire": "Annals Math."
      },
      {
        "cds": "J. Nucl. Mater",
        "inspire": "J.Nucl.Mater."
      },
      {
        "cds": "Phil.Trans.Roy.Soc.Lond.",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Prog. Phys.",
        "inspire": "Prog.Phys."
      },
      {
        "cds": "Ann. Stat.",
        "inspire": "Annals Statist."
      },
      {
        "cds": "At. Data Nucl. Data Tables",
        "inspire": "Atom.Data Nucl.Data Tabl."
      },
      {
        "cds": "Sci.Rept.Saitama Univ.",
        "inspire": "Sci.Rept.Saitama Univ."
      },
      {
        "cds": "Proc. R. Soc. Lond.",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "Proc.Edin.Math.Soc.",
        "inspire": "Proc.Edin.Math.Soc."
      },
      {
        "cds": "Abh.Akad.Wiss.DDR",
        "inspire": "Abh.Akad.Wiss.DDR"
      },
      {
        "cds": "Radiat. Phys. Chem.",
        "inspire": "Radiat.Phys.Chem."
      },
      {
        "cds": "Found. Phys. Lett.",
        "inspire": "Found.Phys.Lett."
      },
      {
        "cds": "Adv. Geom.",
        "inspire": "Adv.Geom."
      },
      {
        "cds": "J. Comput. Appl. Math.",
        "inspire": "J.Comput.Appl.Math."
      },
      {
        "cds": "Transform. Groups",
        "inspire": "Transform. Groups"
      },
      {
        "cds": "Kosmos",
        "inspire": "Kosmos"
      },
      {
        "cds": "SLAC Beam Line",
        "inspire": "SLAC Beam Line"
      },
      {
        "cds": "Hadron. J.",
        "inspire": "Hadronic J."
      },
      {
        "cds": "Astron. Astrophys. Trans.",
        "inspire": "Astron.Astrophys.Trans."
      },
      {
        "cds": "J. Neurophysio.",
        "inspire": "J.Neurophysiol."
      },
      {
        "cds": "Moon Planets",
        "inspire": "Moon Planets"
      },
      {
        "cds": "Dokl. Akad. Nauk SSSR",
        "inspire": "Dokl.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Am. J. Phys.",
        "inspire": "Am.J.Phys."
      },
      {
        "cds": "J. Reine Angew. Math.",
        "inspire": "J.Reine Angew.Math."
      },
      {
        "cds": "Liv. Rev. Rel.",
        "inspire": "Living Rev.Rel."
      },
      {
        "cds": "Prog.Nucl.Med.",
        "inspire": "Prog.Nucl.Med."
      },
      {
        "cds": "Geom. Funct. Anal.",
        "inspire": "Geom.Funct.Anal."
      },
      {
        "cds": "Adv. Inorg. Chem.",
        "inspire": "Adv.Inorg.Chem."
      },
      {
        "cds": "Acta Phys. Austriaca, Suppl.",
        "inspire": "Acta Phys.Austriaca Suppl."
      },
      {
        "cds": "Ann. Phys. (San Diego)",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Comput. J.",
        "inspire": "Comput.J."
      },
      {
        "cds": "Anales AFA",
        "inspire": "Anales AFA"
      },
      {
        "cds": "Stoch.Stoch.Rep.",
        "inspire": "Stoch.Stoch.Rep."
      },
      {
        "cds": "Phys.Chem.Glass.",
        "inspire": "Phys.Chem.Glass."
      },
      {
        "cds": "J.Exper.Biol.",
        "inspire": "J.Exper.Biol."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Manuscr. Math.",
        "inspire": "Manuscr.Math."
      },
      {
        "cds": "Phys. Today",
        "inspire": "Phys.Today"
      },
      {
        "cds": "Eclairage Elect.",
        "inspire": "Eclairage Elect."
      },
      {
        "cds": "Eng. Fail. Anal.",
        "inspire": "Eng.Fail.Anal."
      },
      {
        "cds": "Bull.Abastumani Astrophys.Obs.",
        "inspire": "Bull.Abastumani Astrophys.Obs."
      },
      {
        "cds": "Lett.Nuovo Cim.",
        "inspire": "Lett.Nuovo Cim."
      },
      {
        "cds": "Fiz. Elem. Chastits At. Yadra",
        "inspire": "Fiz.Elem.Chast.Atom.Yadra"
      },
      {
        "cds": "J. Differ. Equ.",
        "inspire": "J.Diff.Eq."
      },
      {
        "cds": "Phys. At. Nucl.",
        "inspire": "Phys.Atom.Nucl."
      },
      {
        "cds": "IEEE Trans. Neural Netw.",
        "inspire": "IEEE Trans.Neural Networks"
      },
      {
        "cds": "Brit.J.Psych.",
        "inspire": "Brit.J.Psych."
      },
      {
        "cds": "J. Differ. Geom.",
        "inspire": "J.Diff.Geom."
      },
      {
        "cds": "Int. Math. Res. Not.",
        "inspire": "Int.Math.Res.Not."
      },
      {
        "cds": "Crit.Rev.Anal.Chem.",
        "inspire": "Crit.Rev.Anal.Chem."
      },
      {
        "cds": "Rep. Prog. Phys.",
        "inspire": "Rept.Prog.Phys."
      },
      {
        "cds": "Phase Trans.",
        "inspire": "Phase Transitions"
      },
      {
        "cds": "Helv. Phys. Acta",
        "inspire": "Helv.Phys.Acta"
      },
      {
        "cds": "Annu. Rev. Astron. Astrophys.",
        "inspire": "Ann.Rev.Astron.Astrophys."
      },
      {
        "cds": "Probl. Inf. Transm.",
        "inspire": "Probl.Info.Transm."
      },
      {
        "cds": "J. Phys. G",
        "inspire": "J.Phys.G"
      },
      {
        "cds": "J. Phys.",
        "inspire": "J.Phys.(France)"
      },
      {
        "cds": "Acta Appl. Math.",
        "inspire": "Acta Appl.Math."
      },
      {
        "cds": "Mat. Fys. Skr. Dan. Vidensk. Selsk.",
        "inspire": "Kong.Dan.Vid.Sel.Mat.Fys.Med."
      },
      {
        "cds": "Observatory",
        "inspire": "Observatory"
      },
      {
        "cds": "J. Nonlinear Math. Phys.",
        "inspire": "J.Nonlin.Mathematical Phys."
      },
      {
        "cds": "Tech. Phys.",
        "inspire": "Tech.Phys."
      },
      {
        "cds": "Pis'ma Astron. Zh.",
        "inspire": "Pisma Astron.Zh."
      },
      {
        "cds": "IEEE Trans. Electron Devices",
        "inspire": "IEEE Trans.Electron.Dev."
      },
      {
        "cds": "Heavy Ion Phys.",
        "inspire": "Heavy Ion Phys."
      },
      {
        "cds": "Monatsh.Math.Phys.",
        "inspire": "Monatsh.Math.Phys."
      },
      {
        "cds": "Far East J. Theor. Stat.",
        "inspire": "Far East J. Theor. Stat."
      },
      {
        "cds": "IBM J. Res. Dev.",
        "inspire": "IBM J.Res.Dev."
      },
      {
        "cds": "Int. J. Math.",
        "inspire": "Int.J.Math."
      },
      {
        "cds": "J. Sound Vib.",
        "inspire": "J.Sound Vibrat."
      },
      {
        "cds": "Ain Shams Sci.Bull.",
        "inspire": "Ain Shams Sci.Bull."
      },
      {
        "cds": "Sci. China, A",
        "inspire": "Sci.Sin."
      },
      {
        "cds": "JAMA",
        "inspire": "JAMA"
      },
      {
        "cds": "Phys. At. Nucl.",
        "inspire": "Phys.Atom.Nucl."
      },
      {
        "cds": "JINR Rapid Commun.",
        "inspire": "JINR Rapid Commun."
      },
      {
        "cds": "J. Gen. Physiol.",
        "inspire": "J.Gen.Physiol."
      },
      {
        "cds": "IEEE Micro",
        "inspire": "IEEE Micro"
      },
      {
        "cds": "Comment.Math.Univ.St.Pauli",
        "inspire": "Comment.Math.Univ.St.Pauli"
      },
      {
        "cds": "Int. J. Mod. Phys.",
        "inspire": "Int.J.Mod.Phys."
      },
      {
        "cds": "Z.Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Bull.Class.Sci. Acad.Roy.Belg.",
        "inspire": "Bull.Class.Sci. Acad.Roy.Belg."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Kernenerg.",
        "inspire": "Kernenerg."
      },
      {
        "cds": "Inorg. Mater.",
        "inspire": "Inorg.Mater."
      },
      {
        "cds": "Opt. Commun.",
        "inspire": "Opt.Commun."
      },
      {
        "cds": "J. Phys. Chem. Ref. Data",
        "inspire": "J.Phys.Chem.Ref.Data"
      },
      {
        "cds": "Faraday Symp.Chem.Soc.",
        "inspire": "Faraday Symp.Chem.Soc."
      },
      {
        "cds": "J. Differ. Geom.",
        "inspire": "J.Diff.Geom."
      },
      {
        "cds": "At. Data Nucl. Data Tables",
        "inspire": "Atom.Data Nucl.Data Tabl."
      },
      {
        "cds": "Q. J. Math.",
        "inspire": "Quart.J.Math.Oxford Ser."
      },
      {
        "cds": "J.Meteor.",
        "inspire": "J.Meteor."
      },
      {
        "cds": "J.Astronaut.Sci.",
        "inspire": "J.Astronaut.Sci."
      },
      {
        "cds": "JETP Lett.",
        "inspire": "JETP Lett."
      },
      {
        "cds": "J.Sci.Explor.",
        "inspire": "J.Sci.Explor."
      },
      {
        "cds": "Dokl. Akad. Nauk SSSR",
        "inspire": "Dokl.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Faraday Disc. Chem. Soc.",
        "inspire": "Faraday Discuss.Chem.Soc."
      },
      {
        "cds": "J. Recr. Math.",
        "inspire": "J.Recr.Math."
      },
      {
        "cds": "Lect. Notes Phys.",
        "inspire": "Lect.Notes Phys."
      },
      {
        "cds": "Mater. Sci.",
        "inspire": "Mater.Sci."
      },
      {
        "cds": "Astron.Astrophys.Suppl.Ser.",
        "inspire": "Astron.Astrophys.Suppl.Ser."
      },
      {
        "cds": "Rend.Mat.",
        "inspire": "Rend.Mat."
      },
      {
        "cds": "Anal. Bioanal. Chem.",
        "inspire": "Anal.Bioanal.Chem."
      },
      {
        "cds": "Russ. Math. Surv.",
        "inspire": "Usp.Mat.Nauk"
      },
      {
        "cds": "Adv. Appl. Probab.",
        "inspire": "Adv.Appl.Probab."
      },
      {
        "cds": "Phys. Earth Planet. Inter.",
        "inspire": "Phys.Earth Planet.Interiors"
      },
      {
        "cds": "Neurocomputing",
        "inspire": "Neurocomput."
      },
      {
        "cds": "IEEE Commun. Mag.",
        "inspire": "IEEE Commun.Mag."
      },
      {
        "cds": "Adv. Phys.",
        "inspire": "Adv.Phys."
      },
      {
        "cds": "Proc. R. Soc. Lond., A",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "IEEE Trans. Plasma Sci.",
        "inspire": "IEEE Trans.Plasma Sci."
      },
      {
        "cds": "J. Math. Sci.",
        "inspire": "J.Math.Sci."
      },
      {
        "cds": "Sov. Phys. - JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Math. Program., Ser. A",
        "inspire": "Math.Programming"
      },
      {
        "cds": "CERN Cour.",
        "inspire": "CERN Cour."
      },
      {
        "cds": "Curr. Appl. Phys.",
        "inspire": "Curr.Appl.Phys."
      },
      {
        "cds": "IEEE Trans. Commun.",
        "inspire": "IEEE Trans.Commun."
      },
      {
        "cds": "J. Fluids Eng.",
        "inspire": "J.Fluids Eng."
      },
      {
        "cds": "J. Symb. Log.",
        "inspire": "J.Symbolic Logic"
      },
      {
        "cds": "Rev.Mex.Astron.Astrofis.",
        "inspire": "Rev.Mex.Fis."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Sov.J.Contemp.Phys.",
        "inspire": "Sov.J.Contemp.Phys."
      },
      {
        "cds": "J. Phys.: Condens. Matter",
        "inspire": "J.Phys.Condens.Matter"
      },
      {
        "cds": "Int.J.Alg.Comput.",
        "inspire": "Int.J.Alg.Comput."
      },
      {
        "cds": "Fiz.Mysl.Ross.",
        "inspire": "Fiz.Mysl.Ross."
      },
      {
        "cds": "J. Appl. Math. Mech.",
        "inspire": "J.Math.Mech."
      },
      {
        "cds": "Phys. Status Solidi",
        "inspire": "Phys.Solid State"
      },
      {
        "cds": "Electrochem. Commun.",
        "inspire": "Electrochem.Commun."
      },
      {
        "cds": "Nuovo Cim.",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "Opt. Lett.",
        "inspire": "Opt.Lett."
      },
      {
        "cds": "Fortschr. Phys.",
        "inspire": "Fortsch.Phys."
      },
      {
        "cds": "Astron. Astrophys. Suppl. Ser.",
        "inspire": "Astron.Astrophys.Suppl.Ser."
      },
      {
        "cds": "Phys. Rev. Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "Int. J. Appl. Radiat. Isot.",
        "inspire": "Int.J.Appl.Radiat.Isotopes"
      },
      {
        "cds": "Riv.Nuovo Cim.",
        "inspire": "Riv.Nuovo Cim."
      },
      {
        "cds": "Proc.Aristotelian Soc.",
        "inspire": "Proc.Aristotelian Soc."
      },
      {
        "cds": "J. Phys. Chem. A",
        "inspire": "J.Phys.Chem."
      },
      {
        "cds": "Phys. Part. Nucl.",
        "inspire": "Phys.Part.Nucl."
      },
      {
        "cds": "Adv. At. Mol. Opt. Phys.",
        "inspire": "Adv.At.Mol.Opt.Phys."
      },
      {
        "cds": "J. Phys. Chem.",
        "inspire": "J.Phys.Chem."
      },
      {
        "cds": "Irish Astron.J.",
        "inspire": "Irish Astron.J."
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "Amer.Nat.",
        "inspire": "Amer.Nat."
      },
      {
        "cds": "Phys.Rev.Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "Biophys. J.",
        "inspire": "Biophys.J."
      },
      {
        "cds": "Phys. Med. Biol.",
        "inspire": "Phys.Med.Biol."
      },
      {
        "cds": "Arch. Mech.",
        "inspire": "Arch.Mech."
      },
      {
        "cds": "J. Math. Pures Appl.",
        "inspire": "J.Math.Pure.Appl."
      },
      {
        "cds": "J. Phys. D",
        "inspire": "J.Phys.D"
      },
      {
        "cds": "Macromol. Symp.",
        "inspire": "Macromol.Symp."
      },
      {
        "cds": "Am. J. Phys.",
        "inspire": "Am.J.Phys."
      },
      {
        "cds": "ESO Messenger",
        "inspire": "ESO Messenger"
      },
      {
        "cds": "J. Chem. Phys.",
        "inspire": "J.Comput.Phys."
      },
      {
        "cds": "Geophys.",
        "inspire": "Geophysics"
      },
      {
        "cds": "Contemp. Math.",
        "inspire": "Contemp.Math."
      },
      {
        "cds": "Vestn.Mosk.Univ.Fiz.Astron.",
        "inspire": "Vestn.Mosk.Univ.Fiz.Astron."
      },
      {
        "cds": "SIAM J. Matrix Anal. Appl.",
        "inspire": "SIAM J.Matrix Anal.Appl."
      },
      {
        "cds": "Int. J. Satell. Commun.",
        "inspire": "Int.J.Satell.Commun."
      },
      {
        "cds": "Dokl.Akad Nauk.Belarus",
        "inspire": "Dokl.Akad Nauk.Belarus"
      },
      {
        "cds": "Biophys. J.",
        "inspire": "Biophys.J."
      },
      {
        "cds": "Ann. Rev. Biophys. Biomol. Struct.",
        "inspire": "Ann.Rev.Biophys.Biomol.Struct."
      },
      {
        "cds": "Ann. Fond. Louis de Broglie",
        "inspire": "Annales Fond.Broglie"
      },
      {
        "cds": "Rev. Adv. Mater. Sci.",
        "inspire": "Rev.Adv.Mater.Sci."
      },
      {
        "cds": "Nonlinear Process. Geophys.",
        "inspire": "Nonlin.Proc.Geophys."
      },
      {
        "cds": "Nucl. Data Sheets",
        "inspire": "Nucl.Data Sheets"
      },
      {
        "cds": "Proc. R. Soc. Lond., A",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "Europhys. News",
        "inspire": "Europhys.News"
      },
      {
        "cds": "Evol. Comput.",
        "inspire": "Evol.Comput."
      },
      {
        "cds": "At.Energiya.Prilozh.",
        "inspire": "At.Energiya.Prilozh."
      },
      {
        "cds": "Pramana - J. Phys.",
        "inspire": "Pramana"
      },
      {
        "cds": "Astron. Lett.",
        "inspire": "Astron.Lett."
      },
      {
        "cds": "Theor. Math. Phys.",
        "inspire": "Theor.Math.Phys."
      },
      {
        "cds": "Quantum Inf. Comput.",
        "inspire": "Quant.Inf.Comput."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Rev. Sci. Instrum.",
        "inspire": "Rev.Sci.Instrum."
      },
      {
        "cds": "Thin Solid Films",
        "inspire": "Thin Solid Films"
      },
      {
        "cds": "Publ. Astron. Soc. Pac.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "Atomkernenerg.",
        "inspire": "Atomkernenerg."
      },
      {
        "cds": "Prog. Elem. Part. Cosm. Ray Phys.",
        "inspire": "Prog.Elem.Part.Cosmic Ray Phys."
      },
      {
        "cds": "Sov.Phys.JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Nuovo Cimento, Riv.",
        "inspire": "Riv.Nuovo Cim."
      },
      {
        "cds": "Math. Methods Appl. Sci.",
        "inspire": "Math.Methods Appl.Sci."
      },
      {
        "cds": "Int. J. Quantum Chem.",
        "inspire": "Int.J.Quant.Chem."
      },
      {
        "cds": "Adv. Math.",
        "inspire": "Adv.Math."
      },
      {
        "cds": "Math. Mag.",
        "inspire": "Math.Mag."
      },
      {
        "cds": "Q. J. Math.",
        "inspire": "Quart.J.Math.Oxford Ser."
      },
      {
        "cds": "Proc. Astron. Soc. Aust.",
        "inspire": "Proc.Astron.Soc.Austral."
      },
      {
        "cds": "Comput. Methods Appl. Mech. Eng.",
        "inspire": "Comput.Methods Appl.Mech.Eng."
      },
      {
        "cds": "Helv. Phys. Acta",
        "inspire": "Helv.Phys.Acta"
      },
      {
        "cds": "Opt. Eng.",
        "inspire": "Opt.Eng."
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Contemp.Concepts Phys.",
        "inspire": "Contemp.Concepts Phys."
      },
      {
        "cds": "Can.J.Math.",
        "inspire": "Can.J.Math."
      },
      {
        "cds": "Acta Astronaut.",
        "inspire": "Acta Astronaut."
      },
      {
        "cds": "Int. J. Heat Mass Transf.",
        "inspire": "Int.J.Heat Mass Transf."
      },
      {
        "cds": "Surv. High Energy Phys.",
        "inspire": "Surveys High Energ.Phys."
      },
      {
        "cds": "Int. J. Numer. Methods Fluids",
        "inspire": "Int. J. Numer. Methods Fluids"
      },
      {
        "cds": "Moscow Univ.Chem.Bull.",
        "inspire": "Moscow Univ.Chem.Bull."
      },
      {
        "cds": "Gravit. Cosmol.",
        "inspire": "Grav.Cosmol."
      },
      {
        "cds": "Res.J.Meiji Gakuin Univ.",
        "inspire": "Res.J.Meiji Gakuin Univ."
      },
      {
        "cds": "Braz. J. Phys.",
        "inspire": "Braz.J.Phys."
      },
      {
        "cds": "Nucleonics",
        "inspire": "Nucleonics"
      },
      {
        "cds": "Denki Gakkai Zasshi",
        "inspire": "Denki Gakkai Zasshi"
      },
      {
        "cds": "Festkorperprobleme",
        "inspire": "Festkorperprobleme"
      },
      {
        "cds": "Quantum Electron.",
        "inspire": "Quant.Electron."
      },
      {
        "cds": "J. Comput. Phys.",
        "inspire": "J.Comput.Phys."
      },
      {
        "cds": "IEEE Circuits Devices Mag.",
        "inspire": "IEEE Circuits Devices Mag."
      },
      {
        "cds": "Phil.Trans.Roy.Soc.Lond.",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "J. Mater. Eng. Perform.",
        "inspire": "J.Mater.Eng.Perform."
      },
      {
        "cds": "Commun.Part.Diff.Eq.",
        "inspire": "Commun.Part.Diff.Eq."
      },
      {
        "cds": "Atmos.Oceanic Phys.",
        "inspire": "Atmos.Oceanic Phys."
      },
      {
        "cds": "Sov. Phys. - Uspekhi",
        "inspire": "Sov.Phys.Usp."
      },
      {
        "cds": "Bull.Seismol.Soc.Am.",
        "inspire": "Bull.Seismol.Soc.Am."
      },
      {
        "cds": "Los Alamos Sci.",
        "inspire": "Los Alamos Sci."
      },
      {
        "cds": "Chem. Geol.",
        "inspire": "Chem. Geol."
      },
      {
        "cds": "Pis'ma Zh. Eksp. Teor. Fiz.",
        "inspire": "Pisma Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Rept.Fac.Sci.Eng.Saga",
        "inspire": "Rept.Fac.Sci.Eng.Saga"
      },
      {
        "cds": "Int. J. Nonlinear Mech.",
        "inspire": "Int.J.Nonlinear Mech."
      },
      {
        "cds": "Quart. Rev. Biophys.",
        "inspire": "Q.Rev.Biophys."
      },
      {
        "cds": "Living Rev. Relativ.",
        "inspire": "Living Rev.Rel."
      },
      {
        "cds": "IBM J. Res. Dev.",
        "inspire": "IBM J.Res.Dev."
      },
      {
        "cds": "Phys. Fluids",
        "inspire": "Phys.Fluids"
      },
      {
        "cds": "Nucl.Met.Met.Soc.AIME",
        "inspire": "Nucl.Met.Met.Soc.AIME"
      },
      {
        "cds": "Prog. Part. Nucl. Phys.",
        "inspire": "Prog.Part.Nucl.Phys."
      },
      {
        "cds": "Sov.Astron.",
        "inspire": "Sov.Astron."
      },
      {
        "cds": "Indian J. Pure Appl. Phys.",
        "inspire": "Indian J.Pure Appl.Phys."
      },
      {
        "cds": "J.Biochem.Biophys.Methods",
        "inspire": "J.Biochem.Biophys.Methods"
      },
      {
        "cds": "Phys. Lett.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Mod. Phys. Lett. A",
        "inspire": "Mod.Phys.Lett."
      },
      {
        "cds": "Nature Phys.",
        "inspire": "Nature Phys."
      },
      {
        "cds": "J. Exp. Theor. Phys.",
        "inspire": "J.Exp.Theor.Phys."
      },
      {
        "cds": "Eur. J. Phys.",
        "inspire": "Eur.J.Phys."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "JHEP",
        "inspire": "JHEP"
      },
      {
        "cds": "Rev. Bras. Ensino Fis.",
        "inspire": "Rev.Bras.Ens.Fis."
      },
      {
        "cds": "Fortschr. Phys.",
        "inspire": "Fortsch.Phys."
      },
      {
        "cds": "Issues Sci. Technol. Librariansh.",
        "inspire": "Issues Sci.Technol.Librariansh."
      },
      {
        "cds": "J. Nucl. Med.",
        "inspire": "J.Nucl.Med."
      },
      {
        "cds": "Quantum Opt.",
        "inspire": "Quant.Opt."
      },
      {
        "cds": "Monatsber.Deutsch.Akad.Wiss.Berlin",
        "inspire": "Monatsber.Deutsch.Akad.Wiss.Berlin"
      },
      {
        "cds": "J. Soviet Math.",
        "inspire": "J.Sov.Math."
      },
      {
        "cds": "Cryogenics",
        "inspire": "Cryogenics"
      },
      {
        "cds": "Ann. Inst. Henri Poincaré, Anal. Non Linéaire",
        "inspire": "Ann.Inst.H.Poincare Anal.Non Lineaire"
      },
      {
        "cds": "Rev.Plasma Phys.",
        "inspire": "Rev.Plasma Phys."
      },
      {
        "cds": "Fiz. Plasmy",
        "inspire": "Fiz.Plasmy"
      },
      {
        "cds": "Fluid Dynamics",
        "inspire": "Fluid Dynamics"
      },
      {
        "cds": "Appl.Anal.",
        "inspire": "Appl.Anal."
      },
      {
        "cds": "Annals Inst.Stat.Math.",
        "inspire": "Annals Inst.Stat.Math."
      },
      {
        "cds": "Astropart. Phys.",
        "inspire": "Astropart.Phys."
      },
      {
        "cds": "Surveys in Geophys.",
        "inspire": "Surveys in Geophys."
      },
      {
        "cds": "Experientia",
        "inspire": "Experientia"
      },
      {
        "cds": "Commun. Pure Appl. Math.",
        "inspire": "Commun.Pure Appl.Math."
      },
      {
        "cds": "Int. Astron. Union Symp.",
        "inspire": "IAU Symp."
      },
      {
        "cds": "Math. Proc. Camb. Philos. Soc.",
        "inspire": "Math.Proc.Cambridge Phil.Soc."
      },
      {
        "cds": "Radiat. Meas.",
        "inspire": "Radiat.Meas."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Math. Scand.",
        "inspire": "Math.Scand."
      },
      {
        "cds": "Sov.J.Plasma Phys.",
        "inspire": "Sov.J.Plasma Phys."
      },
      {
        "cds": "J. Vac. Sci. Technol. A",
        "inspire": "J.Vac.Sci.Tech."
      },
      {
        "cds": "Prog. Nucl. Phys.",
        "inspire": "Prog.Part.Nucl.Phys."
      },
      {
        "cds": "New Phys.Suppl.",
        "inspire": "New Phys.Suppl."
      },
      {
        "cds": "Stochastics",
        "inspire": "Stochastics"
      },
      {
        "cds": "Surf. Rev. Lett.",
        "inspire": "Surf. Rev. Lett."
      },
      {
        "cds": "Appl. Phys. Lett.",
        "inspire": "Appl.Phys.Lett."
      },
      {
        "cds": "J. Med. Chem.",
        "inspire": "J.Med.Chem."
      },
      {
        "cds": "Boll. Soc. Ital. Fis.",
        "inspire": "Boll.Soc.Ital.Fis."
      },
      {
        "cds": "ACM Trans. Graph.",
        "inspire": "ACM Trans.Graph."
      },
      {
        "cds": "J.Magn.Soc.Japan",
        "inspire": "J.Magn.Soc.Japan"
      },
      {
        "cds": "Int. J. Mass Spectrom. Ion Process.",
        "inspire": "Int.J.Mass Spectr.Ion Process."
      },
      {
        "cds": "Math. Z.",
        "inspire": "Math.Z."
      },
      {
        "cds": "Dokl. Akad. Nauk SSSR",
        "inspire": "Dokl.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "J. Appl. Phys.",
        "inspire": "J.Appl.Phys."
      },
      {
        "cds": "Zh. Vych. Mat. Mat. Fiz.",
        "inspire": "Zh.Vych.Mat.Mat.Fiz."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "RIKEN Rev.",
        "inspire": "RIKEN Rev."
      },
      {
        "cds": "Mole. Simul.",
        "inspire": "Molec.Simul."
      },
      {
        "cds": "Phys. J.",
        "inspire": "Physik J."
      },
      {
        "cds": "Math. Mag.",
        "inspire": "Math.Mag."
      },
      {
        "cds": "Physik. Z.",
        "inspire": "Phys.Z."
      },
      {
        "cds": "PMC Phys.",
        "inspire": "PMC Phys."
      },
      {
        "cds": "Biophys. Chem.",
        "inspire": "Biophys.Chem."
      },
      {
        "cds": "Proc. R. Soc. Lond., A",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "Math. Sbornik",
        "inspire": "Mat.Sbornik"
      },
      {
        "cds": "Indian J. Phys.",
        "inspire": "Indian J.Phys.A"
      },
      {
        "cds": "Russ. Phys. J.",
        "inspire": "Russ.Phys.J."
      },
      {
        "cds": "J.Geology",
        "inspire": "J.Geology"
      },
      {
        "cds": "Otkrytiya, Izobret.",
        "inspire": "Otkrytiya, Izobret."
      },
      {
        "cds": "Infrared Phys.Tech.",
        "inspire": "Infrared Phys.Tech."
      },
      {
        "cds": "Proc. R. Soc. Lond.",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "Astrofiz. (USSR)",
        "inspire": "Astrofiz."
      },
      {
        "cds": "Theor. Math. Phys.",
        "inspire": "Theor.Math.Phys."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Discrete Comput. Geom.",
        "inspire": "Discrete Comput.Geom."
      },
      {
        "cds": "J.Anim.Ecol.",
        "inspire": "J.Anim.Ecol."
      },
      {
        "cds": "Radiat. Prot. Dosim.",
        "inspire": "Radiat.Prot.Dosim."
      },
      {
        "cds": "J. Phys. Stud.",
        "inspire": "J.Phys.Stud."
      },
      {
        "cds": "Ric. Sci.",
        "inspire": "Ric.Sci."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Magn.Reson.Rev.",
        "inspire": "Magn.Reson.Rev."
      },
      {
        "cds": "Astron. Astrophys. Suppl. Ser.",
        "inspire": "Astron.Astrophys.Suppl.Ser."
      },
      {
        "cds": "Annu. Rev. Astron. Astrophys.",
        "inspire": "Ann.Rev.Astron.Astrophys."
      },
      {
        "cds": "Proc. Natl. Acad. Sci. U. S. A.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "Int.J.Magn.",
        "inspire": "Int.J.Magn."
      },
      {
        "cds": "Erratum-ibid.",
        "inspire": "Erratum-ibid."
      },
      {
        "cds": "Micron",
        "inspire": "Micron"
      },
      {
        "cds": "Phys.Lett.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "J. Korean Math. Soc.",
        "inspire": "J.Korean Math.Soc."
      },
      {
        "cds": "J. Cosmol. Astropart. Phys.",
        "inspire": "JCAP"
      },
      {
        "cds": "Bull. Astron. Belgrade",
        "inspire": "Bull.Obs.Astron.Belgrade"
      },
      {
        "cds": "J. Phys.: Condens. Matter",
        "inspire": "J.Phys.Condens.Matter"
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Nuovo Cimento, Riv.",
        "inspire": "Riv.Nuovo Cim."
      },
      {
        "cds": "Living Rev. Sol. Phys.",
        "inspire": "Living Rev.Sol.Phys."
      },
      {
        "cds": "Math. Comput. Simul.",
        "inspire": "Math.Comput.Simul."
      },
      {
        "cds": "SIAM J.Sci.Statist.Comput.",
        "inspire": "SIAM J.Sci.Statist.Comput."
      },
      {
        "cds": "Kexue Tongbao (Scientia)",
        "inspire": "Kexue Tongbao (Scientia)"
      },
      {
        "cds": "IEEE Trans. Inf. Theory",
        "inspire": "IEEE Trans.Info.Theor."
      },
      {
        "cds": "Struct. Bond.",
        "inspire": "Struct.Bond."
      },
      {
        "cds": "Phys. Rev. D",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "J. Phys. A",
        "inspire": "J.Phys.A"
      },
      {
        "cds": "Publ. Res. Inst. Math. Sci.",
        "inspire": "Publ.Res.Inst.Math.Sci.Kyoto"
      },
      {
        "cds": "Geochim. Cosmochim. Acta",
        "inspire": "Geochim.Cosmochim.Acta"
      },
      {
        "cds": "Rev. Mex. Astron. Astrofis.",
        "inspire": "Rev.Mex.Astron.Astrofis."
      },
      {
        "cds": "IMA J. Appl. Math.",
        "inspire": "Ima J.Appl.Math."
      },
      {
        "cds": "Discrete Math.",
        "inspire": "Discrete Mathematics"
      },
      {
        "cds": "Inst.Hautes Etudes Sci.Publ.Math.",
        "inspire": "Inst.Hautes Etudes Sci.Publ.Math."
      },
      {
        "cds": "Zh. Eksp. Teor. Fiz.",
        "inspire": "Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Fusion Technol.",
        "inspire": "Fusion Tech."
      },
      {
        "cds": "Jpn. J. Appl. Phys.",
        "inspire": "Jap.J.Appl.Phys."
      },
      {
        "cds": "Trans.IEICE",
        "inspire": "Trans.IEICE"
      },
      {
        "cds": "J. Opt.",
        "inspire": "J.Opt."
      },
      {
        "cds": "Electron. J. Comb.",
        "inspire": "Electron. J. Comb."
      },
      {
        "cds": "IEEE Trans. Inf. Theory",
        "inspire": "IEEE Trans.Info.Theor."
      },
      {
        "cds": "J.Bus.Econ.Stat.",
        "inspire": "J.Bus.Econ.Stat."
      },
      {
        "cds": "World Sci. Lect. Notes Phys.",
        "inspire": "World Sci.Lect.Notes Phys."
      },
      {
        "cds": "Photonics Spectra",
        "inspire": "Photonics Spectra"
      },
      {
        "cds": "Mem. Soc. Astron. Ital.",
        "inspire": "Mem.Soc.Ast.It."
      },
      {
        "cds": "Trans. Am. Nucl. Soc.",
        "inspire": "Trans.Amer.Nucl.Soc."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Sci. Am.",
        "inspire": "Sci.Am."
      },
      {
        "cds": "J. Optim. Theory Appl.",
        "inspire": "J.Optim.Th.Appl."
      },
      {
        "cds": "Bull.Soc.Sci.Lett.Lodz Ser.Rech.Deform.",
        "inspire": "Bull.Soc.Sci.Lett.Lodz Ser.Rech.Deform."
      },
      {
        "cds": "Ann. Probab.",
        "inspire": "Annals Probab."
      },
      {
        "cds": "Cryst. Res. Tech.",
        "inspire": "Cryst.Res.Technol."
      },
      {
        "cds": "Math.Nat.Ber.Ungarn",
        "inspire": "Math.Nat.Ber.Ungarn"
      },
      {
        "cds": "Sankhyan.",
        "inspire": "Sankhyan."
      },
      {
        "cds": "Mat.Zametki",
        "inspire": "Mat.Zametki"
      },
      {
        "cds": "Can. Math. Bull.",
        "inspire": "Can.Math.Bull."
      },
      {
        "cds": "Phil.Stud.",
        "inspire": "Phil.Stud."
      },
      {
        "cds": "J. Math. Soc. Jpn.",
        "inspire": "J.Math.Soc.Jap."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Rev.Mod.Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "J.Approx.Theor.",
        "inspire": "J.Approx.Theor."
      },
      {
        "cds": "J. Res. Nat. Bureau Stand.",
        "inspire": "J.Res.Natl.Bur.Stand."
      },
      {
        "cds": "J. Mod. Opt.",
        "inspire": "J.Mod.Opt."
      },
      {
        "cds": "J. Magn. Magn. Mater.",
        "inspire": "J.Magn.Magn.Mater."
      },
      {
        "cds": "Geophys. Res. Lett.",
        "inspire": "Geophys.Res.Lett."
      },
      {
        "cds": "SIAM J. Optim.",
        "inspire": "SIAM J.Optim."
      },
      {
        "cds": "Helv. Phys. Acta",
        "inspire": "Helv.Phys.Acta"
      },
      {
        "cds": "Can. J. Phys.",
        "inspire": "Can.J.Phys."
      },
      {
        "cds": "IBM J.Res.Dev.",
        "inspire": "IBM J.Res.Dev."
      },
      {
        "cds": "Ukr. J. Phys.",
        "inspire": "Ukr.J.Phys."
      },
      {
        "cds": "Physica",
        "inspire": "Physica"
      },
      {
        "cds": "EMBO J.",
        "inspire": "EMBO J."
      },
      {
        "cds": "Probab. Theory Relat. Fields",
        "inspire": "Probab.Theor.Related Fields"
      },
      {
        "cds": "Teor. Mat. Fiz.",
        "inspire": "Teor.Mat.Fiz."
      },
      {
        "cds": "Math. Nachr.",
        "inspire": "Math.Nachr."
      },
      {
        "cds": "Liet.Fiz.Rink.",
        "inspire": "Liet.Fiz.Rink."
      },
      {
        "cds": "Am. Math. Soc. Transl.",
        "inspire": "Am.Math.Soc.Transl."
      },
      {
        "cds": "Meas. Sci. Technol.",
        "inspire": "Measur.Sci.Tech."
      },
      {
        "cds": "Solid State Electron.",
        "inspire": "Solid State Electron."
      },
      {
        "cds": "At. Energ. (USSR)",
        "inspire": "At.Energ."
      },
      {
        "cds": "Eur. Trans. Telecommun.",
        "inspire": "Eur.Trans.Telecommun."
      },
      {
        "cds": "Jap.J.Phys.",
        "inspire": "Jap.J.Phys."
      },
      {
        "cds": "Phys. Rev. Spec. Top. Accel. Beams",
        "inspire": "Phys.Rev.ST Accel.Beams"
      },
      {
        "cds": "Quantum Semiclass. Opt.",
        "inspire": "Quant.Semiclass.Opt."
      },
      {
        "cds": "Stud.Phil.Gand.",
        "inspire": "Stud.Phil.Gand."
      },
      {
        "cds": "Ann. Math.",
        "inspire": "Annals Math."
      },
      {
        "cds": "Ann. Inst. Henri Poincaré, Phys. Théor.",
        "inspire": "Ann.Inst.Henri Poincare Phys.Theor."
      },
      {
        "cds": "J. Phys. USSR",
        "inspire": "J.Phys.(USSR)"
      },
      {
        "cds": "J. Chim. Phys.",
        "inspire": "J.Chim.Phys."
      },
      {
        "cds": "Science",
        "inspire": "Science"
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Quant.Electron.",
        "inspire": "Quant.Electron."
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Chin. J. Phys.",
        "inspire": "Can.J.Phys."
      },
      {
        "cds": "Nucl.Data Sec.A",
        "inspire": "Nucl.Data Sec.A"
      },
      {
        "cds": "Contemp. Phys.",
        "inspire": "Contemp.Phys."
      },
      {
        "cds": "Can.J.Statist.",
        "inspire": "Can.J.Statist."
      },
      {
        "cds": "Arch. Sci. Phys. Nat.",
        "inspire": "Arch.Sci.Phys.Nat."
      },
      {
        "cds": "Ann. Comb.",
        "inspire": "Ann.Comb."
      },
      {
        "cds": "Russ. Phys. J.",
        "inspire": "Russ.Phys.J"
      },
      {
        "cds": "Ann. Inst. Henri Poincaré, A",
        "inspire": "Annales Poincare Phys.Theor."
      },
      {
        "cds": "At. Data Nucl. Data Tables",
        "inspire": "Atom.Data Nucl.Data Tabl."
      },
      {
        "cds": "Nuovo Cimento, Suppl.",
        "inspire": "Nuovo Cim.Suppl."
      },
      {
        "cds": "Electron. J. Theor. Phys.",
        "inspire": "Electron.J.Theor.Phys."
      },
      {
        "cds": "Appl. Opt.",
        "inspire": "Appl.Opt."
      },
      {
        "cds": "Theochem",
        "inspire": "Theochem"
      },
      {
        "cds": "Sov. J. Nucl. Phys.",
        "inspire": "Sov.J.Nucl.Phys."
      },
      {
        "cds": "Forum Math.",
        "inspire": "Forum Math."
      },
      {
        "cds": "New Astron. Rev.",
        "inspire": "New Astron.Rev."
      },
      {
        "cds": "J. Appl. Math.",
        "inspire": "J.Appl.Math."
      },
      {
        "cds": "Mod. Phys. Lett.",
        "inspire": "Mod.Phys.Lett."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "J. Phys. A",
        "inspire": "J.Phys.A"
      },
      {
        "cds": "Chin. J. Phys.",
        "inspire": "Chin.J.Phys."
      },
      {
        "cds": "IEEE J. Solid State Circuits",
        "inspire": "IEEE J.Solid State Circuits"
      },
      {
        "cds": "Publ.Res.Inst.Math.Sci.Kyoto",
        "inspire": "Publ.Res.Inst.Math.Sci.Kyoto"
      },
      {
        "cds": "Health Phys.",
        "inspire": "Health Phys."
      },
      {
        "cds": "IMA J. Numer. Anal.",
        "inspire": "IMA J.Numer.Anal."
      },
      {
        "cds": "Trans. Am. Math. Soc.",
        "inspire": "Trans.Am.Math.Soc."
      },
      {
        "cds": "Bioelectromagnetics",
        "inspire": "Bioelectromagnetics"
      },
      {
        "cds": "Mat. Fys. Medd. K. Dan. Vidensk. Selsk.",
        "inspire": "Kong.Dan.Vid.Sel.Mat.Fys.Med."
      },
      {
        "cds": "Quart. J. Roy. Astron. Soc.",
        "inspire": "Q.J.Roy.Astron.Soc."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "J. Am. Stat. Assoc.",
        "inspire": "J.Am.Statist.Assoc."
      },
      {
        "cds": "Astron.Astrophys.Suppl.Ser.",
        "inspire": "Astron.Astrophys.Suppl.Ser."
      },
      {
        "cds": "Math.Teacher",
        "inspire": "Math.Teacher"
      },
      {
        "cds": "Quantum Semiclass. Opt.",
        "inspire": "Quant.Semiclass.Opt."
      },
      {
        "cds": "Lett. Math. Phys.",
        "inspire": "Lett.Math.Phys."
      },
      {
        "cds": "Space Sci. Rev.",
        "inspire": "Space Sci.Rev."
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "Am.J.Sci.",
        "inspire": "Am.J.Sci."
      },
      {
        "cds": "Appl. Math. Lett.",
        "inspire": "Appl.Math.Lett."
      },
      {
        "cds": "J. Mod. Opt.",
        "inspire": "J.Mod.Opt."
      },
      {
        "cds": "J.Res.Natl.Bur.Stand.A Phys.Chem.",
        "inspire": "J.Res.Natl.Bur.Stand.A Phys.Chem."
      },
      {
        "cds": "Inverse Probl.",
        "inspire": "Inverse Prob."
      },
      {
        "cds": "Rep. Math. Phys.",
        "inspire": "Rept.Math.Phys."
      },
      {
        "cds": "At. Data Nucl. Data Tables",
        "inspire": "Atom.Data Nucl.Data Tabl."
      },
      {
        "cds": "Complex Syst.",
        "inspire": "Complex Syst."
      },
      {
        "cds": "New Astron. Rev.",
        "inspire": "New Astron.Rev."
      },
      {
        "cds": "Phys. Z Sowjetunion",
        "inspire": "Phys.Z.Sowjetunion"
      },
      {
        "cds": "Chem.Proc.Eng.",
        "inspire": "Chem.Proc.Eng."
      },
      {
        "cds": "C. R. Acad. Sci. Paris",
        "inspire": "Compt.Rend.Math."
      },
      {
        "cds": "Comments Nucl. Part. Phys.",
        "inspire": "Comments Nucl.Part.Phys."
      },
      {
        "cds": "J. Nucl. Sci. Technol.",
        "inspire": "J.Sci.Tech."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "J.Roy.Statist.Soc.A",
        "inspire": "J.Roy.Statist.Soc.A"
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "IEEE J. Solid State Circuits",
        "inspire": "IEEE J.Solid State Circuits"
      },
      {
        "cds": "Acta Phys. Sin.",
        "inspire": "Acta Photonica Sin."
      },
      {
        "cds": "Ann. Phys. (San Diego)",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Int. Math. Res. Not.",
        "inspire": "Int.Math.Res.Not."
      },
      {
        "cds": "J.Chem.Soc.Faraday Trans. 1",
        "inspire": "J.Chem.Soc.Faraday Trans. 1"
      },
      {
        "cds": "Int. Astron. Union Circ.",
        "inspire": "IAU Symp."
      },
      {
        "cds": "Bild Wiss.",
        "inspire": "Bild.Wiss."
      },
      {
        "cds": "Int. J. Quantum Chem.",
        "inspire": "Int.J.Quant.Chem."
      },
      {
        "cds": "IEEE Trans. Dielectr. Electr. Insul.",
        "inspire": "IEEE Trans.Dielectr.Electr.Insul."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Contemp. Math.",
        "inspire": "Contemp.Math."
      },
      {
        "cds": "Gravit. Cosmol.",
        "inspire": "Grav.Cosmol."
      },
      {
        "cds": "Theor. Chem. Acc.",
        "inspire": "Theor. Chem. Acc."
      },
      {
        "cds": "Surf. Sci.",
        "inspire": "Surf.Sci."
      },
      {
        "cds": "Inverse Probl.",
        "inspire": "Inverse Prob."
      },
      {
        "cds": "Plasma Phys. Rep.",
        "inspire": "Plasma Phys.Rep."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.and Phys."
      },
      {
        "cds": "Q. J. Math.",
        "inspire": "Quart.J.Math.Oxford Ser."
      },
      {
        "cds": "Electron.World And Wireless World",
        "inspire": "Electron.World And Wireless World"
      },
      {
        "cds": "Rep. Math. Phys.",
        "inspire": "Rept.Math.Phys."
      },
      {
        "cds": "Compos. Math.",
        "inspire": "Compos.Math."
      },
      {
        "cds": "Beam Sci.Technol.",
        "inspire": "Beam Sci.Technol."
      },
      {
        "cds": "Rev. Mod. Astron.",
        "inspire": "Rev.Mod.Astron."
      },
      {
        "cds": "Proc. SPIE Int. Soc. Opt. Eng.",
        "inspire": "Proc.SPIE Int.Soc.Opt.Eng."
      },
      {
        "cds": "Nature Mater.",
        "inspire": "Nature Materials"
      },
      {
        "cds": "Phys. Educ.",
        "inspire": "Phys.Educ."
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Symmetry",
        "inspire": "Symmetry"
      },
      {
        "cds": "Hozon Kagaku",
        "inspire": "Hozon Kagaku"
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Czechoslov. J. Phys.",
        "inspire": "Czech.J.Phys."
      },
      {
        "cds": "Space Sci. Rev.",
        "inspire": "Space Sci.Rev."
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Proc. Natl. Acad. Sci. U. S. A.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "Zh. Eksp. Teor. Fiz.",
        "inspire": "Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Nucl. Sci. Eng.",
        "inspire": "Nucl.Sci.Eng."
      },
      {
        "cds": "Theor. Comput. Fluid Dyn.",
        "inspire": "Theor.Comput.Fluid Dyn."
      },
      {
        "cds": "Math. USSR Izvestiya",
        "inspire": "Math.USSR Izv."
      },
      {
        "cds": "GR-QC",
        "inspire": "GR-QC"
      },
      {
        "cds": "Radiat. Meas.",
        "inspire": "Radiat.Meas."
      },
      {
        "cds": "Jpn. J. Appl. Phys.",
        "inspire": "Jap.J.Appl.Phys."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Softw.- Pract. Exp.",
        "inspire": "Software Pract.Exper."
      },
      {
        "cds": "Celest. Mech. Dyn. Astron.",
        "inspire": "Celest.Mech.Dyn.Astron."
      },
      {
        "cds": "Astrofiz. (USSR)",
        "inspire": "Astrofiz."
      },
      {
        "cds": "Comput. Math.",
        "inspire": "Compos.Math."
      },
      {
        "cds": "Brain",
        "inspire": "Brain"
      },
      {
        "cds": "Funct. Anal. Appl.",
        "inspire": "Funct.Anal.Appl."
      },
      {
        "cds": "Diff.Eq.",
        "inspire": "Diff.Eq."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Adv. Nucl. Phys.",
        "inspire": "Adv.Nucl.Phys."
      },
      {
        "cds": "J. Phys.: Condens. Matter",
        "inspire": "J.Phys.Condens.Matter"
      },
      {
        "cds": "Cahiers Topo.Geom.Diff.",
        "inspire": "Cahiers Topo.Geom.Diff."
      },
      {
        "cds": "Prog. Theor. Phys., Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "Phil.Q.",
        "inspire": "Phil.Q."
      },
      {
        "cds": "Notre Dame J.Form.Logic",
        "inspire": "Notre Dame J.Form.Logic"
      },
      {
        "cds": "Publ. Astron. Soc. Jpn.",
        "inspire": "Publ.Astron.Soc.Jap."
      },
      {
        "cds": "J.Electrostat.",
        "inspire": "J.Electrostat."
      },
      {
        "cds": "Mech. Res. Commun.",
        "inspire": "Mech.Res.Commun."
      },
      {
        "cds": "Adv. Comput. Math.",
        "inspire": "Adv.Comput.Math."
      },
      {
        "cds": "Astron.Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "ACM Trans.Model.Comput.Simul.",
        "inspire": "ACM Trans.Model.Comput.Simul."
      },
      {
        "cds": "Semicond. Sci. Technol.",
        "inspire": "Semicond.Sci.Tech."
      },
      {
        "cds": "Collect. Phenom.",
        "inspire": "Collect.Phenom."
      },
      {
        "cds": "Theor. Chim. Acta",
        "inspire": "Theor.Chim.Acta"
      },
      {
        "cds": "Am. J. Math.",
        "inspire": "Am.J.Math."
      },
      {
        "cds": "Geomagn.Aeronom.",
        "inspire": "Geomagn.Aeronom."
      },
      {
        "cds": "Abh. Math. Sem. Hamburg",
        "inspire": "Abh.Math.Sem.Univ.Hamburg"
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Appl. Comput. Harmonic Anal.",
        "inspire": "Appl.Comput.Harmon.Anal."
      },
      {
        "cds": "Rend.Sem.Mat.Univ.Politec.Torino",
        "inspire": "Rend.Sem.Mat.Univ.Politec.Torino"
      },
      {
        "cds": "PiN Newslett.",
        "inspire": "PiN Newslett."
      },
      {
        "cds": "Sov. Phys. - Solid State",
        "inspire": "Sov.Phys.Solid State"
      },
      {
        "cds": "Ukr. Fiz. Zh.",
        "inspire": "Ukr.Fiz.Zh.(Russ.Ed.)"
      },
      {
        "cds": "PoS",
        "inspire": "PoS"
      },
      {
        "cds": "Publ. Astron. Soc. Aust.",
        "inspire": "Publ.Astron.Soc.Austral."
      },
      {
        "cds": "J.Chem.Phys.Soc.",
        "inspire": "J.Chem.Phys.Soc."
      },
      {
        "cds": "Publ. Astron. Soc. Pac.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "J. Phys.",
        "inspire": "J.Phys.A"
      },
      {
        "cds": "Fractals",
        "inspire": "Fractals"
      },
      {
        "cds": "Bull. Math. Biol.",
        "inspire": "Bull.Math.Biol."
      },
      {
        "cds": "Sov.Sci.Rev.",
        "inspire": "Sov.Sci.Rev."
      },
      {
        "cds": "J. Am. Chem. Soc.",
        "inspire": "J.Am.Chem.Soc."
      },
      {
        "cds": "Atti. Semin. Mat. Fis. Univ. Modena",
        "inspire": "Atti. Semin. Mat. Fis. Univ. Modena"
      },
      {
        "cds": "Crelle",
        "inspire": "Crelle"
      },
      {
        "cds": "New Astron. Rev.",
        "inspire": "New Astron.Rev."
      },
      {
        "cds": "J. Br. Nucl. Eng. Soc.",
        "inspire": "J.Br.Nucl.Energy Soc."
      },
      {
        "cds": "Vistas Astron.",
        "inspire": "Vistas Astron."
      },
      {
        "cds": "Lett. Math. Phys.",
        "inspire": "Lett.Math.Phys."
      },
      {
        "cds": "Acta Astron.Sin.",
        "inspire": "Acta Astron.Sin."
      },
      {
        "cds": "Sov.Phys.JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Astrophys.Space Sci.",
        "inspire": "Astrophys.Space Sci."
      },
      {
        "cds": "Curr. Sci.",
        "inspire": "Curr.Sci."
      },
      {
        "cds": "Astropart. Phys.",
        "inspire": "Astropart.Phys."
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "Computing",
        "inspire": "Computing"
      },
      {
        "cds": "J. Math. Biol.",
        "inspire": "J.Math.Biol."
      },
      {
        "cds": "Int. J. Theor. Phys.",
        "inspire": "Int.J.Theor.Phys."
      },
      {
        "cds": "Acta Mech.",
        "inspire": "Acta Mech."
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "Radiophys. Quantum Electron.",
        "inspire": "Radiophys.Quant.Electron."
      },
      {
        "cds": "Am. J. Math.",
        "inspire": "Am.J.Math."
      },
      {
        "cds": "Ann. Probab.",
        "inspire": "Annals Probab."
      },
      {
        "cds": "Pis'ma Fiz. Elem. Chastits At. Yadra",
        "inspire": "Pisma Fiz.Elem.Chast.Atom.Yadra"
      },
      {
        "cds": "eConf",
        "inspire": "eConf"
      },
      {
        "cds": "Russ. Math. Surv.",
        "inspire": "Usp.Mat.Nauk"
      },
      {
        "cds": "Phys. Today",
        "inspire": "Phys.Today"
      },
      {
        "cds": "Chem. Phys. Chem.",
        "inspire": "ChemPhysChem"
      },
      {
        "cds": "Cent.Eur.J.Math.",
        "inspire": "Cent.Eur.J.Math."
      },
      {
        "cds": "Philos. Trans. R. Soc. Lond., A",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "APS News",
        "inspire": "APS News"
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Acta Appl. Math.",
        "inspire": "Acta Appl.Math."
      },
      {
        "cds": "C. R. Hebd. Séances Acad. Sci.",
        "inspire": "Compt.Rend.Math."
      },
      {
        "cds": "Jpn. J. Math.",
        "inspire": "Jap.J.Math."
      },
      {
        "cds": "J. Instrum.",
        "inspire": "JINST"
      },
      {
        "cds": "J. Phys. C",
        "inspire": "J.Phys.C"
      },
      {
        "cds": "Astron. Geophys.",
        "inspire": "Astron.Geophys."
      },
      {
        "cds": "ESO Messenger",
        "inspire": "ESO Messenger"
      },
      {
        "cds": "Rev. Bras. Ensino Fis.",
        "inspire": "Rev.Bras.Fis."
      },
      {
        "cds": "J. Mol. Liq.",
        "inspire": "J.Mol.Liq."
      },
      {
        "cds": "J. Phys. F",
        "inspire": "J.Phys.F"
      },
      {
        "cds": "J. Phys. G",
        "inspire": "J.Phys.G"
      },
      {
        "cds": "J.Comb.Theory Ser.",
        "inspire": "J.Comb.Theory Ser."
      },
      {
        "cds": "Gravit. Cosmol.",
        "inspire": "Grav.Cosmol."
      },
      {
        "cds": "Math. Control Signals Syst.",
        "inspire": "Math.Control Signals Syst."
      },
      {
        "cds": "Czechoslov. J. Phys.",
        "inspire": "Czech.J.Phys."
      },
      {
        "cds": "C. R. Phys.",
        "inspire": "Comptes Rendus Physique"
      },
      {
        "cds": "Nonlinear Anal.",
        "inspire": "Nonlinear Anal."
      },
      {
        "cds": "Phys. Bull.",
        "inspire": "Phys.Bull."
      },
      {
        "cds": "Mon.Not.Roy.Astron.Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Condens. Matter Theor.",
        "inspire": "Condensed Matter Theor."
      },
      {
        "cds": "Int. Astron. Union Circ.",
        "inspire": "IAU Circ."
      },
      {
        "cds": "Z.Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Sov.Phys.JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Bull.Calcutta Math.Soc.",
        "inspire": "Bull.Calcutta Math.Soc."
      },
      {
        "cds": "Monatsh. Chem.",
        "inspire": "Monatsh.Chem."
      },
      {
        "cds": "Combinatorica",
        "inspire": "Combinatorica"
      },
      {
        "cds": "High Energy Phys. Nucl. Phys.",
        "inspire": "High Energy Phys.Nucl.Phys."
      },
      {
        "cds": "Astrophys. Space Sci.",
        "inspire": "Astrophys.Space Sci."
      },
      {
        "cds": "Zh. Eksp. Teor. Fiz.",
        "inspire": "Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Ann. Glob. Anal. Geom.",
        "inspire": "Annals Global Anal.Geom."
      },
      {
        "cds": "Zh. Eksp. Teor. Fiz.",
        "inspire": "Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Ann. Math.",
        "inspire": "Annals Math."
      },
      {
        "cds": "Phil.Mag.Suppl.",
        "inspire": "Phil.Mag.Suppl."
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Astrophys. Space Sci. Trans.",
        "inspire": "Astrophys.Space Sci.Trans."
      },
      {
        "cds": "Minerva",
        "inspire": "Minerva"
      },
      {
        "cds": "Acta Phys.Slov.",
        "inspire": "Acta Phys.Slov."
      },
      {
        "cds": "Kodai Math. J.",
        "inspire": "Kodai Math.Sem.Rept."
      },
      {
        "cds": "Ann. Mat. Pura Appl.",
        "inspire": "Annali Mat.Pura Appl."
      },
      {
        "cds": "Moscow Univ.Phys.Bull.",
        "inspire": "Vestn.Mosk.Univ.Fiz.Astron."
      },
      {
        "cds": "Memory Cognition",
        "inspire": "Memory Cognition"
      },
      {
        "cds": "Publ. Astron. Soc. Aust.",
        "inspire": "Publ.Astron.Soc.Austral."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Plasma Phys. Control. Fusion",
        "inspire": "Plasma Phys.Control.Fusion"
      },
      {
        "cds": "Croat.Chem.Acta",
        "inspire": "Croat.Chem.Acta"
      },
      {
        "cds": "Stat. Sci.",
        "inspire": "Statist.Sci."
      },
      {
        "cds": "Scientia",
        "inspire": "Scientia"
      },
      {
        "cds": "Quad.Ricerca",
        "inspire": "Quad.Ricerca"
      },
      {
        "cds": "Sovr.Probl.Mat.",
        "inspire": "Sovr.Probl.Mat."
      },
      {
        "cds": "IEEE ASSP Magazine",
        "inspire": "IEEE ASSP Magazine"
      },
      {
        "cds": "Adv.Photochem.",
        "inspire": "Adv.Photochem."
      },
      {
        "cds": "Ann. Sci. Ec. Norm. Supér.",
        "inspire": "Annales Sci.Ecole Norm.Sup."
      },
      {
        "cds": "Found. Phys.",
        "inspire": "Found.Phys."
      },
      {
        "cds": "Anal. Math.",
        "inspire": "Anal.Math."
      },
      {
        "cds": "J. Algebra Geom.",
        "inspire": "J.Alg.Geom."
      },
      {
        "cds": "J. Mater. Sci.: Mater. Electron.",
        "inspire": "J.Mater.Sci.Mater.Electron."
      },
      {
        "cds": "Nucl. Phys. B",
        "inspire": "Nucl.Phys."
      },
      {
        "cds": "Comput. Archit. News",
        "inspire": "Comput.Architecture News"
      },
      {
        "cds": "Phys. Med. Biol.",
        "inspire": "Phys.Med.Biol."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "KFK Nachr.",
        "inspire": "KFK Nachr."
      },
      {
        "cds": "Portugal.Phys.",
        "inspire": "Portugal.Phys."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Invent. Math.",
        "inspire": "Invent.Math."
      },
      {
        "cds": "Annali Mat.Pura Appl.",
        "inspire": "Annali Mat.Pura Appl."
      },
      {
        "cds": "Acta Phys. Pol.",
        "inspire": "Acta Phys.Polon."
      },
      {
        "cds": "Commun.Math.Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "J. Atmos. Terr. Phys.",
        "inspire": "J.Atmos.Terr.Phys."
      },
      {
        "cds": "J. Fluid Mech.",
        "inspire": "J.Fluid Mech."
      },
      {
        "cds": "Adv. Colloid Interface Sci.",
        "inspire": "Adv.Colloid Interface Sci."
      },
      {
        "cds": "Int.J.High Speed Computing",
        "inspire": "Int.J.High Speed Computing"
      },
      {
        "cds": "Rep. Math. Phys.",
        "inspire": "Rept.Math.Phys."
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "Instrum. Exp. Tech.",
        "inspire": "Instrum.Exp.Tech."
      },
      {
        "cds": "J. High Energy Phys.",
        "inspire": "JHEP"
      },
      {
        "cds": "Proc. SPIE",
        "inspire": "Proc.SPIE Int.Soc.Opt.Eng."
      },
      {
        "cds": "At. Data Nucl. Data Tables",
        "inspire": "Atom.Data Nucl.Data Tabl."
      },
      {
        "cds": "Prog. Theor. Phys., Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "Ukr. Math. J.",
        "inspire": "Ukr.Math.J."
      },
      {
        "cds": "MPG Spieg.",
        "inspire": "MPG Spiegel"
      },
      {
        "cds": "Astron. Astrophys. Suppl. Ser.",
        "inspire": "Astron.Astrophys.Suppl.Ser."
      },
      {
        "cds": "Proc. SPIE",
        "inspire": "Proc.SPIE Int.Soc.Opt.Eng."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "J. Lond. Math. Soc.",
        "inspire": "J.Lond.Math.Soc."
      },
      {
        "cds": "Astron. Educ. Rev.",
        "inspire": "Astron.Educ.Rev."
      },
      {
        "cds": "Quantum Inf. Process.",
        "inspire": "Quant.Inf.Proc."
      },
      {
        "cds": "Publ. Mat. Debrecen",
        "inspire": "Publ.Math.Debrecen"
      },
      {
        "cds": "Sov. Math. - Dokl.",
        "inspire": "Sov.Math.Dokl."
      },
      {
        "cds": "LBL Res. Rev.",
        "inspire": "LBL Res. Rev."
      },
      {
        "cds": "Yad.Fiz.",
        "inspire": "Yad.Fiz."
      },
      {
        "cds": "Cah. Phys.",
        "inspire": "Cahiers Phys."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "J. Opt. Soc. Am.",
        "inspire": "J.Opt.Soc.Am."
      },
      {
        "cds": "Electron. J. Probab.",
        "inspire": "Electron.J.Probab."
      },
      {
        "cds": "Twistor Newsl.",
        "inspire": "Twistor Newslett."
      },
      {
        "cds": "Ark. Fys.",
        "inspire": "Ark.Fys."
      },
      {
        "cds": "Acustica",
        "inspire": "Acta Acustica"
      },
      {
        "cds": "Phys. Status Solidi, B B",
        "inspire": "Phys.Status Solidi"
      },
      {
        "cds": "Phys. Status Solidi, A",
        "inspire": "Phys.Status Solidi"
      },
      {
        "cds": "Z. Naturforsch. A",
        "inspire": "Z.Naturforsch."
      },
      {
        "cds": "Phys. Rev. Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "Pub. Astron. Soc. Jpn.",
        "inspire": "Publ.Astron.Soc.Jap."
      },
      {
        "cds": "Adv. Chem. Phys.",
        "inspire": "Adv.Chem.Phys."
      },
      {
        "cds": "Sci. Program.",
        "inspire": "Sci.Prog."
      },
      {
        "cds": "J. Cryptol.",
        "inspire": "J.Cryptolog."
      },
      {
        "cds": "Ann. Phys. (Les Ulis)",
        "inspire": "Annales Phys.(France)"
      },
      {
        "cds": "Infrared Physics",
        "inspire": "Infrared Physics"
      },
      {
        "cds": "Beitr.Anal.",
        "inspire": "Beitr.Anal."
      },
      {
        "cds": "Q.Rev.Chem.Soc.",
        "inspire": "Q.Rev.Chem.Soc."
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "Uspekhi Fiz. Nauk",
        "inspire": "Usp.Fiz.Nauk"
      },
      {
        "cds": "Annals N.Y.Acad.Sci.",
        "inspire": "Annals N.Y.Acad.Sci."
      },
      {
        "cds": "Phys. Rev. C",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "Ann. Math. Stat.",
        "inspire": "Annals Math.Statist."
      },
      {
        "cds": "Nuovo Cimento",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "Neural Netw.",
        "inspire": "Neural Networks"
      },
      {
        "cds": "J. Chem. Phys.",
        "inspire": "J.Chem.Phys."
      },
      {
        "cds": "Microw. Opt. Technol. Lett.",
        "inspire": "Microw.Opt.Technol.Lett."
      },
      {
        "cds": "Phil.Rev.",
        "inspire": "Phil.Rev."
      },
      {
        "cds": "Mon.Not.Roy.Astron.Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Math.Chronicle",
        "inspire": "Math.Chronicle"
      },
      {
        "cds": "Invent. Math.",
        "inspire": "Invent.Math."
      },
      {
        "cds": "Apeiron",
        "inspire": "Apeiron"
      },
      {
        "cds": "Rev. Geophys.",
        "inspire": "Rev.Geophys."
      },
      {
        "cds": "Prog. Opt.",
        "inspire": "Prog.Opt"
      },
      {
        "cds": "J. Physiol.",
        "inspire": "J.Physiol."
      },
      {
        "cds": "Astron. Lett.",
        "inspire": "Astron.Lett."
      },
      {
        "cds": "Annu. Rev. Nucl. Part. Sci.",
        "inspire": "Complete"
      },
      {
        "cds": "C. R. Acad. Sci. USSR",
        "inspire": "C.R.Acad.Sci.URSS"
      },
      {
        "cds": "Izv.Vuz.Radiofiz.",
        "inspire": "Izv.Vuz.Radiofiz."
      },
      {
        "cds": "Wave Motion",
        "inspire": "Wave Motion"
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Duke Math. J.",
        "inspire": "Duke Math.J."
      },
      {
        "cds": "Am. Math. Mon.",
        "inspire": "Am.Math.Mon."
      },
      {
        "cds": "Pac. J. Math.",
        "inspire": "Pacific J.Math."
      },
      {
        "cds": "Proc. Symp. Pure Math.",
        "inspire": "Proc.Symp.Pure Math."
      },
      {
        "cds": "Acta Metall.",
        "inspire": "Acta Metall."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals N.Y.Acad.Sci."
      },
      {
        "cds": "Nucl. Track Detect.",
        "inspire": "Nucl.Track Detect."
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Am. J. Phys.",
        "inspire": "Am.J.Phys."
      },
      {
        "cds": "Mon. Not. R. Astron. Soc.",
        "inspire": "Mon.Not.Roy.Astron.Soc."
      },
      {
        "cds": "Z. Phys.",
        "inspire": "Z.Phys."
      },
      {
        "cds": "Nucl. Instrum. Methods Phys. Res.,",
        "inspire": "Nucl.Instrum.Meth."
      },
      {
        "cds": "Rev.Mod.Phys.",
        "inspire": "Rev.Mod.Phys."
      },
      {
        "cds": "Adv.Astron.Astrophys.",
        "inspire": "Adv.Astron.Astrophys."
      },
      {
        "cds": "Bull.Inst.Math.Appl.",
        "inspire": "Bull.Inst.Math.Appl."
      },
      {
        "cds": "J. Supercond.",
        "inspire": "J.Supercond."
      },
      {
        "cds": "Funkt.Anal.Pril.",
        "inspire": "Funkt.Anal.Pril."
      },
      {
        "cds": "J. Lumin.",
        "inspire": "J.Lumin"
      },
      {
        "cds": "Ann. Math. Stat.",
        "inspire": "Annals Math.Statist."
      },
      {
        "cds": "Austral.Physicist",
        "inspire": "Austral.Physicist"
      },
      {
        "cds": "Phys.Lett.",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "Z.Astrophys.",
        "inspire": "Z.Astrophys."
      },
      {
        "cds": "Phys.Rev.",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "Adv. Theor. Math. Phys.",
        "inspire": "Adv.Theor.Math.Phys."
      },
      {
        "cds": "J. Lond. Math. Soc.",
        "inspire": "Proc.Lond.Math.Soc."
      },
      {
        "cds": "Proc. Phys. Math. Soc. Jpn.",
        "inspire": "Proc.Phys.Math.Soc.Jap."
      },
      {
        "cds": "Q.J.Mech.Appl.Math.",
        "inspire": "Q.J.Mech.Appl.Math."
      },
      {
        "cds": "Algebra Colloq.",
        "inspire": "Algebra Colloq."
      },
      {
        "cds": "Probl. Inf. Transm.",
        "inspire": "Probl.Info.Transm."
      },
      {
        "cds": "J. Mol. Spectrosc.",
        "inspire": "J.Molec.Spectrosc."
      },
      {
        "cds": "Astron.J.",
        "inspire": "Astron.J."
      },
      {
        "cds": "Entropy",
        "inspire": "Entropy"
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Phil.Topics",
        "inspire": "Phil.Topics"
      },
      {
        "cds": "Pharmacol.Biochem.Behavior.",
        "inspire": "Pharmacol.Biochem.Behavior."
      },
      {
        "cds": "Astron. Lett.",
        "inspire": "Astron.Lett."
      },
      {
        "cds": "Commun. Theor. Phys.",
        "inspire": "Commun.Theor.Phys."
      },
      {
        "cds": "Adv.Spectrosc.",
        "inspire": "Adv.Spectrosc."
      },
      {
        "cds": "Am. Math. Mon.",
        "inspire": "Am.Math.Mon."
      },
      {
        "cds": "Annu. Rev. Nucl. Sci.",
        "inspire": "Complete"
      },
      {
        "cds": "IEEE Trans. Syst. Man Cybern.",
        "inspire": "IEEE Trans.Syst.Man Cybern."
      },
      {
        "cds": "Phys. Rev. Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "BioEssays",
        "inspire": "BioEssays"
      },
      {
        "cds": "Contemp. Phys.",
        "inspire": "Contemp.Phys."
      },
      {
        "cds": "Sci. Rep. Tohoku Univ.",
        "inspire": "Sci.Rept.Tohoku Univ.(Ser.A)"
      },
      {
        "cds": "Ann.Rev.Astron.Astrophys.",
        "inspire": "Complete"
      },
      {
        "cds": "Astropart. Phys.",
        "inspire": "Astropart.Phys."
      },
      {
        "cds": "J. Phys. Radium",
        "inspire": "J.Phys.Radium"
      },
      {
        "cds": "Math. Z.",
        "inspire": "Math.Z."
      },
      {
        "cds": "Adv. Electron.",
        "inspire": "Adv.Electron."
      },
      {
        "cds": "Sov.Phys.JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "J. Korean Phys. Soc.",
        "inspire": "J.Korean Phys.Soc."
      },
      {
        "cds": "Springer Proc.Phys.",
        "inspire": "Springer Proc.Phys."
      },
      {
        "cds": "Fusion Eng. Des.",
        "inspire": "Fusion Eng.Des."
      },
      {
        "cds": "Rep. Prog. Phys.",
        "inspire": "Rept.Prog.Phys."
      },
      {
        "cds": "Chin. J. Nucl. Phys.",
        "inspire": "Chin.J.Nucl.Phys."
      },
      {
        "cds": "Comput. Phys.",
        "inspire": "Comput.Phys."
      },
      {
        "cds": "Appl. Phys. Solids Surf.",
        "inspire": "Astrophys.Space Sci."
      },
      {
        "cds": "IEEE Network",
        "inspire": "IEEE Network"
      },
      {
        "cds": "Spektrum Wiss.",
        "inspire": "Spektrum Wiss."
      },
      {
        "cds": "SIAM J. Comput.",
        "inspire": "SIAM J.Comput."
      },
      {
        "cds": "Astron. Rep.",
        "inspire": "Astron.Rep."
      },
      {
        "cds": "Fundam. Cosm. Phys.",
        "inspire": "Fund.Cosmic Phys."
      },
      {
        "cds": "Order",
        "inspire": "Order"
      },
      {
        "cds": "Exp. Astron.",
        "inspire": "Exper.Astron."
      },
      {
        "cds": "Beam Line",
        "inspire": "SLAC Beam Line"
      },
      {
        "cds": "Mind",
        "inspire": "Mind"
      },
      {
        "cds": "Pis'ma Zh. Eksp. Teor. Fiz.",
        "inspire": "Pisma Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "Phys. Fluids B",
        "inspire": "Phys.Fluids B"
      },
      {
        "cds": "Michigan Math. J.",
        "inspire": "Michigan Math.J."
      },
      {
        "cds": "Phys. Rev. Spec. Top. Accel. Beams",
        "inspire": "Phys.Rev.ST Accel.Beams"
      },
      {
        "cds": "Health Phys.",
        "inspire": "Health Phys."
      },
      {
        "cds": "Laser Part. Beams",
        "inspire": "Laser Part.Beams"
      },
      {
        "cds": "Pour Sci.",
        "inspire": "Pour La Science"
      },
      {
        "cds": "Isr. J. Chem.",
        "inspire": "Israel J.Chem."
      },
      {
        "cds": "Messenger",
        "inspire": "ESO Messenger"
      },
      {
        "cds": "Phys. Educ.",
        "inspire": "Phys.Educ.India"
      },
      {
        "cds": "J. Appl. Math. Mech.",
        "inspire": "J.Math.Mech."
      },
      {
        "cds": "Br. J. Philos. Sci.",
        "inspire": "Brit.J.Phil.Sci."
      },
      {
        "cds": "Spectrum",
        "inspire": "Spectrum"
      },
      {
        "cds": "Nucl.Phys.",
        "inspire": "Nucl.Phys."
      },
      {
        "cds": "Chin.J.Math.",
        "inspire": "Chin.J.Math."
      },
      {
        "cds": "J. High Energy Phys.",
        "inspire": "JHEP"
      },
      {
        "cds": "Riv.Mat.Pure Appl.",
        "inspire": "Riv.Mat.Pure Appl."
      },
      {
        "cds": "Rocky Mt. J. Math.",
        "inspire": "Rocky Mt.J.Math."
      },
      {
        "cds": "Comment. Math. Helv.",
        "inspire": "Comment.Math.Helv."
      },
      {
        "cds": "Ann. Inst. Henri Poincaré, A",
        "inspire": "Ann.Inst.H.Poincare Anal.Non Lineaire"
      },
      {
        "cds": "Ann. Henri Poincaré, Phys. Théor.",
        "inspire": "Annales Poincare Phys.Theor."
      },
      {
        "cds": "Math.Studies",
        "inspire": "Math.Studies"
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Beitr.Geophys.",
        "inspire": "Beitr.Geophys."
      },
      {
        "cds": "Usp.Mat.Nauk",
        "inspire": "Usp.Mat.Nauk"
      },
      {
        "cds": "Probab. Theory Relat. Fields",
        "inspire": "Probab.Theor.Related Fields"
      },
      {
        "cds": "Solid State Technol.",
        "inspire": "Solid State Technol."
      },
      {
        "cds": "J. Phys. Chem.",
        "inspire": "J.Phys.Chem."
      },
      {
        "cds": "Air Space Eur.",
        "inspire": "Air Space Eur."
      },
      {
        "cds": "J. High Energy Phys.",
        "inspire": "JHEP"
      },
      {
        "cds": "Phys. Part. Nucl.",
        "inspire": "Phys.Part.Nucl."
      },
      {
        "cds": "Math. Biosci.",
        "inspire": "Math.Biosci."
      },
      {
        "cds": "Lect. Notes Phys.",
        "inspire": "Lect.Notes Phys."
      },
      {
        "cds": "Int. J. Thermophys.",
        "inspire": "Int.J.Thermophys."
      },
      {
        "cds": "Antarc.J.",
        "inspire": "Antarc.J."
      },
      {
        "cds": "J. Math. Phys.",
        "inspire": "J.Math.Phys."
      },
      {
        "cds": "Publ. Astron. Soc. Aust.",
        "inspire": "Publ.Astron.Soc.Austral."
      },
      {
        "cds": "Contrib.Plasma Phys.",
        "inspire": "Contrib.Plasma Phys."
      },
      {
        "cds": "Rev. Sci. Instrum.",
        "inspire": "Rev.Sci.Instrum."
      },
      {
        "cds": "J. Math. Pures Appl.",
        "inspire": "J.Math.Pure.Appl."
      },
      {
        "cds": "Mem. Am. Math. Soc.",
        "inspire": "Mem.Am.Math.Soc."
      },
      {
        "cds": "Atom",
        "inspire": "ATOM-PH"
      },
      {
        "cds": "Sci.News",
        "inspire": "Sci.News"
      },
      {
        "cds": "Fiz.Szemle",
        "inspire": "Fiz.Szemle"
      },
      {
        "cds": "J. Complexity",
        "inspire": "J.Complexity"
      },
      {
        "cds": "Geom. Dedicata",
        "inspire": "Geom.Dedicata"
      },
      {
        "cds": "Nature Struct. Biol.",
        "inspire": "Nature Struct.Biol."
      },
      {
        "cds": "IEEE Trans. Appl. Supercond.",
        "inspire": "IEEE Trans.Appl.Supercond."
      },
      {
        "cds": "Recherche",
        "inspire": "Recherche"
      },
      {
        "cds": "Signal Process.",
        "inspire": "Signal Processing"
      },
      {
        "cds": "Fyz. Cas.",
        "inspire": "Fyz. Cas."
      },
      {
        "cds": "Opt. Spectrosc.",
        "inspire": "Opt.Spectrosc."
      },
      {
        "cds": "Adv. Appl. Math.",
        "inspire": "Adv.Appl.Math."
      },
      {
        "cds": "Can.Math.Congr.Proc.",
        "inspire": "Can.Math.Congr.Proc."
      },
      {
        "cds": "Math. Sbornik",
        "inspire": "Mat.Sbornik"
      },
      {
        "cds": "Low Temp. Phys.",
        "inspire": "Low Temp.Phys."
      },
      {
        "cds": "Lett. Math. Phys.",
        "inspire": "Lett.Math.Phys."
      },
      {
        "cds": "J.Nonmath.Phys.",
        "inspire": "J.Nonmath.Phys."
      },
      {
        "cds": "Bull. Lebedev Phys. Inst.",
        "inspire": "Bull.Lebedev Phys.Inst."
      },
      {
        "cds": "J. Cryptol.",
        "inspire": "J.Cryptolog."
      },
      {
        "cds": "Comments Nucl.Part.Phys.",
        "inspire": "Comments Nucl.Part.Phys."
      },
      {
        "cds": "Zap. Nauchnykh Semin. POMI",
        "inspire": "Zap.Nauchn.Semin."
      },
      {
        "cds": "Astrophysics",
        "inspire": "Astrophysics"
      },
      {
        "cds": "Comput. Phys. Commun.",
        "inspire": "Comput.Phys.Commun."
      },
      {
        "cds": "Linear Algebra Appl.",
        "inspire": "Linear Algebra Appl."
      },
      {
        "cds": "Pontif.Acad.Sci.Scrivaria",
        "inspire": "Pontif.Acad.Sci.Scrivaria"
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "J. Geom. Phys.",
        "inspire": "J.Geom.Phys."
      },
      {
        "cds": "KronoScope",
        "inspire": "KronoScope"
      },
      {
        "cds": "Dokl.Akad.Nauk Ser.Fiz.",
        "inspire": "Dokl.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Astrophys.J.Suppl.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Physik. Z.",
        "inspire": "Phys.Z."
      },
      {
        "cds": "Philos. Trans. R. Soc.",
        "inspire": "Phil.Trans.Roy.Soc.Lond."
      },
      {
        "cds": "Abstracts Amer.Math.Soc.",
        "inspire": "Abstracts Amer.Math.Soc."
      },
      {
        "cds": "Nature Phys.",
        "inspire": "Nature Phys."
      },
      {
        "cds": "Turk.J.Math",
        "inspire": "Turk.J.Math"
      },
      {
        "cds": "J.Macromol.Sci.Chem.",
        "inspire": "J.Macromol.Sci.Chem."
      },
      {
        "cds": "Rev.Quest.Sci.",
        "inspire": "Rev.Quest.Sci."
      },
      {
        "cds": "IEEE Trans. Commun.",
        "inspire": "IEEE Trans.Commun."
      },
      {
        "cds": "Ann.Rev.Physiol.",
        "inspire": "Ann.Rev.Physiol."
      },
      {
        "cds": "Phys. Can.",
        "inspire": "Phys.Canada"
      },
      {
        "cds": "J. Geophys. Res.",
        "inspire": "J.Geophys.Res."
      },
      {
        "cds": "Q.J.Roy.Astron.Soc.",
        "inspire": "Q.J.Roy.Astron.Soc."
      },
      {
        "cds": "Surf. Sci. Rep.",
        "inspire": "Surf.Sci.Rept."
      },
      {
        "cds": "Nucl. Phys. B, Proc. Suppl.",
        "inspire": "Nucl.Phys.Proc.Suppl."
      },
      {
        "cds": "Ann. Phys.",
        "inspire": "Annals Phys."
      },
      {
        "cds": "Comput.Graphics",
        "inspire": "Comput.Graphics"
      },
      {
        "cds": "Astron. Lett.",
        "inspire": "Astron.Lett."
      },
      {
        "cds": "Sci. Comput. Program.",
        "inspire": "Sci.Comput.Program."
      },
      {
        "cds": "Math.Oper.Res.",
        "inspire": "Math.Oper.Res."
      },
      {
        "cds": "Acta Physica Acad.Sci.Hung.",
        "inspire": "Acta Phys.Hung."
      },
      {
        "cds": "Int.J.Comput.Math.",
        "inspire": "Int.J.Comput.Math."
      },
      {
        "cds": "Sov. Astron.",
        "inspire": "Sov.Astron."
      },
      {
        "cds": "IEEE Trans. Nucl. Sci.",
        "inspire": "IEEE Trans.Nucl.Sci."
      },
      {
        "cds": "Geom. Dedicata",
        "inspire": "Geom.Dedicata"
      },
      {
        "cds": "Phys. Rep.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "J. Mol. Biol.",
        "inspire": "J.Mol.Biol."
      },
      {
        "cds": "J. Stat. Mech.",
        "inspire": "J.Stat.Mech."
      },
      {
        "cds": "Astrophys. J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Nature",
        "inspire": "Nature"
      },
      {
        "cds": "Physicist",
        "inspire": "The Physicist"
      },
      {
        "cds": "Int. J. Math.",
        "inspire": "Int.J.Math."
      },
      {
        "cds": "IEEE J. Quantum Electron.",
        "inspire": "IEEE J.Quant.Electron."
      },
      {
        "cds": "Kobe J. Math.",
        "inspire": "Kobe J.Math."
      },
      {
        "cds": "J. Electrochem. Soc.",
        "inspire": "J.Electrochem.Soc."
      },
      {
        "cds": "Astrophys.J.Suppl.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "Int. J. Quantum Chem. Symp.",
        "inspire": "Int.J.Quant.Chem.Symp."
      },
      {
        "cds": "Adv. Appl. Clifford Algebr.",
        "inspire": "Adv.Appl.Clifford Algebras"
      },
      {
        "cds": "Astron. Soc. Pac. Conf. Proc.",
        "inspire": "ASP Conf.Ser."
      },
      {
        "cds": "Ann. Rev. Fluid. Mech.",
        "inspire": "Ann.Rev.Fluid Mech."
      },
      {
        "cds": "J. Phys. II",
        "inspire": "J.Phys.II"
      },
      {
        "cds": "Proc. Camb. Philos. Soc.",
        "inspire": "Proc.Cambridge Phil.Soc."
      },
      {
        "cds": "JINR Rapid Commun.",
        "inspire": "JINR Rapid Commun."
      },
      {
        "cds": "Pramana - J. Phys.",
        "inspire": "Pramana"
      },
      {
        "cds": "Stud. Hist. Philos. Mod. Phys.",
        "inspire": "Stud.Hist.Philos.Mod.Phys."
      },
      {
        "cds": "Turk. J. Math.",
        "inspire": "Turk.J.Math"
      },
      {
        "cds": "Space Sci. Rev.",
        "inspire": "Space Sci.Rev."
      },
      {
        "cds": "Astrophys.J.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Sci.Technol.",
        "inspire": "Sci.Technol."
      },
      {
        "cds": "Physik. Z.",
        "inspire": "Phys.Z."
      },
      {
        "cds": "Algebra Analiz. (Russia)",
        "inspire": "Alg.Anal."
      },
      {
        "cds": "Philos. Mag.",
        "inspire": "Phil.Mag."
      },
      {
        "cds": "Adv. Space Res.",
        "inspire": "Adv.Space Res."
      },
      {
        "cds": "Phys. World",
        "inspire": "Phys.World"
      },
      {
        "cds": "J. Res. Natl. Inst. Stand. Technol.",
        "inspire": "J.Res.Natl.Bur.Stand."
      },
      {
        "cds": "HEP Lib.Web.",
        "inspire": "HEP Lib.Web."
      },
      {
        "cds": "Appl.Math.Mech.",
        "inspire": "Appl.Math.Mech."
      },
      {
        "cds": "Nature",
        "inspire": "Nature Phys.Sci."
      },
      {
        "cds": "Astrophys. J. Suppl. Ser.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "J. Funct. Anal.",
        "inspire": "J.Funct.Anal."
      },
      {
        "cds": "J.Appl.Polymer Sci.",
        "inspire": "J.Appl.Polymer Sci."
      },
      {
        "cds": "New Sci.",
        "inspire": "New Sci."
      },
      {
        "cds": "Adv. Quantum Chem.",
        "inspire": "Adv.Quant.Chem."
      },
      {
        "cds": "Wireless World",
        "inspire": "Wireless World"
      },
      {
        "cds": "IEEE Trans. Appl. Supercond.",
        "inspire": "IEEE Trans.Appl.Supercond."
      },
      {
        "cds": "J.Phys.A",
        "inspire": "J.Phys.G"
      },
      {
        "cds": "Phys. Scr. Top. Issues",
        "inspire": "Phys.Scripta"
      },
      {
        "cds": "Bol.Soc.Bras.Mat.",
        "inspire": "Bol.Soc.Bras.Mat."
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Rev.Solid State Sci.",
        "inspire": "Rev.Solid State Sci."
      },
      {
        "cds": "Adv. Math.",
        "inspire": "Adv.Math."
      },
      {
        "cds": "Util.Math.",
        "inspire": "Util.Math."
      },
      {
        "cds": "Prog. Theor. Phys., Suppl.",
        "inspire": "Prog.Theor.Phys.Suppl."
      },
      {
        "cds": "Proc. R. Soc. Lond.",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "J. Phys. Chem. Ref. Data",
        "inspire": "J.Phys.Chem.Ref.Data"
      },
      {
        "cds": "APCTP Bull.",
        "inspire": "APCTP Bull."
      },
      {
        "cds": "Waves Random Media",
        "inspire": "Waves Random Media"
      },
      {
        "cds": "Radiology",
        "inspire": "Radiology"
      },
      {
        "cds": "Prog. Theor. Phys.",
        "inspire": "Prog.Theor.Phys."
      },
      {
        "cds": "Theor. Math. Phys.",
        "inspire": "Theor.Math.Phys."
      },
      {
        "cds": "Differ. Geom. Appl.",
        "inspire": "Differ.Geom.Appl."
      },
      {
        "cds": "Sov. Astron.",
        "inspire": "Sov.Astron."
      },
      {
        "cds": "New Astron. Rev.",
        "inspire": "New Astron.Rev."
      },
      {
        "cds": "Sol. Syst. Res.",
        "inspire": "Sol.Syst.Res."
      },
      {
        "cds": "IEEE Trans. Neural Netw.",
        "inspire": "IEEE Trans.Neural Networks"
      },
      {
        "cds": "Surv. High Energy Phys.",
        "inspire": "Surveys High Energ.Phys."
      },
      {
        "cds": "Combust.Sci.Tech.",
        "inspire": "Combust.Sci.Tech."
      },
      {
        "cds": "Q.J.R.Meteorol.Soc.",
        "inspire": "Q.J.R.Meteorol.Soc."
      },
      {
        "cds": "Zh.Prikl.Spektrosk.",
        "inspire": "Zh.Prikl.Spektrosk."
      },
      {
        "cds": "J. Phys. A",
        "inspire": "J.Phys.A"
      },
      {
        "cds": "J. Phys. C",
        "inspire": "J.Phys.C"
      },
      {
        "cds": "J. Phys. B",
        "inspire": "J.Phys.B"
      },
      {
        "cds": "J. Phys. E",
        "inspire": "J.Phys.E"
      },
      {
        "cds": "J. Phys. D",
        "inspire": "J.Phys.D"
      },
      {
        "cds": "J. Phys. G",
        "inspire": "J.Phys.G"
      },
      {
        "cds": "J. Phys. F",
        "inspire": "J.Phys.F"
      },
      {
        "cds": "IEEE Trans. Geosci. Remote Sens.",
        "inspire": "IEEE Trans.Geosci.Remote Sensing"
      },
      {
        "cds": "IEEE Trans. Comput.",
        "inspire": "IEEE Trans.Comput."
      },
      {
        "cds": "Bol. Soc. Mex. Fis.",
        "inspire": "Bol.Soc.Mex.Fis."
      },
      {
        "cds": "J. Lond. Math. Soc.",
        "inspire": "Bull.London Math.Soc."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Rend.Mat.Appl.",
        "inspire": "Rend.Mat.Appl."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "IEEE Trans. Nucl. Sci.",
        "inspire": "IEEE Trans.Nucl.Sci."
      },
      {
        "cds": "J. Phys. B",
        "inspire": "J.Phys.B.At.Mol.Opt.Phys."
      },
      {
        "cds": "J. R. Stat. Soc., B",
        "inspire": "J.Roy.Statist.Soc.B"
      },
      {
        "cds": "Int. Rev. Nucl. Phys.",
        "inspire": "Int.Rev.Nucl.Phys."
      },
      {
        "cds": "Eur. Phys. J.",
        "inspire": "Eur.Phys.J."
      },
      {
        "cds": "J. Algebra",
        "inspire": "J.Algebra"
      },
      {
        "cds": "Mem. Soc. Astron. Ital.",
        "inspire": "Mem.Soc.Ast.It."
      },
      {
        "cds": "J. Solid State Chem.",
        "inspire": "J.Solid State Chem."
      },
      {
        "cds": "Mater. Sci. Eng. C",
        "inspire": "Mater.Sci.Eng.C"
      },
      {
        "cds": "Mater. Sci. Eng. B",
        "inspire": "Mater.Sci.Eng.B"
      },
      {
        "cds": "Mater. Sci. Eng. A",
        "inspire": "Mater.Sci.Eng."
      },
      {
        "cds": "Meccanica",
        "inspire": "Meccanica"
      },
      {
        "cds": "Q. J. Math.",
        "inspire": "Quart.J.Math.Oxford Ser."
      },
      {
        "cds": "Opt. Quantum Electron.",
        "inspire": "Opt.Quant.Electron."
      },
      {
        "cds": "Semicond. Sci. Technol.",
        "inspire": "Semicond.Sci.Tech."
      },
      {
        "cds": "Adv. Math.",
        "inspire": "Adv.Math."
      },
      {
        "cds": "Ukr.Fiz.Zh.(Russ.Ed.)",
        "inspire": "Ukr.Fiz.Zh.(Russ.Ed.)"
      },
      {
        "cds": "Astrophys. J. Lett.",
        "inspire": "Astrophys.J."
      },
      {
        "cds": "Sigmod Record",
        "inspire": "Sigmod Record"
      },
      {
        "cds": "Acta Math.",
        "inspire": "Acta Math."
      },
      {
        "cds": "J. Magn. Reson.",
        "inspire": "J.Magn.Resonance"
      },
      {
        "cds": "Comment.Phys.Math.Soc.Sci.Fenn.",
        "inspire": "Comment.Phys.Math.Soc.Sci.Fenn."
      },
      {
        "cds": "Chem. Phys. Lett.",
        "inspire": "Chem.Phys.Lett."
      },
      {
        "cds": "Pis'ma Zh. Eksp. Teor. Fiz.",
        "inspire": "Pisma Zh.Eksp.Teor.Fiz."
      },
      {
        "cds": "J.Soc.Indust.Appl.Math.",
        "inspire": "J.Soc.Indust.Appl.Math."
      },
      {
        "cds": "J. Phys. I",
        "inspire": "J.Phys.I(France)"
      },
      {
        "cds": "Funkt.Anal.Pril.",
        "inspire": "Funkt.Anal.Pril."
      },
      {
        "cds": "Interdisc.Sci.Rev.",
        "inspire": "Interdisc.Sci.Rev."
      },
      {
        "cds": "J. ACM Assoc. Comput. Mach.",
        "inspire": "J.Assoc.Comput.Machinery"
      },
      {
        "cds": "Condens. Matter Phys.",
        "inspire": "Condensed Matter Phys."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Res. Inf.",
        "inspire": "Res.Inf."
      },
      {
        "cds": "J. Magn. Reson.",
        "inspire": "J.Magn.Resonance"
      },
      {
        "cds": "Compt.Rend.Acad.Sci.(Ser.II)",
        "inspire": "Compt.Rend.Acad.Sci.(Ser.II)"
      },
      {
        "cds": "Radiat. Eff.",
        "inspire": "Radiat.Eff.Def.Solids"
      },
      {
        "cds": "Lith.Phys.J.",
        "inspire": "Lith.Phys.J."
      },
      {
        "cds": "Abh. Math. Sem. Hamburg",
        "inspire": "Abh.Math.Sem.Univ.Hamburg"
      },
      {
        "cds": "Nature Struct. Biol.",
        "inspire": "Nature Struct.Biol."
      },
      {
        "cds": "J. Phys., Colloq.",
        "inspire": "J.Phys.Colloq."
      },
      {
        "cds": "J. Phys.",
        "inspire": "J.Phys.A"
      },
      {
        "cds": "Acta Cosmol.",
        "inspire": "Acta Cosmologica"
      },
      {
        "cds": "New Astron.",
        "inspire": "New Astron."
      },
      {
        "cds": "Lett. Math. Phys.",
        "inspire": "Lett.Math.Phys."
      },
      {
        "cds": "J. Electroanal. Chem.",
        "inspire": "J.Electroanal.Chem."
      },
      {
        "cds": "Superlattices Microstruct.",
        "inspire": "Superlattices Microstruct."
      },
      {
        "cds": "Izv.Akad.Nauk Ser.Fiz.",
        "inspire": "Izv.Akad.Nauk Ser.Fiz."
      },
      {
        "cds": "Aust. J. Phys.",
        "inspire": "Austral.J.Phys."
      },
      {
        "cds": "Phys. Lett. B",
        "inspire": "Phys.Lett."
      },
      {
        "cds": "J. Phys. Soc. Jpn.",
        "inspire": "J.Phys.Soc.Jap."
      },
      {
        "cds": "Proc. R. Soc. Lond.",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "Alg.Groups Geom.",
        "inspire": "Alg.Groups Geom."
      },
      {
        "cds": "Commun. Theor. Phys.",
        "inspire": "Commun.Theor.Phys."
      },
      {
        "cds": "Jpn. J. Appl. Phys.",
        "inspire": "Jap.J.Appl.Phys."
      },
      {
        "cds": "Phys.Rept.",
        "inspire": "Phys.Rept."
      },
      {
        "cds": "Vision Res.",
        "inspire": "Vision Res."
      },
      {
        "cds": "J. Comput. Appl. Math.",
        "inspire": "J.Comput.Appl.Math."
      },
      {
        "cds": "Turk. J. Phys.",
        "inspire": "Turk.J.Phys."
      },
      {
        "cds": "Astrophys.Lett.Commun.",
        "inspire": "Astrophys.Lett.Commun."
      },
      {
        "cds": "Phys. Rev. E",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "Phys. Rev. D",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "Phys. Rev. A",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "Phys. Rev. B",
        "inspire": "Phys.Rev."
      },
      {
        "cds": "C. R. Acad. Bulg. Sci.",
        "inspire": "Compt.Rend.Acad.Sci.Bulg."
      },
      {
        "cds": "J. Korean Phys. Soc.",
        "inspire": "J.Korean Phys.Soc."
      },
      {
        "cds": "Compt.Rend.Acad.Sci.Bulg.",
        "inspire": "C.R.Acad.Sci."
      },
      {
        "cds": "Astron. Soc. Pac. Conf. Proc.",
        "inspire": "Publ.Astron.Soc.Pac."
      },
      {
        "cds": "Astron. Astrophys.",
        "inspire": "Astron.Astrophys."
      },
      {
        "cds": "Complex Syst.",
        "inspire": "Complex Syst."
      },
      {
        "cds": "Ann. Phys. (Leipzig)",
        "inspire": "Annalen Phys."
      },
      {
        "cds": "Anal. Chem.",
        "inspire": "Anal.Chem."
      },
      {
        "cds": "Mem. R. Astron. Soc.",
        "inspire": "Mem.Roy.Astron.Soc."
      },
      {
        "cds": "Nuovo Cimento",
        "inspire": "Nuovo Cim."
      },
      {
        "cds": "Discuss.Faraday Soc.",
        "inspire": "Discuss.Faraday Soc."
      },
      {
        "cds": "Wirel. Netw.",
        "inspire": "Wirel.Netw."
      },
      {
        "cds": "Eur. Phys. J. C",
        "inspire": "Eur.Phys.J."
      },
      {
        "cds": "Q.Rev.Chem.Soc.",
        "inspire": "Q.Rev.Chem.Soc."
      },
      {
        "cds": "Rend. Lincei",
        "inspire": "Rend. Acc. Naz. Lincei"
      },
      {
        "cds": "Anest.Analg.",
        "inspire": "Anest.Analg."
      },
      {
        "cds": "IEEE Trans. Electron Devices",
        "inspire": "IEEE Trans.Electron.Dev."
      },
      {
        "cds": "Sov. Phys. - Dokl.",
        "inspire": "Sov.Phys.Dokl."
      },
      {
        "cds": "Pac. J. Math.",
        "inspire": "Pacific J.Math."
      },
      {
        "cds": "Geophys.Prospect.",
        "inspire": "Geophys.Prospect."
      },
      {
        "cds": "Few-Body Syst.",
        "inspire": "Few Body Syst."
      },
      {
        "cds": "Daedalus",
        "inspire": "Daedalus"
      },
      {
        "cds": "Russ. Math. Surv.",
        "inspire": "Russ.Math.Surveys"
      },
      {
        "cds": "J. Phys.: Conf. Ser.",
        "inspire": "J.Phys.Conf.Ser."
      },
      {
        "cds": "IEEE Trans. Inf. Theory",
        "inspire": "IEEE Trans.Info.Theor."
      },
      {
        "cds": "Appl. Surf. Sci.",
        "inspire": "Appl.Surf.Sci."
      },
      {
        "cds": "Sov.Phys.JETP",
        "inspire": "Sov.Phys.JETP"
      },
      {
        "cds": "Energ.Nucl.(Milan)",
        "inspire": "Energ.Nucl.(Milan)"
      },
      {
        "cds": "Gen. Relativ. Gravit.",
        "inspire": "Gen.Rel.Grav."
      },
      {
        "cds": "Biochem.",
        "inspire": "Biochem."
      },
      {
        "cds": "At. Data Nucl. Data Tables",
        "inspire": "Nucl.Data Tables"
      },
      {
        "cds": "Proc. Natl. Acad. Sci. U. S. A.",
        "inspire": "Proc.Nat.Acad.Sci."
      },
      {
        "cds": "Proc. R. Soc. Lond., A",
        "inspire": "Proc.Roy.Soc.Lond."
      },
      {
        "cds": "J.Math.Soc.Jap.",
        "inspire": "J.Math.Soc.Jap."
      },
      {
        "cds": "Int. J. Neural Syst.",
        "inspire": "Int.J.Neural Syst."
      },
      {
        "cds": "Science",
        "inspire": "Science"
      },
      {
        "cds": "J. Phys.",
        "inspire": "J.Phys.(France)"
      },
      {
        "cds": "Phys. Rev. Lett.",
        "inspire": "Phys.Rev.Lett."
      },
      {
        "cds": "J.Integ.Eq.",
        "inspire": "J.Integ.Eq."
      },
      {
        "cds": "Ark. Fys.",
        "inspire": "Ark.Fys."
      },
      {
        "cds": "Phys. Perspect.",
        "inspire": "Phys.Perspect."
      },
      {
        "cds": "Bull.Geodesique",
        "inspire": "Bull.Geodesique"
      },
      {
        "cds": "Helv.Phys.Acta Suppl.",
        "inspire": "Helv.Phys.Acta Suppl."
      },
      {
        "cds": "J. Russ. Laser Res.",
        "inspire": "J.Russ.Laser Res."
      },
      {
        "cds": "High Energy Phys. Libr. Webzine",
        "inspire": "HEP Lib.Web."
      },
      {
        "cds": "Riv.Nuovo Cim.",
        "inspire": "Riv.Nuovo Cim."
      },
      {
        "cds": "Opt. Commun.",
        "inspire": "Opt.Commun."
      },
      {
        "cds": "Comments Mod. Phys.",
        "inspire": "Comments Mod.Phys."
      },
      {
        "cds": "Wear",
        "inspire": "Wear"
      },
      {
        "cds": "Math.Rev.",
        "inspire": "Math.Rev."
      },
      {
        "cds": "Astrophys.J.Suppl.",
        "inspire": "Astrophys.J.Suppl."
      },
      {
        "cds": "J. Fourier Anal. Appl.",
        "inspire": "J.Fourier Anal.Appl."
      },
      {
        "cds": "Usp.Fiz.Nauk",
        "inspire": "Usp.Fiz.Nauk"
      },
      {
        "cds": "Integral Equ. Oper. Theor.",
        "inspire": "Integral Equ. Oper. Theor."
      },
      {
        "cds": "Theor. Comput. Sci.",
        "inspire": "Theor.Comput.Sci."
      },
      {
        "cds": "Proc.Lebedev Phys.Inst.",
        "inspire": "Proc.Lebedev Phys.Inst."
      },
      {
        "cds": "Geol.Soc.Am.Bull.",
        "inspire": "Geol.Soc.Am.Bull."
      },
      {
        "cds": "Geom. Topol.",
        "inspire": "Geom.Topol."
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Class. Quantum Gravity",
        "inspire": "Class.Quant.Grav."
      },
      {
        "cds": "Inf. Sci.",
        "inspire": "Info.Sci."
      },
      {
        "cds": "Differ. Uravneniya",
        "inspire": "Diff.Urav."
      },
      {
        "cds": "Commun. Math. Phys.",
        "inspire": "Commun.Math.Phys."
      },
      {
        "cds": "Appl. Math. Comput.",
        "inspire": "Appl.Math.Comput."
      },
      {
        "cds": "Numer. Math.",
        "inspire": "Numer.Math."
      },
      {
        "cds": "Commun.Contemp.Math.",
        "inspire": "Commun.Contemp.Math."
      },
      {
        "cds": "Eur. Phys. J.",
        "inspire": "Eur.Phys.J."
      },
      {
        "cds": "Problemy Mat.Fiz.",
        "inspire": "Problemy Mat.Fiz."
      }
    ]
  }
}
